
c=============================================================================
c
c $Id: convec_ncar.F,v 1.29 2003/07/01 18:45:04 jrt Exp $
c
c CODE DEVELOPER
c   Original code from Phil Rasch, NCAR.
c   LLNL modifications:  Dan Bergmann
c                          dbergmann@llnl.gov
c                        John Tannahill
c                          jrt@llnl.gov
c                        Al Franz
c                          franz2@llnl.gov
c
c FILE
c   convec_ncar.F
c
c ROUTINES
c   Do_Convec_Ncar
c   Conv_Tran
c
c=============================================================================
 
 
c-----------------------------------------------------------------------------
c
c ROUTINE
c   Do_Convec_Ncar
c
c DESCRIPTION
c   This is the interface routine to Conv_Tran.  It formats the gem variables
c   to satisfy Conv_Tran.
c
c ARGUMENTS
c   metdata_name_org   : first  part of metdata_name, e.g., "NCAR"
c   metdata_name_model : second part of metdata_name, e.g., "MATCH"
c   do_wetdep    : flag for doing wet deposition
c   pr_wet_depos : flag for saving and then printing accumulated wet
c   pr_wet_depos_3type : flag for saving and then printing accumulated wet deposition(updraft, convetion, stratus)
c   chem_opt     : chemistry option
c   ih2o2_num    : const array index for H2O2
c   ihno3_num    : const array index for HNO3
c   lwi_flags    : land water ice flags (1-water 2-land 3-ice)
c   tdt          : model time step  (s)
c   mw           : array of species' molecular weights (g/mol)
c   mcor         : horizontal area of each grid box    (m^2)
c   pbl          : planetary boundary layer thickness  (m)
c   cldmas       : convective mass flux in     updraft (kg/m^2*s)
c   dtrn         : detrainment rate (DAO:kg/m^2*s, NCAR:s^-1)
c   eu           : entrainment into convective updraft (s^-1)
c   grid_height  : grid box height  (m) 
c   mass         : mass of air in each grid box (kg)
c   wet_depos    : accumulated wet deposition   (kg/m^2/step)
c   kel          : temperature      (degK)
c   press3e      : atmospheric pressure at the edge of each grid box (mb)
c   const        : species concentration, known at zone centers (mixing ratio)
c   humidity     : specific humidity (g/kg)
c   press3c      : atmospheric pressure at the center of each grid box (mb)
c   REL_SCAV_EFF_new
c                : relative scavenging efficiency for aerosols (0-1)
c
c-----------------------------------------------------------------------------
 
      subroutine Do_Convec_Ncar
!     &  (metdata_name_org, metdata_name_model, do_wetdep,
!     &   pr_wet_depos,pr_wet_depos_3type, chem_opt, ih2o2_num, ihno3_num, lwi_flags, tdt, 
!     &   mw, mcor, pbl, cldmas, dtrn, eu, grid_height, mass, wet_depos,wet_depos_1,
!     &   kel, press3e, const, humidity, press3c, REL_SCAV_EFF_new)
     & ( ih2o2_num, ihno3_num, lwi_flags, tdt, 
     &   mw, mcor, pbl, cldmas, grid_height, mass, wet_depos,
     &   kel, press3e, const, humidity, press3c, REL_SCAV_EFF_new,dust_so4_layer)
     
      use h_imp_phys_constants
      use h_imp_time_constants
      use h_imp_dims
      use h_imp_aerosol

      use h_setkin_par
      use h_setkin_depos
      use h_umaerosol
      
      implicit none

!#     include "imp_phys_constants.h"
!#     include "imp_time_constants.h"
!#     include "imp_dims.h"
!#     include "imp_aerosol.h"
!cc?   Tight coupling to setkin?
!#     include "setkin_par.h"
!#     include "setkin_depos.h"
!#     include "umaerosol.h"
 
 
c     ----------------------
c     Argument declarations.
c     ----------------------

!      character (len=16) :: metdata_name_org
!      character (len=16) :: metdata_name_model
!      logical :: pr_wet_depos
      integer,intent(in) :: ih2o2_num
      integer,intent(in) :: ihno3_num
      integer,intent(in) :: lwi_flags(i1:i2, ju1:j2)
      real*8,intent(in)  :: tdt
      real*8,intent(in)  :: mw(num_species)
      real*8,intent(in)  :: mcor       (i1:i2,   ju1:j2)
      real*8,intent(in)  :: pbl        (i1:i2,   ju1:j2)
      real*8,intent(in)  :: cldmas     (i1:i2,   ju1:j2,   k1:k2)
      real*8,intent(in)  :: grid_height(i1:i2,   ju1:j2,   k1:k2)
      real*8,intent(in)  :: mass       (i1:i2,   ju1:j2,   k1:k2)
      real*8,intent(in)  :: kel        (ilo:ihi, julo:jhi, k1:k2)
      real*8,intent(in)  :: press3e    (ilo:ihi, julo:jhi, k1-1:k2)
      real*8,intent(in)  :: humidity   (i1:i2,   ju1:j2,   k1:k2)
      real*8,intent(in)  :: press3c    (ilo:ihi, julo:jhi, k1:k2)
      
      real*8,intent(out) :: REL_SCAV_EFF_new(i1:i2, ju1:j2, k1:k2, num_species),dust_so4_layer(i1:i2, ju1:j2, k1:k2, 4)
      real*8,intent(inout)  :: const      (i1:i2,   ju1:j2,   k1:k2, num_species)
      real*8,intent(out) :: wet_depos  (i1:i2,   ju1:j2,   num_species)     
      
      real*8  :: dtrn       (i1:i2,   ju1:j2,   k1:k2)
!     real*8  :: eu         (i1:i2,   ju1:j2,   k1:k2) 
      
      ! now local
      logical :: do_wetdep = .true.
      integer :: chem_opt
c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter ::
     &  MBSTH = 1.0d-15     ! threshold below which we treat mass fluxes
                            ! as zero (mb/s)


c     ----------------------
c     Variable declarations.
c     ----------------------

      character (len=75) :: err_msg

      integer :: iku
      integer :: il, ij, ik, ic
      integer :: il2g                      ! gathered index to operate over
      integer :: inum
      integer :: itdt_conv
      integer :: num_conv_steps

      real*8  :: rnum_conv_steps
      real*8  :: tdt_conv

      real*8  :: xmbsth

      integer :: ideep(ilong)              ! gathering array
      integer :: pbli (ilong)              ! index of pbl height

      real*8  :: col_sum_depos   (i1:i2)   ! sum of column wet_deposition loss
      real*8  :: kloss           (i1:i2)   ! wet loss rate constant at all
                                           ! longitudes (s^-1)
      real*8  :: updraft_velocity(i1:i2)   ! velocity in convective updraft
                                           ! (m/s)

      real*8  :: dpi(ilong, ivert)         ! delta pressure between interfaces, unit:Pa
      real*8  :: eui(ilong, ivert)         ! mass entraining from updraft, unit: Pa/s
      real*8  :: mui(ilong, ivert)         ! mass flux up, unit: Pa/s

      real*8  ::
     &  fracis(i1:i2, k1:k2, num_species)  ! insoluble fraction of tracer
      real*8  ::
     &  qq    (i1:i2, k1:k2, num_species)  ! tracer array including moisture

      real*8  :: mcor3(i1:i2, ju1:j2, k1:k2)

c     --------------------
c     adding for umaerosol
c     --------------------

      real*8  :: relhume   (i1:i2, ju1:j2, k1:k2)
      real*8  :: h2osat    (i1:i2, ju1:j2, k1:k2)
      real*8  :: h2ogas    (i1:i2, ju1:j2, k1:k2)
      real*8  :: so4mfrac  (i1:i2, ju1:j2, k1:k2)
      real*8  :: so4dens   (i1:i2, ju1:j2, k1:k2)
      real*8  :: wetmas    (i1:i2, ju1:j2, k1:k2)
      real*8  :: so4aer    (i1:i2, ju1:j2, k1:k2, naer)
      real*8  :: so4radv   (i1:i2, ju1:j2, k1:k2, nso4)
#ifndef NONON
      real*8  :: so4non    (i1:i2, ju1:j2, k1:k2, nnon)
      real*8  :: aernon    (i1:i2, ju1:j2, k1:k2, nnon)
      real*8  :: xnonum    (i1:i2, ju1:j2, k1:k2, nnon)
      real*8  :: xso4non   (i1:i2, ju1:j2, k1:k2, nnon)
      real*8  :: fso4non   (i1:i2, ju1:j2, k1:k2, nnon)
      real*8  :: rso4min   (i1:i2, ju1:j2, k1:k2)
#endif
      real*8  :: xcldnum   (i1:i2, ju1:j2, k1:k2)
      real*8  :: so4cldcoag(i1:i2, ju1:j2, k1:k2, nso4-1)

c     varialbes for scavenging efficiency, added by Cheng Zhou, following minghuai
      real*8  :: tmax        ! max temperature for have ice water
      real*8  :: tmin        ! min temperature for have liquid water
      real*8  :: fice        ! fraction of ice water in total water

c     ----------------------
c     Function declarations.
c     ----------------------

      logical :: Is_Fixed_Const


c     ----------------
c     Begin execution.
c     ----------------
			dtrn (:,:,:) = 0.d0 ! set to 0 now
!			eu   (:,:,:) = 0.d0

c Add tmax =263, tmin =233 by Cheng Zhou for the scavenge efficiency
      tmax = 273.0d0-10.0d0
      tmin = tmax - 30.0d0

      ideep(:) = 0
      pbli (:) = 0

      kloss(:) = 0.0d0

      updraft_velocity(:) = 0.0d0

      dpi(:,:) = 0.0d0
      eui(:,:) = 0.0d0
      mui(:,:) = 0.0d0

      fracis(:,:,:) = 0.0d0


      xmbsth = MBSTH


ccc!? For now just hardwire a few values for the radon/lead problem
cc    (chem_opt = 1).  Setkin will eventually provide this information
cc    for all species when doing a full chemistry calculation.
      chem_opt = 8
      if (chem_opt == 1) then

        aerosol(1) = 0
        aerosol(2) = 1
  
        hstar(1)   = 9.3d-3
        hstar(2)   = 0.0d0
  
        delh_298_over_r(1) = 2600.0d0
        delh_298_over_r(2) =    0.0d0

        retention_eff  (1) =    0.0d0
        retention_eff  (2) =    0.0d0

      else if (chem_opt == 6) then

        aerosol(1) = 15
        aerosol(2) = 15

        hstar(1)   = 0.0d0
        hstar(2)   = 0.0d0

        delh_298_over_r(1) =    0.0d0
        delh_298_over_r(2) =    0.0d0

        retention_eff  (1) =    0.0d0
        retention_eff  (2) =    0.0d0

      else if (chem_opt == 8) then

        inum = IFSO2
        hstar(inum) = 600.0d0

        inum = INSO2
        hstar(inum) = 600.0d0

      end if

c     -----------------------------------------------------------
c     Calculate any needed sub-cycling of the convection operator
c     by comparing the mass flux over the full time step to the
c     mass within the grid box. 
c     -----------------------------------------------------------

ccc   num_conv_steps = 
ccc  &  Maxval (tdt * cldmas(:,:,:) *
ccc  &          Spread (mcor(:,:), 3, ivert) /  mass(:,:,:)) +
ccc  &         1.0d0
      mcor3(:,:,:)   = Spread (mcor(:,:), 3, ivert)
      num_conv_steps =
     &  Maxval (tdt * cldmas(:,:,:) * mcor3(:,:,:) / mass(:,:,:)) +
     &  1.0d0

      rnum_conv_steps = num_conv_steps
      tdt_conv        = tdt / rnum_conv_steps


cc    --------------------
cc    adding for umaerosol
cc    --------------------

cc    compute relative humidity (0-1)

      relhume(:,:,:) = 1.0d0 - (373.15d0 / kel(i1:i2,ju1:j2,k1:k2))
      relhume(:,:,:) =
     &  1013.25d0 * Exp (13.3185d0 * relhume(:,:,:)    -
     &                    1.9760d0 * relhume(:,:,:)**2 -
     &                    0.6445d0 * relhume(:,:,:)**3 -
     &                    0.1299d0 * relhume(:,:,:)**4)

      relhume(:,:,:) =
     &  humidity(:,:,:) * MWTAIR / 18.0d0 /
     &  GPKG * press3c(i1:i2,ju1:j2,k1:k2) / relhume(:,:,:)

      relhume(:,:,:) = Max (Min (relhume(:,:,:), 0.95d0), 0.0d0)

cc    compute so4 aerosol size (m)

      so4aer(:,:,:,1) = const(:,:,:,ISO4M1)
      so4aer(:,:,:,2) = const(:,:,:,ISO4N1)
      so4aer(:,:,:,3) = const(:,:,:,ISO4M2)
      so4aer(:,:,:,4) = const(:,:,:,ISO4N2)
      so4aer(:,:,:,5) = const(:,:,:,ISO4M3)
      so4aer(:,:,:,6) = const(:,:,:,ISO4N3)

      do il = i1, i2
        do ij = ju1, j2
          do ik = k1, k2
            h2osat(il,ij,ik) = h2osat_f(kel(il,ij,ik))
            h2ogas(il,ij,ik) = relhume(il,ij,ik)*h2osat(il,ij,ik)

            so4mfrac(il,ij,ik) = so4mfrac_f(kel(il,ij,ik),
     &                                      h2osat(il,ij,ik),
     &                                      h2ogas(il,ij,ik))
            so4dens(il,ij,ik) = so4dens_f(kel(il,ij,ik),
     &                                    so4mfrac(il,ij,ik))
          end do
        end do
      end do

      do iso4 = 1, nso4

        iso4n = iso4 * nmomso4
        iso4m = iso4n - 1

        wetmas(:,:,:) = max(r2so4min, so4aer(:,:,:,iso4m)
     &                / max(epsilo,   so4aer(:,:,:,iso4n)))
     &                / so4mfrac(:,:,:)

        so4radv(:,:,:,iso4) = (r3q * wetmas(:,:,:)
     &                      / (IMP_PI * so4dens(:,:,:)))**r1td
        so4radv(:,:,:,iso4) = max(so4radvmin,
     &                        min(so4radv(:,:,:,iso4),so4radvmax))

      end do

#ifndef NONON
      so4non(:,:,:,1:nnon) = const(:,:,:,ISO4NOC:ISO4S4)
      aernon(:,:,:,1:nnon) = const(:,:,:,INOC:ISSLT4)

cc    non-so4 aerosol radius (m) and mass (kg/particle)
      do inon = 1, nnon

        radvolm(:) = radgnon(:,inon)
     &             * exp(r3h*log(siggnon(:,inon))**2)
        radvnon(inon) = sum(fracnon(:,inon)*radvolm(:)**3)
        pmsnon(inon) = r4td*IMP_PI*rhonon(inon)
     &               * radvnon(inon)
        radvnon(inon) = radvnon(inon)**r1td

      end do

cc    non-so4 aerosol number concentration (#particles/kg air)
      do inon = 1, nnon
        xnonum(:,:,:,inon) = aernon(:,:,:,inon) / pmsnon(inon)
      end do

cc    so4 molecule # on non-so4 aerosol surface (#molec/particle)
      xso4non(:,:,:,1:nnon) = max(1.0d-30,so4non(:,:,:,1:nnon)) /
     &  max(r1,xnonum(:,:,:,1:nnon)) / so4min

cc    radius of each so4 molecule (m)
      rso4min(:,:,:) =
     &  (so4min * r3q /
     &  (IMP_PI*so4mfrac(:,:,:)*so4dens(:,:,:)))**r1td

cc    area fraction of so4 molecules on non-so4 aerosol surface
      do inon = 1, nnon

        fso4non(:,:,:,inon) =
     &    rso4min(:,:,:)**2.0d0 * xso4non(:,:,:,inon)
     &    / (4.0d0*(radvnon(inon)+rso4min(:,:,:))**2.0d0)

      end do
#endif

cc    compute cloud droplet number (cm^-3) to be the sum of 
cc    accumulation mode so4 number and non-so4 aerosol

      xcldnum(:,:,:) = so4aer(:,:,:,naer)

#ifndef NONON
      do inon = 1, nnon

        if (inon <= 10) then                  ! 5 oc/bc, aBC/POM, dust 1-4
          xcldnum(:,:,:) = xcldnum(:,:,:) + xnonum(:,:,:,inon)
     &                   * min(fso4non(:,:,:,inon)/fso4crit, r1)
        else                                 ! ss 1-4
          xcldnum(:,:,:) = xcldnum(:,:,:) + xnonum(:,:,:,inon)
        end if

      end do
#endif

cc    change unit from kg^-1 to cm^-3
      xcldnum(:,:,:) = min(r3000,max(r10,xcldnum(:,:,:) *
     &  (press3c(i1:i2,ju1:j2,k1:k2) * MWTAIR * MB2CGS) /
     &  (GPKG * kel(i1:i2,ju1:j2,k1:k2) * BOLTZMN_E * AVOGAD)))

cc    compute Brownian coagulation coefficient K12 (cm3 s-1)
cc    for nucleation and Aitken so4 modes with cloud droplets
cc    fit from Table 12.3 and Fig.12.5, in Seinfeld and Pandis (1997)
cc    as log10(K12) = -1.8436 * log10(D(um)) - 9.21

      do iso4 = 1, nso4-1

        so4radv(:,:,:,iso4) = min(so4radv(:,:,:,iso4),1.0d-7)
        so4cldcoag(:,:,:,iso4) =
     &    10 ** (a11 * log10(so4radv(:,:,:,iso4)*2.0d+6) + a12)   

      end do

c     ==============================
      ICLOOP: do ic = 1, num_species
c     ==============================

cc      calculate scavenging efficiency of aerosols

        if (Is_Fixed_Const(ic) .or.
     &     (aerosol(ic) <= 0)  .or.
     &     (aerosol(ic) >  NUM_AEROSOL)) then
c         ============
          cycle ICLOOP
c         ============
        end if

cc      SO4 nucleation and Aitken modes (assume cloud lifetime 4 hr)
        if (aerosol(ic) == 1 .or. aerosol(ic) == 2) then

          REL_SCAV_EFF_new(:,:,:,ic) = min(r1,
     &      so4cldcoag(:,:,:,aerosol(ic)) * 
     &      4.0d0 * SECPHR * xcldnum(:,:,:))

        else
#ifdef NONON
          REL_SCAV_EFF_new(:,:,:,ic) = REL_SCAV_EFF(aerosol(ic))
#else

!          inon = aerosol(ic) - nso4 ! aerosol() has 19 types defined in h_setkin_depos,nnon=21 
!       if (inon>0 .and. inon<=8) then         ! 5 oc/bc,aPOM/BC, dust1-2. Old version before 4/27/2017. inon is from 1-16 for all non-sulfate. see h_imp_aerosol.F . dust1 in aerosol=11, -3 is 8


       ! set default values from prescribed array. acutally only SO4m3/n3, dust3-4 are needed
       REL_SCAV_EFF_new(:,:,:,ic) = REL_SCAV_EFF(aerosol(ic))
       
       inon = 0 ! default inon, do nothing. But if inon is from 1-15, or from nOC to dust2, modify  REL_SCAV_EFF_new based on the coating #
 
       !!!!!!!!!!!!!!!!12                     32   !!!! so inon is from 1 to 21
       if ( ic .ge. ISO4NOC .and.  ic .le. ISO4S4 ) then 
           inon = ic - ISO4NOC + 1
       end if

       !!!!!!!!!!!!!!!!33                     53
       if ( ic .ge. INOC .and.  ic .le. ISSLT4 ) then 
           inon = ic - INOC + 1
       end if
       
       !! modify these types only, inon 1-15
       if (inon .gt. 0 .and. inon .le. 15) then         ! 5 oc/bc,4 phil1/2,cpom/cbc,aPOM/BC,  dust1-2. 
           REL_SCAV_EFF_new(:,:,:,ic) =
     &      REL_SCAV_EFF_MAX(aerosol(ic))*min(fso4non(:,:,:,inon)/fso4crit, r1)
     &      + REL_SCAV_EFF_MIN(aerosol(ic))*(1 -
     &                min(fso4non(:,:,:,inon)/fso4crit, r1))

       end if

#endif

       end if !! not Nuc. Aitken mode

       
      !!! add output of sulfate coating layer on dust   
       dust_so4_layer(:, :, :, 1:4) = fso4non(:,:,k2:k1:-1,14:17)! also reverse in Z from bottom up to top down.
       
c  Further change the scavenge efficiency by Cheng Zhou following minghuai. Not sure about the index, 17
c  In the code from Yuxinig, it is 18 because there is an extra paritcal aBC (9th). I do not have it. 
c  It seem stratopheric sulfate should not be affected by this.  
        if(aerosol(ic).le.17.and.aerosol(ic).ge.1) then
          do il = i1, i2
           do ij = ju1, j2
            do ik = k1, k2
              if(kel(il,ij,ik).gt.tmax) then
                 fice = 0.0d0
              elseif (kel(il,ij,ik).lt.tmin) then
                 fice = 1.0d0
              else
                 fice =(kel(il,ij,ik)-tmax)/(tmin-tmax)
              endif
              REL_SCAV_EFF_new(il,ij,ik,ic) =
     &               fice * REL_SCAV_EFF_ICE(aerosol(ic)) +
     &               (1.0d0-fice) * REL_SCAV_EFF_new(il,ij,ik,ic)
            end do
           end do
          end do
        endif


c     =============
      end do ICLOOP
c     =============


c     =======
      ijloop: do ij = ju1, j2
c     =======

        il2g = 0

c       =======
        illoop: do il = i1, i2
c       =======

c         ----------------------------------------------------
c         Verify that there is convection in the current cell.
c         ----------------------------------------------------

          if (Maxval (cldmas(il,ij,:)) >= 0.0d0) then

            il2g = il2g + 1

            ideep(il2g) = il

            dpi(il2g,:) =
     &        (press3e(il,ij,k2-1:k1-1:-1) -
     &         press3e(il,ij,k2:k1:-1)) *
     &        PASPMB       ! to Pa

              mui(il2g,:) = cldmas(il,ij,k2:k1:-1) * IMP_G ! from kg/m2/s to Pa/s

              eui(il2g,k1+1:k2-1) =
     &          Max (0.0d0,  
     &               cldmas(il,ij,k2-1:k1+1:-1) -
     &               cldmas(il,ij,k2-2:k1  :-1) +
     &               dtrn  (il,ij,k2-1:k1+1:-1))

              eui(il2g,:) = eui(il2g,:) * IMP_G ! to Pa/s


          end if

c       =============
        end do illoop
c       =============


c       =======
        il2gif: if (il2g /= 0) then
c       =======

          if (do_wetdep) then

c           ----------------------------------------------------------------
c           Calculate the insoluble fraction at each level for each species.
c           ----------------------------------------------------------------

            where ((lwi_flags(i1:i2,ij) == 1) .or. 
     &             (lwi_flags(i1:i2,ij) == 3)) 
              updraft_velocity(:) =  5.0d0 
            elsewhere 
              updraft_velocity(:) = 10.0d0
            end where

            do ik = k1, k2

              iku = k2 - ik + 1

              do ic = 1, num_species

c               if (ic == ihno3_num) then
                if (ic == ihno3_num .or. ic == ISO4G) then

                  kloss(:) = 5.0d-3

                else if (aerosol(ic) >= 1) then

c                 kloss(:) = 5.0d-3 * REL_SCAV_EFF(aerosol(ic))
                  kloss(:) = 5.0d-3
     &                     * REL_SCAV_EFF_new(:,ij,ik,ic)

                else if (hstar(ic) > 0.0d0) then

c                 =======================
                  call Calc_Wet_Loss_Rate
c                 =======================
     &              (ic, ih2o2_num, delh_298_over_r(ic), hstar(ic),
     &               retention_eff(ic), press3e(:,ij,ik), kel(:,ij,ik),
     &               kloss(:))

                  kloss(:) = 5.0d-3 * kloss(:)

                else

                  kloss(:) = 0.0d0

                end if

                fracis(:,iku,ic) =
     &            Exp (-kloss(:) * grid_height(:,ij,ik) / 
     &                 updraft_velocity(:))

              end do

            end do

          else

            fracis(:,:,:) = 1.0d0

          end if


c         ----------------------------------------------------------------
c         Find the index of the top of the planetary boundary layer (pbl).
c         Convection will assume well mixed tracers below that level.
c         ----------------------------------------------------------------

          do il = 1, il2g
          
            pbli(il) = 0

            ikloop: do ik = k1, k2

              if (pbl(ideep(il),ij) < 
     &            Sum (grid_height(ideep(il),ij,k1:ik))) then
                pbli(il) = ik
                exit ikloop
              end if

            end do ikloop
  
            if (pbli(il) == 0) then

              err_msg = 'Could not find pbl in Do_Convec_Ncar.'
!              call Do_Err_Out (err_msg, .true., 2, ideep(il), ij,
!     &                         1, pbl(ideep(il),ij), 0.0d0)
              stop 'Could not find pbl in Do_Convec_Ncar.'
            end if

            pbli(il) = k2 - pbli(il)

          end do


c         ==========
          itdtcloop: do itdt_conv = 1, num_conv_steps
c         ==========

            qq(:,k2:k1:-1,1:num_species) =
     &        const(:,ij,k1:k2,1:num_species)

c           ==============
            call Conv_Tran
c           ==============
     &        (il2g, tdt_conv, xmbsth, ideep, pbli, eui,
     &         mui, dpi, fracis, qq)


!            if (pr_wet_depos) then

c             ---------------------------------------------------
c             Calculate the wet deposition from the difference in
c             mixing ratio.
c             ---------------------------------------------------
              do ic = 1, num_species
                if ((hstar(ic) > 0.0d0) .or. 
     &              (aerosol(ic) >= 1)  .or. 
     &              (ic == ihno3_num)) then
                  col_sum_depos(i1:i2) = 0.0d0
                  do ik = k1, k2
                    iku = k2 - ik + 1
                    col_sum_depos(:) =
     &                col_sum_depos(:) +
     &                (const(:,ij,ik,ic) - qq(:,iku,ic)) *
     &                mass(:,ij,ik)
                  end do
                  
                  where (col_sum_depos(i1:i2) > 0.0d0)      
                      wet_depos(i1:i2,ij,ic) =
     &                wet_depos(i1:i2,ij,ic) + 
     &                col_sum_depos(i1:i2) * mw(ic) /
     &                MWTAIR / mcor(i1:i2,ij)
                  end where 
                  


                end if
              end do

!            end if


            const(:,ij,k1:k2,1:num_species) =
     &        qq(:,k2:k1:-1,1:num_species)

c         ================
          end do itdtcloop
c         ================

c       =============
        end if il2gif
c       =============

c     =============
      end do ijloop
c     =============


      return

      end
 

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Conv_Tran
c
c DESCRIPTION
c   This routine performs convective transport of trace species.
c
c   Note that we assume that the tracers are in a moist mixing ratio
c   (this should change soon).
c
c ARGUMENTS
c   INPUT:
c     il2g   : gathered max lon indices over which to operate
c     delt   : convection time step (s)
c     xmbsth : threshold below which we treat mass fluxes as zero (mb/s)
c     ideep  : gathering array
c     pbli   : index of planetary boundary layer
c     eui    : mass entraining from updraft (s^-1)
c     mui    : mass flux up
c     dpi    : delta pressure between interfaces
c     fracis : insoluble fraction of tracer
c   INPUT/OUTPUT:
c     qq     : tracer array including moisture (mixing ratio)
c
c-----------------------------------------------------------------------------
 
      subroutine Conv_Tran
     &  (il2g, delt, xmbsth, ideep, pbli, eui, mui, dpi, fracis, qq)

      use h_imp_dims
      
      implicit	none

!#     include "imp_dims.h"

 
c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: il2g
      real*8  :: delt
      real*8  :: xmbsth
      integer :: ideep (ilong)
      integer :: pbli  (ilong)
      real*8  :: eui   (ilong, ivert)
      real*8  :: mui   (ilong, ivert)
      real*8  :: dpi   (ilong, ivert)
      real*8  :: fracis(i1:i2, k1:k2, num_species)
      real*8  :: qq    (i1:i2, k1:k2, num_species)


c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter :: SMALL = 1.0d-36


c     ----------------------
c     Function declarations.
c     ----------------------

      logical :: Is_Fixed_Const


c     ----------------------
c     Variable declarations.
c     ----------------------

      character (len=75) :: err_msg

      integer :: il, ik, ic
      integer :: km1, kp1

      real*8  :: avg_pbl    ! average mixing ratio in pbl
      real*8  :: cabv       ! mix ratio of constituent above
      real*8  :: cbel       ! mix ratio of constituent below
      real*8  :: cdifr      ! normalized diff. between cabv and cbel
      real*8  :: fluxin     ! flux coming into  each box at a k level
      real*8  :: fluxout    ! flux going out of each box at a k level
      real*8  :: maxc
      real*8  :: minc
      real*8  :: scav       ! mixing ratio of a scavenged tracer
      real*8  :: sqrt_fisg

      real*8  :: chat  (il2g,ivert)  ! mix ratio in env.      at interfaces
      real*8  :: conu  (il2g,ivert)  ! mix ratio in updraft   at interfaces
      real*8  :: dcondt(il2g,ivert)  ! gathered tend.  array 
      real*8  :: fisg  (il2g,ivert)  ! gathered insoluble frac. of tracer
      real*8  :: xconst(il2g,ivert)  ! gathered tracer array 


c     ----------------
c     Begin execution.
c     ----------------

c     =======
      icloop: do ic = 1, num_species
c     =======

c       ====================================
        if (Is_Fixed_Const(ic)) cycle icloop
c       ====================================

c       ------------------------------------------------
c       Gather up the constituent and set tend. to zero.
c       ------------------------------------------------

        do ik = k1, k2
          do il = 1, il2g

            fisg  (il,ik) = fracis(ideep(il),ik,ic)
            xconst(il,ik) = qq    (ideep(il),ik,ic)

          end do
        end do


c       -----------------------------------------
c       From now on work only with gathered data.
c       -----------------------------------------

c       ----------------------------------------------------
c       Interpolate environment tracer values to interfaces.
c       ----------------------------------------------------

c       ========
        ikloop1: do ik = k1, k2
c       ========

          km1 = Max (k1, ik-1)

c         ========
          illoop1: do il = 1, il2g
c         ========

            minc = Min (xconst(il,km1), xconst(il,ik))
            maxc = Max (xconst(il,km1), xconst(il,ik))

            if (minc < 0.0d0) then

              cdifr = 0.0d0

            else

              cdifr = Abs (xconst(il,ik) - xconst(il,km1)) /
     &                Max (maxc, SMALL)

            end if


            if (cdifr > 1.0d-6) then

c             -------------------------------------------------------
c             The two layers differ significantly, so use a geometric
c             averaging procedure.
c             -------------------------------------------------------

              cabv = Max (xconst(il,km1), (maxc * 1.0d-12))
              cbel = Max (xconst(il,ik),  (maxc * 1.0d-12))

              chat(il,ik) =
     &          Log (cabv/cbel) / (cabv - cbel) * cabv * cbel

            else

c             ---------------------------------------------------
c             The two layers have only a small difference, so use
c             arithmetic mean.
c             ---------------------------------------------------

              chat(il,ik) = 0.5d0 * (xconst(il,ik) + xconst(il,km1))

            end if


c           -------------------------------------
c           Provisional up and down draft values.
c           -------------------------------------

            conu(il,ik) = chat(il,ik)


c           ------------------
c           Provisional tends.
c           ------------------

            dcondt(il,ik) = 0.0d0

c         ==============
          end do illoop1
c         ==============

c       ==============
        end do ikloop1
c       ==============


c       ========
        illoop2: do il = 1, il2g
c       ========

c         ------------------------------------------------------
c         Calculate updrafts with scavenging from bottom to top.
c         ------------------------------------------------------

c         -------------------------------------
c         Do the bottom most levels in the pbl.
c         -------------------------------------

          if (Sum (dpi(il,k2:pbli(il):-1)) == 0.0d0)  then

            err_msg = 'Problem in Conv_Tran.'
!            call Do_Err_Out (err_msg, .true., 2, ideep(il), pbli(il),
!     &                       0, 0.0d0, 0.0d0)
           stop 'Problem in Conv_Tran.'
          end if 

          avg_pbl = 
     &      Sum (xconst(il,k2:pbli(il):-1) * dpi(il, k2:pbli(il):-1)) /
     &      Sum (dpi(il,k2:pbli(il):-1))

          sqrt_fisg = Sqrt (fisg(il,pbli(il)))

          scav    = avg_pbl * (1.0d0 -  sqrt_fisg)
 
          conu(il,pbli(il)) = avg_pbl * sqrt_fisg

          fluxin  = mui(il,pbli(il)) * 
     &              Min (chat(il,pbli(il)), xconst(il,pbli(il)-1))

          fluxout = mui(il,pbli(il)) * conu(il,pbli(il)) +
     &              mui(il,pbli(il)) * scav                 

          dcondt(il,k2) = (fluxin - fluxout) / 
     &                    Sum (dpi(il,k2:pbli(il):-1))

          if (avg_pbl > 0.0d0) then

            do ik = k2, pbli(il), -1

              qq(ideep(il),ik,ic) =
     &          (avg_pbl + (dcondt(il,k2)*delt)) /
     &          avg_pbl * xconst(il,ik)

            end do

          end if


c         ---------------------------
c         Loop over all other levels.
c         ---------------------------

c         ========
          ikloop2: do ik = pbli(il)-1, 2, -1
c         ========

            kp1 = ik + 1
            km1 = ik - 1

c           -----------------------------
c           Find mixing ratio in updraft.
c           -----------------------------

            if ((mui(il,kp1) + eui(il,ik)) > xmbsth) then

              sqrt_fisg   = Sqrt (fisg(il,ik))

              scav        = (mui(il,kp1) * conu(il,kp1)  * 
     &                       (1.0d0 - fisg(il,ik)) + 
     &                       eui(il,ik)  * xconst(il,ik) * 
     &                       (1.0d0 - sqrt_fisg)) /
     &                      (mui(il,kp1) + eui(il,ik))

              conu(il,ik) = (mui(il,kp1) * conu(il,kp1)  * 
     &                       fisg(il,ik) +
     &                       eui(il,ik)  * xconst(il,ik) *
     &                       sqrt_fisg) /
     &                      (mui(il,kp1) + eui(il,ik))

            else 

              conu(il,ik) = xconst(il,ik)

              scav        = 0.0d0

            end if

c           -------------------------------------------------------
c           Calculate fluxes into and out of box.  With scavenging
c           included the net flux for the whole column is no longer
c           guaranteed to be zero.
c           -------------------------------------------------------

            fluxin  = mui(il,kp1) * conu(il,kp1) +
     &                mui(il,ik)  * Min (chat(il,ik), xconst(il,km1))
  
            fluxout = mui(il,ik)  * conu(il,ik) + 
     &                mui(il,kp1) * Min (chat(il,kp1), xconst(il,ik)) +
     &                (mui(il,ik) + mui(il,kp1)) * 0.5d0 * scav

            dcondt(il,ik) = (fluxin - fluxout) / dpi(il,ik)

c           --------------------------------------------
c           Update and scatter data back to full arrays.
c           --------------------------------------------

            qq(ideep(il),ik,ic) = xconst(il,ik) + (dcondt(il,ik) * delt)

c         ==============
          end do ikloop2
c         ==============

c       ==============
        end do illoop2
c       ==============

c     =============
      end do icloop
c     =============


      return

      end

