 
c=============================================================================
c
c $Id: imp_drydep_data.h,v 1.4 2003/01/17 04:52:36 jrt Exp $
c
c CODE DEVELOPER
c   Dan Bergmann, LLNL
c   dbergmann@llnl.gov
c
c FILE
c   imp_drydep_data.h
c
c DESCRIPTION
c   This include file sets the data required for the dry deposition algorithm.
c   It includes land types, resistances, and polynomial coefficients from
c   files "drydep_table" and "drydep_coef", which were supplied by Daniel
c   Jacob at Harvard.  The original files follow these parameter statements.
c
c   NOTE THAT SURFACE DEPENDENT PARAMETERS WERE ADDED ON 2/11/02 TO CALCULATE
c   SURFACE RESISTANCE FOR AEROSOLS.
c
c=============================================================================

 
c     -------------------
c     Integer parameters.
c     -------------------

c     --------------------------------------------------------------------
c     IDEP   : deposition surface type for each Olson surface type
c     IRAC   : resistance that depends on canopy height and density (s^-1)
c     IRCLO  : resistance for leaves, twig, bark in lower canopy    (s^-1)
c     IRCLS  : resistance for leaves, twig, bark in lower canopy    (s^-1)
c     IRGSO  : ground    resistance (s^-1) 
c     IRGSS  : ground    resistance (s^-1)
c     IRI    : internal  resistance (s^-1)
c     IRLU   : cuticular resistance (s^-1)
c     IWATER : id's for surface types that are water
c     IZO    : roughness height (m/10000)
c     NWATER : number of Olson's surface types that are water
c     --------------------------------------------------------------------

      integer, parameter :: IDEP(NVEGTYPE) = (/
     &  11, 10,  5,  1,  1,  1,  2,  1,  8,  1,
     &   1,  1,  1,  1,  1,  1,  5,  1,  1,  1,
     &   3,  3,  3,  3,  2,  2,  2,  3,  2,  2,
     &   4,  4,  2,  6,  1,  1,  9,  4,  4,  4,
     &   5,  5,  5,  5,  5,  9,  5,  5,  5,  5,
     &   8,  8,  5,  7,  6,  2,  2,  2,  2,  2,
     &   3,  3,  3,  5,  5, 11, 11, 11, 11,  8,
     &   1,  8,  9, 11 /)

      integer, parameter :: IRAC(NTYPE) = (/
     &     0, 2000, 2000,  200,  100, 
     &  2000,    0,    0,  300,  100, 
     &     0,    0,    0,    0,    0 /)

      integer, parameter :: IRCLO(NTYPE) = (/
     &  1000, 1000, 1000, 1000, 1000, 
     &  9999, 9999, 9999, 1000, 9999, 
     &  9999,    0,    0,    0,    0 /)

      integer, parameter :: IRCLS(NTYPE) = (/
     &  9999, 2000, 2000, 2000, 2000,
     &  9999, 9999, 9999, 2500, 9999, 
     &  9999,    0,    0,    0,    0 /)

      integer, parameter :: IRGSO(NTYPE) = (/
     &  3500,  200,  200,  150,  200,
     &   200,  340,  400, 1000,  300, 
     &  2000,    0,    0,    0,    0 /)

      integer, parameter :: IRGSS(NTYPE) = (/
     &   100,  500,  500,  150,  350, 
     &   200,  340, 1000,    0,  400, 
     &     0,    0,    0,    0,    0 /)

      integer, parameter :: IRI(NTYPE) = (/
     &  9999,  200,  400,  200,  200,
     &   200,  200, 9999,  200, 9999,
     &  9999,    0,    0,    0,    0 /)

      integer, parameter :: IRLU(NTYPE) = (/
     &  9999, 9000, 9000, 9000, 9000, 
     &  1000, 4000, 9999, 9000, 9999, 
     &  9999,    0,    0,    0,    0 /)

      integer, parameter :: IWATER(NVEGTYPE) = (/
     &  1, 66, 67, 68, 69, 74,  0,  0,  0,  0,
     &  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     &  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     &  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     &  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     &  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     &  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     &  0,  0,  0,  0 /)

      integer, parameter :: IZO(NVEGTYPE) = (/
     &      1,10000,   50, 1000, 1000, 1000,10000, 1000,    1, 1000,
     &   1000, 1000, 1000, 1000, 1000, 1000, 1000,    1, 1000, 1000,
     &  10000,10000,10000,10000,10000,10000,10000,10000, 1000,10000,
     &   1000, 1000, 1000,10000, 1000, 1000,  100, 1000, 1000, 1000,
     &    100,  100,  100,  100,  100,  100, 1000, 1000, 1000, 1000,
     &    100,  100,  100,   50,10000, 1000, 1000, 1000, 1000, 1000,
     &  10000,10000,10000, 1000,   50,    1,    1,    1,    1,   10,
     &     10,    1,  500,    1 /)

      integer, parameter :: NWATER = 6 

 
c     --------------------------------------------------
c     From Zhang et al. [Atmos. Env., p549-560, 2001] =>
c
c     IWET : flag for wet surface = 1, otherwise = -1
c     --------------------------------------------------

      integer, parameter :: IWET(NTYPE) = (/
     &     1,   -1,   -1,   -1,   -1, 
     &    -1,   -1,   -1,    1,   -1, 
     &     1,    0,    0,    0,    0 /)


c     ----------------
c     Real parameters.
c     ----------------

c     ------------------------------------------
c     DRYCOEFF : polynomial fitting coefficients
c     ------------------------------------------

      real*8, parameter  :: DRYCOEFF(NPOLY) = (/
     &  -3.58d-01,  3.02d+00,  3.85d+00, -9.78d-02, -3.66d+00, 
     &   1.20d+01,  2.52d-01, -7.80d+00,  2.26d-01,  2.74d-01, 
     &   1.14d+00, -2.19d+00,  2.61d-01, -4.62d+00,  6.85d-01, 
     &  -2.54d-01,  4.37d+00, -2.66d-01, -1.59d-01, -2.06d-01 /)


c     ------------------------------------------------------------------------
c     From Zhang et al. [Atmos. Env., p549-560, 2001] =>
c
c     A_VEG   : characteristic radius of collectors over vegetated surface (m)
c     ALFA_IM : value for calculating collection efficiency from impaction (m)
c     GAMA_BR : value for calculating collection efficiency from Brownian  (m)
c     ------------------------------------------------------------------------

      real*8, parameter  :: A_VEG(NTYPE) = (/
     &  -1.00d+00,  5.10d-03,  3.50d-03,  3.20d-03, 10.00d-03, 
     &   5.00d-03, -1.00d+00, -1.00d+00, 10.00d-03, 10.00d-03, 
     &  -1.00d+00,  0.00d+00,  0.00d+00,  0.00d+00,  0.00d+00 /) 

      real*8, parameter  :: ALFA_IM(NTYPE) = (/
     &  50.00d+00,  0.95d+00,  0.80d+00,  1.20d+00,  1.30d+00, 
     &   0.80d+00, 50.00d+00, 50.00d+00,  2.00d+00,  1.50d+00, 
     & 100.00d+00,  0.00d+00,  0.00d+00,  0.00d+00,  0.00d+00 /) 

      real*8, parameter  :: GAMA_BR(NTYPE) = (/
     &   0.54d+00,  0.56d+00,  0.57d+00,  0.54d+00,  0.54d+00, 
     &   0.56d+00,  0.54d+00,  0.54d+00,  0.54d+00,  0.56d+00, 
     &   0.50d+00,  0.00d+00,  0.00d+00,  0.00d+00,  0.00d+00 /) 


c ****************Daniel J. Jacob 6/27/94 ****************************************
c *** Olson surface types, corresponding surface types for deposition,
c ***  and corresponding roughness height
c ********************************************************************************
c Olson ID# - deposition ID# - z0 in 1.E-4 m
c      1    11     1    Water
c      2    10 10000    Urban
c      3     5    50    Shrub
c      4     1  1000    Not used
c      5     1  1000    ibid.
c      6     1  1000    ibid.
c      7     2 10000    Tropical evergreen
c      8     1  1000    Not used
c      9     8     1    Desert
c     10     1  1000    Not used
c     11     1  1000    ibid.
c     12     1  1000    ibid.
c     13     1  1000    ibid.
c     14     1  1000    ibid.
c     15     1  1000    ibid.
c     16     1  1000    ibid.
c     17     5  1000    Scrub
c     18     1     1    Ice
c     19     1  1000    Not used
c     20     1  1000    ibid.
c     21     3 10000    Conifer
c     22     3 10000    Conifer
c     23     3 10000    Conifer
c     24     3 10000    Conifer/deciduous
c     25     2 10000    Deciduous/conifer
c     26     2 10000    Deciduous
c     27     2 10000    Deciduous
c     28     3 10000    Conifer
c     29     2  1000    Dwarf forest
c     30     2 10000    Tropical broadleaf
c     31     4  1000    Agric.
c     32     4  1000    ibid.
c     33     2  1000    Dec. woodland
c     34     6 10000    Tropical rainforest
c     35     1  1000    Not used
c     36     1  1000    Not used
c     37     9   100    Rice paddies
c     38     4  1000    Agric.
c     39     4  1000    Agric.
c     40     4  1000    Agric.
c     41     5   100    Shrub/grass
c     42     5   100    Shrub/grass
c     43     5   100    Shrub/grass
c     44     5   100    Shrub/grass
c     45     5   100    Shrub/grass
c     46     9   100    Wetland
c     47     5  1000    Scrub
c     48     5  1000    Scrub
c     49     5  1000    Scrub
c     50     5  1000    Scrub
c     51     8   100    Desert
c     52     8   100    Desert
c     53     5   100    Steppe
c     54     7    50    Tundra
c     55     6 10000    Rainforest
c     56     2  1000    Mixed wood/open
c     57     2  1000    Mixed wood/open
c     58     2  1000    Mixed wood/open
c     59     2  1000    Mixed wood/open
c     60     2  1000    Mixed wood/open
c     61     3 10000    Conifers
c     62     3 10000    Conifers
c     63     3 10000    Conifers
c     64     5  1000    Wooded tundra
c     65     5    50    Moor
c     66    11     1    Coastal
c     67    11     1    Coastal
c     68    11     1    Coastal
c     69    11     1    Coastal
c     70     8    10    Desert
c     71     1    10    Ice
c     72     8     1    Salt flats
c     73     9   500    Wetland
c     74    11     1    Water
c **
c   6  1 66 67 68 69 74                    #/Olson ID's of water surface types
c **  Resistances (s m-1) for each deposition surface type,
c **  and maximum deposition velocity Vsmax(1.d-2 cm s-1) for aerosol
c   Type  Ri  Rlu  Rac RgsS RgsO RclS RclO Vsmax
c     1 9999 9999    0  100 3500 9999 1000  100   Snow/Ice (Wesely, AE 1989)-
c                                                   listed first.
c     2  200 9000 2000  500  200 2000 1000  100   Deciduous  forest (Wesely)
c     3  400 9000 2000  500  200 2000 1000  100   Coniferous forest (Wesely)
c     4  200 9000  200  150  150 2000 1000  100   Agricultural land (Wesely)
c     5  200 9000  100  350  200 2000 1000  100   Shrub/grassland   (Wesely)
c     6  200 1000 2000  200  200 9999 9999  100   Amazon forest
c                                                   (Jacob & Wofsy, JGR 1990)
c     7  200 4000    0  340  340 9999 9999  100   Tundra
c                                                   (Jacob et al., JGR 1992)
c     8 9999 9999    0 1000  400 9999 9999   10   Desert  (Wesely)
c     9  200 9000  300    0 1000 2500 1000  100   Wetland (Wesely)
c    10 9999 9999  100  400  300 9999 9999  100   Urban   (Wesely)
c    11 9999 9999    0    0 2000 9999 9999   10   Water   (Wesely)
c 

