
c=============================================================================
c
c $Id: imp_emiss_constants.h,v 1.6 2003/03/21 23:32:23 jrt Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL
c   jrt@llnl.gov
c
c FILE
c   imp_emiss_constants.h
c
c DESCRIPTION
c   This include file contains constants for the Harvard biogenic and soil
c   emissions.
c
c=============================================================================


cc!?  Should be fixed sometime.
c     -------------------------------------------------------------
c     NOTE THAT NEED TO MATCH THE NLANDHAR PARAMETER BELOW WITH THE
c     RESOLUTION YOU ARE RUNNING AT!
c     -------------------------------------------------------------

      integer, parameter ::
cc   &  NLANDHAR = 3920      ! number of grids on land for 2x2.5 resolution
     &  NLANDHAR = 1118      ! number of grids on land for 4x5   resolution


      integer, parameter ::
     &  NPOLY    =   20,     ! number of coefficients for polynomial fits
     &  NPULSE   =    3,     ! number of types of pulsing
     &  NSOIL    =   11,     ! defined soil types
     &  NTYPE    =   15,     ! maximum number of vegetation types in any
                             ! grid box
     &  NVEGTYPE =   74      ! maximum number of surface types (Olson)


c     ----------------------------------------------------------
c     MWTCARBON : molecular weight of Carbon (g/mol)
c     MWTCO     : molecular weight of CO     (g/mol)
c     MWTMONOT  : molecular weight of monoterpene (C2H4) (g/mol)
c     ----------------------------------------------------------

      real*8, parameter ::
     &  MWTCARBON = 12.01d0,
     &  MWTCO     = 28.01d0,
     &  MWTMONOT  = 28.00d0


c     ---------------------------------------------
c     ATOMSC_PER_MOLECCO    : atomsC/molec CO
c     ATOMSC_PER_MOLECISOP  : atomsC/molec isoprene
c     ATOMSC_PER_MOLECMONOT : atomsC/molec proprene
c     ATOMSC_PER_MOLECPRPE  : atomsC/molec proprene
c     ---------------------------------------------

      real*8, parameter ::
     &  ATOMSC_PER_MOLECCO    = 1.0d0,
     &  ATOMSC_PER_MOLECISOP  = 5.0d0,
     &  ATOMSC_PER_MOLECMONOT = 2.0d0,
     &  ATOMSC_PER_MOLECPRPE  = 3.0d0


c     ----------------------------------------------------------------------
c     BIOSCAL : factor for biogenic source of propene, scaled from isoprene
c               (atomsCalkenes / atomsCisoprene)
c
c     We need to scale the isoprene flux to get the biogenic emissions of
c     alkenes (probably OK for summertime conditions).
c
c     The scaling factor is based on work by Allen Goldstein.  His values
c     indicate emission ratios of ethene:propene:butene=4:2:1 (on a per
c     molecule basis), with total emissions approx. equal to 10% of isoprene
c     emissions (again, on a per molecule basis):
c
c       (10 molec alkenes  / 100 molec  isoprene) *
c       ( 1 molec isoprene /   5 atomsC isoprene) *
c       ( 3      molec butene + propene   / 7 molec total alkenes) *
c       ( 3.3333 atomsCbutene+propene mix / 1 molec butene+propene mix) =
c
c         0.0286 atomsCbutene+propene / atomsCisoprene
c
c     Note that 3.3333 atomsC/molec is the weighted average for this mix.
c     Note also that this factor now excludes ethene.
c     ----------------------------------------------------------------------

      real*8, parameter ::
     &  BIOSCAL = 0.0286d0


c     ------------------------------------------------------------------
c     ICO_FAC_ISOP : factor for biogenic source of CO from methanol
c                    oxidation, scaled from isoprene
c
c     We need to scale the isoprene flux to get the CH3OH (methanol)
c     flux.  Currently, the annual isoprene flux in GEOS-CHEM is
c     ~397 tg C.
c
c     Daniel Jacob recommends a flux of 100 tg/yr CO from CH3OH
c     oxidation based on Singh et al., 2000 (JGR 105, 3795-3805), who
c     estimate a global methanol source of 122 tg/yr, of which most
c     (75 tg/yr) is "primary biogenic".  He also recommends for now,
c     that the CO flux from CH3OH oxidation be scaled from monthly mean
c     isoprene flux.
c
c     To get CO from methanol oxidation, we must therefore multiply
c     the isoprene flux by the following scale factor:
c
c       (100 tg CO from CH3OH oxidation / 397 tg C from isoprene flux) *
c       ( 60 g C/mole isoprene          /  68 g isoprene/mole)
c     ------------------------------------------------------------------

      real*8, parameter ::
     &  ICO_FAC_ISOP = (100.0d0 / 397.0d0) * (60.0d0 / 68.0d0)


c     ------------------------------------------------------------------
c     ICO_FAC_MONOT : factor for biogenic source of CO from monoterpene
c                     oxidation.
c
c     Assume the production of CO from monoterpenes is instantaneous
c     even though the lifetime of intermediate species may be on the
c     order of hours or days.  This assumption will likely cause CO
c     from monoterpene oxidation to be too high in the box in which
c     the monoterpene is emitted.
c
c     The CO yield here is taken from:
c
c     Hatakeyama et al., JGR, Vol. 96, p. 947-958 (1991) =>
c       "The ultimate yield of CO from the tropospheric oxidation of
c       terpenes (including both O3 and OH reactions) was estimated to
c       be 20% on the carbon number basis."  (They studied alpha- &
c       beta-pinene.)
c
c     Vinckier et al., Fresenius Env. Bull., Vol. 7, p.361-368 (1998) =>
c       "R(CO) = 1.8+/-0.3"  ((1.8/10) is about 20%.)
c     ------------------------------------------------------------------

      real*8, parameter ::
     &  ICO_FAC_MONOT =
     &    0.2d0 *
     &    (ATOMSC_PER_MOLECMONOT / ATOMSC_PER_MOLECCO) *
     &    (MWTCO / MWTMONOT)


      real*8, parameter  ::
     &  PULSE_DECAY(NPULSE) =  ! values from Yienger & Levy
     &    (/ 0.805d0, 0.384d0, 0.208d0 /)

      real*8, parameter  ::
     &  PULSE_FAC(NPULSE)   =  ! initial pulsing factor following a precip.
                               ! event
     &    (/ 5.0d0, 10.0d0, 15.0d0 /)


      real*8, parameter  ::
     &  SOIL_AD(NSOIL)      =  ! dry biome coefficient
     &    (/ 0.0d0,  8.6d0,  0.22d0, 0.40d0,  0.22d0, 1.44d0,
     &       2.65d0, 2.65d0, 2.65d0, 0.003d0, 0.37d0 /)

      real*8, parameter  ::
     &  SOIL_AW(NSOIL)      =  ! wet biome coefficient
     &    (/ 0.0d0,  2.6d0,  0.03d0, 0.06d0,  0.03d0, 0.17d0,
     &       0.36d0, 0.36d0, 0.36d0, 0.003d0, 0.05d0 /)


      real*8, parameter  ::
     &  SOIL_EXT(NSOIL)     =  ! canopy wind extinction coefficient
     &    (/ 0.1d0, 4.0d0, 4.0d0, 4.0d0, 4.0d0, 2.0d0,
     &       1.0d0, 2.0d0, 2.0d0, 0.5d0, 0.1d0 /)


      real*8, parameter  ::
     &  SOIL_T1(NSOIL)      =  ! first  coefficient used to convert from
                               ! surface temperture to soil temperature
     &    (/ 0.0d0,  0.84d0, 0.84d0, 0.84d0, 0.84d0, 0.66d0, 0.66d0,
     &       1.03d0, 1.03d0, 0.92d0, 0.66d0 /)

      real*8, parameter  ::
     &  SOIL_T2(NSOIL)      =  ! second coefficient used to convert from
                               ! surface temperture to soil temperature
     &    (/ 0.0d0, 3.6d0, 3.6d0, 3.6d0, 3.6d0, 8.8d0, 8.8d0,
     &       2.9d0, 2.9d0, 4.4d0, 8.8d0 /)

