       module h_imp_aerosol 
c=============================================================================
c
c=============================================================================
c
c $Id: imp_aerosol.h,v 1.2 2001/12/13 22:27:08 jrt Exp $
c
c CODE DEVELOPER
c   Dan Bergmann, LLNL
c   dbergmann@llnl.gov
c
c FILE
c   imp_aerosol.h
c
c DESCRIPTION
c   This include file contains information about various aerosols  
c
c=============================================================================


c     -----------------------
c     Parameter declarations.
c     -----------------------

c     ----------------------------------------------------------------------
c     NUM_AEROSOL : # of different aerosol types having settling velocities:
c                   1) sulfate 1  (0.0   - 0.005 um)
c                   2) sulfate 2  (0.005 - 0.05 um)
c                   3) sulfate 3  (0.05  - 0.63 um)
c                   4) natural organic
c                   5) fossil fuel OC
c                   6) fossil fuel BC
c                   7) biomass OC
c                   8) biomass BC
c                   9) aircraft soot BC  added by Roy
c                  10) dust 1     (0.05 - 0.63 um)
c                  11) dust 2     (0.63 - 1.26 um)
c                  12) dust 3     (1.26 - 2.5  um)
c                  13) dust 4     (2.5  - 10   um)
c                  14) sea salt 1 (0.05 - 0.63 um)
c                  15) sea salt 2 (0.63 - 1.26 um)
c                  16) sea salt 3 (1.26 - 2.5  um)
c                  17) sea salt 4 (2.5  - 10   um)
c                  18) stratospheric sulfate (mono-dispersal)
c                  19) aircraft soot
c     ----------------------------------------------------------------------

      integer, parameter :: NUM_AEROSOL = 19
      integer, parameter :: NUM_WAV_AER = 1

 
c     ----------------------------------------------------
c     AER_DENSITY : density of each aerosol type (kg/m^3).
c     ----------------------------------------------------

      real*8, parameter :: AER_DENSITY(NUM_AEROSOL) =
     &  (/ 1769.d0, 1769.d0, 1769.d0, 1500.d0, 1500.d0, 1500.d0, 1500.d0,
     &     1500.d0, 1500.d0, 2500.d0, 2500.d0, 2650.d0, 2650.d0,
     &     2216.d0, 2216.d0, 2216.d0, 2216.d0, 1769.d0, 1500.d0 /)

c     --------------------------------------------------------
c     RADIUS_EFF : effective radius of each aerosol type (m).
c     --------------------------------------------------------

c      real*8, parameter :: RADIUS_EFF(NUM_AEROSOL) =
c     &  (/ 0.0405d-6, 0.0405d-6, 0.2695d-6, 0.2064d-6, 0.2130d-6,
c     &     0.2130d-6, 0.2064d-6, 0.2064d-6, 0.0444d-6, 0.4610d-6, ! summer dust
c     &     0.9868d-6, 1.9140d-6, 5.8800d-6, 0.4040d-6, 0.9759d-6,
c     &     1.9420d-6, 7.1030d-6, 0.1000d-6, 0.0444d-6 /)
cZhuo:change the RE of sslt3 and sslt4(mass averaged dry radii) 
c since the real range of sslt4 is [2.5um dry r to 10um dry r]
      real*8, parameter :: RADIUS_EFF(NUM_AEROSOL) =
     &  (/ 0.0405d-6, 0.0405d-6, 0.2695d-6, 0.2064d-6, 0.2130d-6,
     &     0.2130d-6, 0.2064d-6, 0.2064d-6, 0.0444d-6, 0.4610d-6, ! summer dust
     &     0.9868d-6, 1.9140d-6, 5.8800d-6, 0.4040d-6, 0.9759d-6,
     &     1.8375d-6, 5.4319d-6, 0.1000d-6, 0.0444d-6 /)

c     -------------------------------------------------------
c     RADIUS_EFF_EMISN : effective radius (m) for interactive
c                        sea salt and dust emission modules
c     -------------------------------------------------------

      real*8, parameter :: RADIUS_EFF_EMISN(NUM_AEROSOL) =
     &  (/0.0000d-6, 0.0000d-6, 0.0000d-6, 0.0000d-6, 0.0000d-6,
     &    0.0000d-6, 0.0000d-6, 0.0000d-6, 0.0000d-6, 0.7500d-6,
     &    0.7500d-6, 1.9140d-6, 5.8800d-6, 0.2945d-6, 0.9653d-6,
     &    1.8623d-6, 4.6262d-6, 0.0000d-6, 0.0000d-6 /)

c     ----------------------------------------------------
c     SIZE_RANGE_LO : lower end of size range (m) for each
c                     aerosol type
c     ----------------------------------------------------

      real*8, parameter :: SIZE_RANGE_LO(NUM_AEROSOL) =
     &  (/ 0.0000d-6, 0.0050d-6, 0.0500d-6, 0.0001d-6, 0.0001d-6,
     &     0.0001d-6, 0.0001d-6, 0.0001d-6, 0.0001d-6, 0.0500d-6,
     &     0.6300d-6, 1.2600d-6, 2.5000d-6, 0.0500d-6, 0.6300d-6,
     &     1.2600d-6, 2.5000d-6, 0.0001d-6, 0.0001d-6 /)

c     ----------------------------------------------------
c     SIZE_RANGE_HI : upper end of size range (m) for each
c                     aerosol type
c     ----------------------------------------------------

      real*8, parameter :: SIZE_RANGE_HI(NUM_AEROSOL) =
     &  (/ 0.0050d-6, 0.0500d-6, 0.6300d-6,10.0000d-6,10.0000d-6,
     &    10.0000d-6,10.0000d-6,10.0000d-6,10.0000d-6,0.6300d-6,
     &     1.2600d-6, 2.5000d-6,10.0000d-6, 0.6300d-6,1.2600d-6,
     &     2.5000d-6,10.0000d-6,10.0000d-6,10.0000d-6 /)
c     --------------------------------------------------------
c     Constants used in swelling calculation;
c     to turn off swelling for a particular aerosol, set C1=0.
c     --------------------------------------------------------

      real*8, parameter :: C1(NUM_AEROSOL) =
     &  (/  0.4809d0,  0.4809d0,  0.4809d0,  0.2789d0,  0.3926d0,
     &      0.3926d0,  0.2789d0,  0.2789d0,  0.4809d0,  0.0000d0,
     &      0.0000d0,  0.0000d0,  0.0000d0,  0.7674d0,  0.7674d0,
     &      0.7674d0,  0.7674d0,  0.4809d0,  0.4809d0 /)

      real*8, parameter :: C2(NUM_AEROSOL) =
     &  (/  3.082d0,   3.082d0,   3.082d0,   3.115d0,   3.101d0,
     &      3.101d0,   3.115d0,   3.115d0,   3.082d0,   0.000d0,
     &      0.000d0,   0.000d0,   0.000d0,   3.079d0,   3.079d0,
     &      3.079d0,   3.079d0,   3.082d0,   3.082d0 /)

      real*8, parameter :: C3(NUM_AEROSOL) =
     &  (/  3.110d-11, 3.110d-11, 3.110d-11, 5.415d-11, 4.190d-11,
     &      4.190d-11, 5.415d-11, 5.415d-11, 3.110d-11, 0.000d-00,
     &      0.000d-00, 0.000d-00, 0.000d-00, 2.573d-11, 2.573d-11,
     &      2.573d-11, 2.573d-11, 3.110d-11, 3.110d-11 /)

      real*8, parameter :: C4(NUM_AEROSOL) =
     &  (/ -1.428d0,  -1.428d0,  -1.428d0,  -1.399d0,  -1.404d0,
     &     -1.404d0,  -1.399d0,  -1.399d0,  -1.428d0,   0.000d0,
     &      0.000d0,   0.000d0,   0.000d0,  -1.424d0,  -1.424d0,
     &     -1.424d0,  -1.424d0,  -1.428d0,  -1.428d0 /)


c     ----------------------------------------------------------------
c     REL_SCAV_EFF : scavenging efficiency of each aerosol relative to
c                    sulfate (unitless)
c     ----------------------------------------------------------------

      real*8, parameter :: REL_SCAV_EFF(NUM_AEROSOL) =
     &  (/ 1.0d0, 1.0d0, 1.0d0, 0.4d0, 0.4d0, 0.4d0, 0.4d0, 0.4d0,
     &     0.4d0, 1.0d0, 1.0d0, 1.0d0, 1.0d0, 1.0d0, 1.0d0, 1.0d0,
     &     1.0d0, 1.0d0, 1.0d0 /)

c    --------------------------------------------------------------------
c    REL_SCAV_EFF_MAX, REL_SCAV_EFF_MIN are from Stier et al., 2005, ACP
c    by Minghuai Wang (minghuai@umich.edu)
c    --------------------------------------------------------------------
            real*8, parameter :: REL_SCAV_EFF_MAX(NUM_AEROSOL) =
     &  (/ 0.80d0, 0.80d0, 1.00d0, 0.80d0, 0.80d0, 0.80d0, 0.80d0,
     &     0.80d0, 0.80d0, 0.85d0, 0.99d0, 0.99d0, 0.99d0, 0.85d0,
     &     0.99d0, 0.99d0, 0.99d0, 1.00d0, 1.00d0/)

      real*8, parameter :: REL_SCAV_EFF_MIN(NUM_AEROSOL) =
     &  (/ 0.30d0, 0.30d0, 1.00d0, 0.30d0, 0.30d0, 0.30d0, 0.30d0,
     &     0.30d0, 0.30d0, 0.40d0, 0.40d0, 0.60d0, 0.60d0, 0.40d0,
     &     0.40d0, 0.60d0, 0.60d0, 1.00d0, 1.00d0/)

      real*8, parameter :: REL_SCAV_EFF_ICE(NUM_AEROSOL) =
     &  (/ 0.10d0, 0.10d0, 0.10d0, 0.10d0, 0.10d0, 0.10d0, 0.10d0,
     &     0.10d0, 0.10d0, 0.10d0, 0.10d0, 0.10d0, 0.10d0, 0.10d0,
     &     0.10d0, 0.10d0, 0.10d0, 0.10d0, 0.10d0/)

C revised ice cloud freezing fraction based on Koehler et al. 2009 for soot.
C for hydroscopic soot, nucleation fraction is 3%. For hydrophilic and hydrophobic, it is 0.1%, see his table 1
!      real*8, parameter :: REL_SCAV_EFF_ICE(NUM_AEROSOL) =
!     &  (/ 0.10d0, 0.10d0, 0.10d0, 0.10d0, 0.03d0, 0.03d0, 0.03d0,
!     &     0.03d0,         0.10d0, 0.10d0, 0.10d0, 0.10d0, 0.10d0,
!     &     0.10d0, 0.10d0, 0.10d0, 0.10d0, 0.10d0/)
      end module h_imp_aerosol 