!---------------------
!Purpose: read in SOA and compute the heterogeneous IN numbers from it
!Created by Cheng Zhou, zhouc@umich.edu, 2014-07-23
!--------------

   module  um_SOA_IN_intr
   use shr_kind_mod,     only: r8=>shr_kind_r8
   use cam_history,      only: addfld, add_default, phys_decomp
   use ioFileMod,        only: getfil
   use pio
   use cam_pio_utils,    only : cam_pio_openfile
   use pio,              only : pio_closefile !JIALEI
   use phys_grid,        only : get_ncols_p, get_rlat_all_p, get_rlon_all_p
   use interpolate_data, only : lininterp_init, lininterp, lininterp_finish, interp_type
   use ppgrid,           only : begchunk, endchunk, pcols, pver, pverp
   use mo_constants,     only : pi, d2r
   use abortutils,       only : endrun
   use cam_history,      only : outfld		
   use cam_logfile,      only : iulog
   use time_manager,     only : get_curr_date
   use spmd_utils,       only : masterproc  		
   implicit none
   private
   save
   
   real(r8), allocatable :: var_in_4d(:,:,:,:),um_SOA_mass(:,:,:,:)  	! store SOA data
   real(r8), allocatable :: in_lons(:)
   real(r8), allocatable :: in_lats(:)
   character(256)        :: in_file_base="/glade/u/home/jialeiz/working_jialeiz/input_SOA/constsoa_T1.nc" !hard coded here for now. Change of this line is not necessary if you put the file in the run directory since getfil will always search  running directory first
   character(len=2)      :: mon_name(12)
   data mon_name/"01","02","03","04","05","06","07","08","09","10","11","12"/
    	  	
   real(r8):: pm_soa                   ! mass of one SOA IN particle
   real(r8):: re_soot = 0.0896*1.0e-6  ! Pueschel et al., 1992: rg=0.07um, sig=1.5, m
   real(r8):: rho_soa = 1.000*1.0e3    ! BC,OC,kg/m3
   	
   	   
   public  um_SOA_IN_init
   public  um_SOA_IN_num_cal
   
   contains
   
   subroutine um_SOA_IN_init () !called by phys_init in physpkg.F90, all chunks, each node
!----------------------------------------------------------------------- 
!
!  Purpose: declare history variables, initialize data sets
!  This subroutine is called at the beginning of an initial or restart run
!
!-----------------------------------------------------------------------

   implicit none
   
! Begin Execution 
   

                     
    !-----------------------------------------------------------------------
    !    from this subroutine
    !-----------------------------------------------------------------------      

        call addfld      ('um_SOA_IN', '#/cm-3', pver, 'I', 'UM SOA IN number', phys_decomp) 
        call add_default ('um_SOA_IN', 1, 'A') ! added to h0; ' ' is instant; 'A', average
        call add_default ('um_SOA_IN', 4, 'I') ! added to h0; ' ' is instant; 'A', average

!        call addfld      ('IN_SOA_toLP', '#/cm-3', pver, 'I', 'UM SOA acting glassy IN to LP scheme', phys_decomp) 
!        call add_default ('IN_SOA_toLP', 1, 'A') ! added to h0; ' ' is instant; 'A', average
!       ! call add_default ('IN_SOA_toLP', 4, 'I') ! added to h0; ' ' is instant; 'A', average

        call addfld      ('um_SOA_glassy_flag', '', pver, 'I', 'Glassy SOA IN flag. 1: yes, 0:no.', phys_decomp) 
        call add_default ('um_SOA_glassy_flag', 1, 'A') ! added to h0; ' ' is instant; 'A', averag
      !  call add_default ('um_SOA_glassy_flag', 4, 'I') ! added to h3; ' ' is instant; 'A', average
        
    !-- read in 12 months SOA fields    
      call  um_SOA_IN_read ()  !! read in SOA
      
    
    ! particle mass
      pm_soa  = 4.0_r8*3.141592653_r8/3.0_r8*(re_soot**3)*rho_soa ! kg/#
      if (masterproc) then 
      print*, "UM_IMPACT_SOA pm = ", pm_soa
      end if	
      
  end subroutine um_SOA_IN_init
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ !   
   subroutine  um_SOA_IN_read ()  !read in 12 monthly emission, read in all chunks, so should be called outside one chunk
   implicit none
!  input argument
   integer          :: mon,day! current month
  
! local variables
   integer            :: m,irec
   character(256)     :: in_file0,in_file
   integer            :: did, vid, nlat, nlon, nlev, nrec
   type(file_desc_t)  :: ncid
   type(interp_type)  :: lon_wgts, lat_wgts
   real(r8)           :: to_lats(pcols), to_lons(pcols)

   	
   integer             :: c, ncols, ierr
   real(r8), parameter :: zero=0._r8, twopi=2._r8*pi

!
    integer :: start(4)
    integer :: count(4)
    

   	   
 ! Get file name.
   in_file0 = trim(in_file_base)
   call getfil(in_file0, in_file, 0)
   call cam_pio_openfile (ncid, trim(in_file), PIO_NOWRITE)
   
   
! Get input data resolution.
   ierr = pio_inq_dimid( ncid, 'lon', did )
   ierr = pio_inq_dimlen( ncid, did, nlon )

   ierr = pio_inq_dimid( ncid, 'lat', did )
   ierr = pio_inq_dimlen( ncid, did, nlat )

   ierr = pio_inq_dimid( ncid, 'lev', did )
   ierr = pio_inq_dimlen( ncid, did, nlev )
   
   ierr = pio_inq_dimid( ncid, 'time', did)
   ierr = pio_inq_dimlen( ncid, did, nrec )   
 
   	
   if( .not. allocated(var_in_4d)) then !first time
   allocate(var_in_4d (nlon,nlat,nlev,nrec)) !! specific to the input file, nrec should be 12 and nlev = pver
   allocate(in_lons(nlon))
   allocate(in_lats(nlat))
   end if
   	
   ierr = pio_inq_varid( ncid, 'lon', vid )
   ierr = pio_get_var( ncid, vid, in_lons )

   ierr = pio_inq_varid( ncid, 'lat', vid )
   ierr = pio_get_var( ncid, vid, in_lats )

   start = (/ 1, 1, 1, 1 /)
   count = (/ nlon, nlat, nlev, nrec /)
   ierr = pio_inq_varid( ncid, 'um_ORGn3', vid ) !! mass_concentration_of_secondary_particulate_organic_matter_dry_aerosol_in_air, ug m-3
   ierr = pio_get_var( ncid, vid, start, count, var_in_4d )
 
   if (masterproc) then 
   print*, "Reading UM-IMPACT SOA fields, nlon, nlat,nlev,nrec:",nlon, nlat,nlev,nrec
   !!NOTE: nrec has to be 12,12 months,  and nlev has to be 30 same as CAM pver
   print*, "-----------lon_in-----------",in_lons
   print*, "-----------lat_in-----------",in_lats
   print*, "------constsoa(72,48,:,1)---",var_in_4d(72,48,:,1)
   end if
!------------------------------------------------------------------
!  ... close the netcdf file
!------------------------------------------------------------------
    call pio_closefile( ncid )
       	
! var_in_4d(:,:,:,:) = 2.0_r8 ! for test purpose
!-----------------------------------------------------------------------
!     	... convert to radians and setup regridding
!-----------------------------------------------------------------------
    in_lats(:) = d2r * in_lats(:)
    in_lons(:) = d2r * in_lons(:)

!--------------allocate emission arrays --------------    
    if( .not. allocated(um_SOA_mass)) then
    allocate( um_SOA_mass(begchunk:endchunk,pcols,pver,nrec) ) !! specific to the input file (begchunk:endchunk, pcols,pver,totmonth)
    end if 	
    um_SOA_mass(:,:,:,:) = 0.0_r8 	
!-----------------------------------------------------------------------
!     	... regrid ..
!-----------------------------------------------------------------------
    do c=begchunk,endchunk
       ncols = get_ncols_p(c)
       call get_rlat_all_p(c, pcols, to_lats)
       call get_rlon_all_p(c, pcols, to_lons)
       
       call lininterp_init(in_lons, nlon, to_lons, ncols, 2, lon_wgts, zero, twopi)
       call lininterp_init(in_lats, nlat, to_lats, ncols, 1, lat_wgts)
         
         do irec = 1, nrec
            do m = 1, nlev
             call lininterp(var_in_4d(:,:,m,irec), nlon,nlat , um_SOA_mass(c,1:ncols,m,irec), ncols, lon_wgts,lat_wgts)
          	end do
         end do  	

       call lininterp_finish(lat_wgts)
       call lininterp_finish(lon_wgts)
    end do

    
    deallocate( var_in_4d, stat=ierr )
!    if( ierr /= 0 ) then
!       write(iulog,*) 'um_readin_qj: failed',ierr
!       call endrun
!    end if
!   
!   um_SOA_mass(:,:,:,:) = 1.0_r8 	! for test purpose

   end subroutine  um_SOA_IN_read
   
  !============================================================================ !
  !                                                                             !
  !============================================================================ !  
   subroutine um_SOA_IN_num_cal(um_SOA_IN_num,relhum, t, rho, ncol,lchnk) ! one chunk
   implicit none
   
   integer  :: ncol,lchnk
   real(r8) :: um_SOA_IN_num(pcols,pver) !#/cm3
   real(r8) :: relhum(pcols,pver)  ! relative humidity
   real(r8) :: t(pcols,pver)         ! input temperature (K)
   real(r8) :: rho(pcols,pver)         ! input air density
  !local

   integer :: tod               ! time of day relative to current date [seconds]
   integer :: yr, mon, day      ! year, month, day components of a date
   integer :: curr_h            ! current hour
   integer :: i,k
   real (r8) :: glassy_flag(pcols,pver)         !SOA as glassy IN.  1: yes, 0: no glassy IN
   	
  !Begin
   		
   call get_curr_date(yr, mon, day, tod)
   
   !curr_h = INT(tod/3600) + 1
   !print*, 'mon_day =', mon*100 + day , 'tod=',tod
   
   !mon = mod ( curr_h, 12) + 1
   
   !print*, 'UM_IMPACT, mon',mon
  
   um_SOA_IN_num(:,:)         = 0.0_r8 ! initialization
   um_SOA_IN_num(:ncol,:pver) =  um_SOA_mass  (lchnk,:ncol,:pver, mon) * rho(:ncol,:pver) *1e-6 ! from kg/kg to #/cm3
!   um_SOA_IN_num(:ncol,:pver) =  um_SOA_mass  (lchnk,:ncol,:pver, mon) * 1e-9_r8 / pm_soa * 1e-6_r8  ! from ug/m3 to kg/m3 to #/m3 to #/cm3		! * 1e-9_r8 / pm_soa * 1e6_r8 = 3.31885e2  
 !  call  outfld('um_SOA_num', um_SOA_IN_num(:pcols,:pver) , pcols, lchnk)
   	   
   glassy_flag(:,:) = 0 ! initialization
   do k = 1, pver
      do i = 1, ncol
         call get_glassy_flag(relhum(i,k), t(i,k), glassy_flag(i,k))

         um_SOA_IN_num(i,k)=um_SOA_IN_num(i,k)* glassy_flag(i,k)

!         !! 0% of SOA acting as IN for all T, RH
!         um_SOA_IN_num(i,k) = 0.0_r8 

!         !! 0.1% of SOA acting as IN for all T, RH
!         um_SOA_IN_num(i,k) = 0.001_r8 * um_SOA_IN_num(i,k) 
!         um_SOA_IN_num(i,k) = min ( 1.0_r8, um_SOA_IN_num(i,k)) !further set upper limit
!         
         !! 1% of SOA acting as IN for all T, RH
       !   um_SOA_IN_num(i,k) = 0.01_r8 * um_SOA_IN_num(i,k) 
!         um_SOA_IN_num(i,k) = min ( 1.0_r8, um_SOA_IN_num(i,k)) !further set upper limit
           
!         !! 10% of SOA acting as IN for all T, RH
!         um_SOA_IN_num(i,k) = 0.1_r8 * um_SOA_IN_num(i,k)
!         um_SOA_IN_num(i,k) = min ( 1.0_r8, um_SOA_IN_num(i,k)) !further set upper limit
!                  
!         !! best esitmate , wang et al , plus 0.1% criteria
!         um_SOA_IN_num(i,k) = 0.01_r8 * um_SOA_IN_num(i,k) * glassy_flag(i,k) ! 1% SOA acting as IN, alos apply the glassy IN flag
!         um_SOA_IN_num(i,k) = min ( 1.0_r8, um_SOA_IN_num(i,k)) !further set upper limit
      end do     
   end do    	
   

   call  outfld('um_SOA_IN', um_SOA_IN_num(:pcols,:pver) , pcols, lchnk)
   
   call  outfld('um_SOA_glassy_flag', glassy_flag(:pcols,:pver) , pcols, lchnk)
      	 				
   end subroutine um_SOA_IN_num_cal


  !===============================================================
  !
  !================================================================
  subroutine  get_glassy_flag(relhum, t, glassy_flag)
  implicit none 
 
  real (r8) :: relhum, t ! relhum:[0-1+], t (K)
  real (r8) :: glassy_flag
  	
  !local, from Wang et al. 2012 to determine the transition temperature SOA to IN
  real (r8) ::     T_gw=136.0_r8
  real (r8) ::     kappa_org=0.14_r8
  real (r8) ::     rho_org=1.2_r8 !  the density of oragnic species
  real (r8) ::     rho_w= 1.0_r8   !  the density of water
             
!  the Grodon-Taylor constant for the specfic water/organic system, its value is 
  real (r8) ::     k_GT=2.5_r8  !%%% 3 scenarios% !    1.5, 2.5, or 3.5 according to Fig.9 in Koop et al. (2011)
  real (r8) ::     T_gdry= 270.0_r8 ! %%% 3 scenarios% ! the T_g at RH=0, its value is 291,270, or 249 according to Fig.9 in Koop et al. (2011)

!  real (r8) ::     k_GT=3.5_r8  !%%% 3 scenarios% !    1.5, 2.5, or 3.5 according to Fig.9 in Koop et al. (2011)
!  real (r8) ::     T_gdry= 249.0_r8 ! %%% 3 scenarios% ! the T_g at RH=0, its value is 291,270, or 249 according to Fig.9 in Koop et al. (2011)
 
 real (r8) ::     RH, f_rh, T_g
!begin

!  RH: relative humidity *100 (e.g. if 100% then RH=100)       
        RH = relhum * 100.0_r8
       f_rh=(100.0_r8-RH)/(RH)*(1.0_r8/kappa_org)*(rho_org)/(rho_w)

!  T_g: the glass transition temperature       
       T_g=(T_gw*k_GT+ f_rh*T_gdry)/(k_GT+f_rh) 
       
       
       
!  determine if the environment T is less then the transition temperature
       if (t .lt. T_g) then 
      	  glassy_flag = 1.0_r8
     		 else
     		  glassy_flag = 0.0_r8
       end if

! also let flag be 1 for all T< 205k
!      if ( t .lt. 205.0_r8) then
!         glassy_flag = 1.0_r8
!      end if

  
  end subroutine get_glassy_flag
  

  end module um_SOA_IN_intr
  
