  module um_aerosol_intr

  !-------------------------------------------------------------------------------------------------------
  ! Purpose:
  !
  ! Provides the interface to the U of M aerosol module
  !
  ! Author: Cheng Zhou, Aug. 2012,Zhouc@umich.edu
  !-------------------------------------------------------------------------------------------------------

  use shr_kind_mod,  only: r8=>shr_kind_r8
  use ppgrid,        only: pver, pcols
  use abortutils,    only: endrun
  use cam_logfile,   only: iulog
  use spmd_utils,    only: masterproc,iam
!----  	
  use h_setkin_par !,  only: NSP,NACT,IO3,IOH,IHO2,INO3,IMGAS !NACT=NSP-5 constant species
  use h_imp_dims,    only: num_emiss,num_qjs,num_qks !emiss_map is set in h_init
  	
  implicit none
  private
  save

  public :: um_aerosol_register 
  public ::	um_aerosol_init
  public :: um_aerosol_step
  public :: um_get_spc_ndx !in const
  public :: um_get_spc_ndx_in_q ! in state q

  character(len=16), dimension(NSP), parameter :: cnst_names=(/&
  'H2O2            ',&   !
  'fSO2            ',&   !
  'nSO2            ',&   !
  'nDMS            ',&   !
  'SO4g            ',&   !
  'SO4m1           ',&   !
  'SO4n1           ',&   !
  'SO4m2           ',&   !
  'SO4n2           ',&   !
  'SO4m3           ',&   !
  'SO4n3           ',&   !
  'SO4nOC          ',&   !
  'SO4fOC          ',&   !
  'SO4fBC          ',&   !
  'SO4bOC          ',&   !
  'SO4bBC          ',&   !
  'SO4OC1      ',&   !     
  'SO4OC2      ',&   !     
  'SO4BC1      ',&   !     
  'SO4BC2      ',&   !     
  'SO4cPOM         ',&   !       
  'SO4cBC          ',&   !        
  'SO4aPOM         ',&   !       
  'SO4aBC          ',&   !        
  'SO4d1           ',&   !
  'SO4d2           ',&   !
  'SO4d3           ',&   !
  'SO4d4           ',&   !
  'SO4s1           ',&   !
  'SO4s2           ',&   !
  'SO4s3           ',&   !
  'SO4s4           ',&   !
  'nOC             ',&   ! All OC are actually OM, a factor of 1.4 has been applied to the oc emissions.
  'fOC             ',&   !
  'fBC             ',&   !
  'bOC             ',&   !
  'bBC             ',&   !
  'fOC1         ',&   !        
  'fOC2         ',&   !        
  'fBC1         ',&   !        
  'fBC2         ',&   !         
  'cPOM            ',&   !           
  'cBC             ',&   !           
  'aPOM            ',&   !           
  'aBC             ',&   !             
  'dust1           ',&   !
  'dust2           ',&   !
  'dust3           ',&   !
  'dust4           ',&   !
  'sslt1           ',&   !
  'sslt2           ',&   !
  'sslt3           ',&   !
  'sslt4           ',&   !
  'aSO2            ',&   !           
  'aSO4            ',&   !    
  'aH2O            ',&   !          
  'O3              ',&   !
  'OH              ',&   ! 
  'HO2             ',&   !
  'NO3             ',&   !
  'ad              '&   ! air density in #/cm^3
  /) 	 
  

   integer, dimension(NACT) :: cnst_idx_q  ! index of advective constituent in state%q  
  	
   real(r8)  :: dtime !  step  size of CAM in seconds.
   
   
 
!---- below are for the input arguments of the IMPACT box model -----------
   logical  :: inv_o3, inv_oh, inv_no3, inv_ho2
   	
!   !2D in the box model , just 1d in CAM5, 1st dimension is set to 1:1 in the box model.
!   real(r8), dimension(pcols) :: latdeg, londeg,                                   mcor, grid_height !mcor, grid_height are not used 
!   
!   !3D in the box model , just 2d in CAM5  
!   real(r8), dimension(pcols,pver) ::  kel,humidity,IC_tot_liq,mass,cld,press3c,   moistq,cmf        ! moistq,cmf are not used
! 
!   real(r8), dimension(pcols,pver,num_emiss) ::  semiss  ! emission of fSO2, nSO2, DMS, #/cm^3, num_emiss=3
!   real(r8), dimension(pcols,pver,num_qjs)   ::	qjimp   ! qjimp = 1
!   real(r8), dimension(pcols,pver,num_qjs)   ::	qkimp 	 ! qkimp = 8  , this is not used actually. 	
!   real(r8), dimension(pcols,pver,NSP)       ::  const
!   	
!   !output from the box model	
!   real(r8), dimension(pcols,pver) :: dms_oh, dms_no3, so2_oh, so2_h2o2, so2_o3, so2_hete
   	
   !2D in the box model , just 1d in CAM5, 1st dimension is set to 1:1 in the box model.
   real(r8), dimension(:),   allocatable :: latdeg, londeg,                              mcor!mcor, are not used 
   
   !3D in the box model , just 2d in CAM5  
   real(r8), dimension(:,:), allocatable ::  kel,humidity,IC_tot_liq,mass,cld,press3c,   moistq,cmf , grid_height,ispb       ! moistq,cmf are not used
 
   real(r8), dimension(:,:,:), allocatable   :: semiss  ! emission of fSO2, nSO2, DMS, #/cm^3, num_emiss=3
   real(r8), dimension(:,:  ), allocatable   ::	qjimp   ! qjimp = 1
   real(r8), dimension(:,:,:), allocatable   ::	qkimp 	 ! qkimp = 8  , this is not used actually.   	
   real(r8), dimension(:,:,:), allocatable   :: const
   	
   !output from the box model	
   real(r8), dimension(:,:), allocatable :: dms_oh, dms_no3, so2_oh, so2_h2o2, so2_o3, so2_hete	   	
!--------------------------------------------------------------

! ------- some other additional  fields for large scale preciptitation deposition
   real(r8), dimension(:),   allocatable :: con_precip, tot_precip
   real(r8), dimension(:,:), allocatable :: press3e
! ------- end some other additional  fields for large scale preciptitation deposition

! ------- some other additional  fields for convective preciptitation deposition
   real(r8), dimension(:),   allocatable :: pblh    
   real(r8)  :: lwi_flags (pcols)

! ------- field for dry deposition
   real(r8)  :: ustar (pcols)	

 !----------- emission arrays ---------------------	
  real(r8), allocatable,public :: um_emis_srf(:,:,:), &  !surface  emission  dimension is (begchunk:endchunk, pcols,ncnst_tot)
                                	um_emis_ver(:,:,:,:)   !elevated emission  dimension is (begchunk:endchunk, pcols,pver,ncnst_tot)
  integer :: astat
  logical, public :: um_emis_srf_has(NACT) = .false., &
  	                 um_emis_ver_has(NACT) = .false.
 !----------- end emission arrays ---------------------	


 !--- dust coating -----------------
  real(r8), allocatable,public :: dust_so4_layer(:,:,:,:) 	!(begchunk:endchunk,pcols,pver,4)
    	
  contains

  ! ===============================================================================

  subroutine um_aerosol_register !called by phys_register,before phys decomposition
  !---------------------------------------------------------------------- !
  !                                                                       !
  ! Register the constituents and the fields                              !
  ! in state%q                                                 !
  !                                                                       !
  !---------------------------------------------------------------------- !
    use constituents, only: cnst_add
    use physconst,    only: mwdry, cpair
    use physics_buffer, only : pbuf_add_field, dtype_r8
    	
    implicit none

  !---------------------------Local workspace-----------------------------
    integer :: m, temp_idx

  !------------------------------------------------------------------------
  
	!! add all advective constituents to state%q 
	 do m = 1, NACT
     call cnst_add('um_'//cnst_names(m), mwdry, cpair, 0._r8, temp_idx,mixtype='dry')
     cnst_idx_q (m) = temp_idx
	 end do
	 
	 
	     if (masterproc) then
       !-----------------------------------------------------------------------
       !            output what has been added to state%q to be advected
       !-----------------------------------------------------------------------
         write(iulog,*) '------------- Constituents added by um_aerosol_driver to state%q -----'
         write(iulog,'(A4,A15,A6)') '-#- ', '--const_name--',' q_idx'
         do m = 1, NACT
         write(iulog,'(I4,A,A15,I6)') m,': ', 'um_'//cnst_names(m), cnst_idx_q (m) 
	       end do
	      end if
	      	
	 !to store DCQ from cam_diagnostics.F90
	 call pbuf_add_field('DCQ','physpkg',dtype_r8,(/pcols,pver/),temp_idx) !  (kg/kg/s) moist q change due to precipitation processes.  in cam_diagnostics.F90

	 !to store cloud mass flux in zm_conv_intr 
	 call pbuf_add_field('ZMMU','physpkg',dtype_r8,(/pcols,pver/),temp_idx) !upward cloud mass flux (kg/m^2/s) in zm_conv_intr.F90

	 !to store ustar in vertical_diffusion.F90
	 call pbuf_add_field('USTAR','physpkg',dtype_r8,(/pcols/),temp_idx) ! 	Surface friction velocity
	   	
	end subroutine um_aerosol_register
 
  !============================================================================ !
  !                                                                             !
  !============================================================================ !
 
  subroutine um_aerosol_init (phys_state) !called by phys_init, all chunks, each node
!----------------------------------------------------------------------- 
!
! Purpose: declare history variables, initialize data sets
!  This subroutine is called at the beginning of an initial or restart run
!
!-----------------------------------------------------------------------

   use cam_history,     only: addfld, add_default, phys_decomp
   use ppgrid,          only: begchunk, endchunk
   use physics_types,   only: physics_state
   use um_readin,       only: um_readin_init
   use um_read_aircraft_intr, only: um_readin_aircraft
   implicit none
   
   type(physics_state), intent(inout), dimension(begchunk:endchunk) :: phys_state 
   	
   ! Local Variables
   integer m,k
   integer lchnk



   ! Begin Execution 
    
    !-----------------------------------------------------------------------
    !     set dimensions and input parameters in imp_dims, imp_control
    !----------------------------------------------------------------------- 
      call h_init()
 
    !-----------------------------------------------------------------------
    !      Add UM aerosol fields to the master list, it is a must not optional
    !-----------------------------------------------------------------------    
      do m = 1,NACT
       call addfld      ('um_'//cnst_names(m), 'kg/kg', pver, 'A', cnst_names(m), phys_decomp) 
       call add_default ('um_'//cnst_names(m), 1, 'A') ! added to first history file, h0; ' ' is instant; 'A', average
       call add_default ('um_'//cnst_names(m), 3, 'A') ! added to history file, h1
      end do
     
 
    !-----------------------------------------------------------------------
    !      Allocate the arrays used for emissions
    !----------------------------------------------------------------------- 
    allocate( um_emis_srf(begchunk:endchunk, pcols,     NACT), stat=astat )  
       if( astat/= 0 ) then
        call endrun('Failed to allocate um_emis_srf')
       end if
    allocate( um_emis_ver(begchunk:endchunk, pcols,pver,NACT), stat=astat )
       if( astat/= 0 ) then
        call endrun('Failed to allocate um_emis_ver')
       end if
    um_emis_srf = 0.0_r8
    um_emis_ver = 0.0_r8
    um_emis_srf_has = .FALSE.
    um_emis_ver_has = .FALSE.

     do m = 1,NACT
!      call addfld      ('emis_srf_'//trim(cnst_names(m)), '1e3*mole/m2/s', 1, 'A', 'CAM5 emission, simply times mw (g) to get kg/m2/s', phys_decomp) ! must less than 16 chars
      call addfld      ('emis_srf_'//trim(cnst_names(m)), '#/cm2/s for sulf related, kg/m2/s for others', 1, 'A', 'emis_srf_'//trim(cnst_names(m)), phys_decomp) ! must less than 16 chars
      call add_default ('emis_srf_'//trim(cnst_names(m)), 3, 'A')
      call add_default ('emis_srf_'//trim(cnst_names(m)), 1, 'A')
      
      call addfld      ('emis_ver_'//trim(cnst_names(m)), '#/cm3/s ', pver, 'A', 'emis_ver_'//trim(cnst_names(m)), phys_decomp) ! must less than 16 chars
      call add_default ('emis_ver_'//trim(cnst_names(m)), 3, 'A')
      call add_default ('emis_ver_'//trim(cnst_names(m)), 1, 'A')
       
      call addfld      ('um_wetdep1_'//trim(cnst_names(m)), 'kg/m2/s', 1, 'A', 'in convective updrafts :'//trim(cnst_names(m)), phys_decomp) ! must less than 16 chars
      call add_default ('um_wetdep1_'//trim(cnst_names(m)), 3, 'A')
      call add_default ('um_wetdep1_'//trim(cnst_names(m)), 1, 'A')

      call addfld      ('um_wetdep2_'//trim(cnst_names(m)), 'kg/m2/s', 1, 'A', 'large + shallow convec precip :'//trim(cnst_names(m)), phys_decomp) ! must less than 16 chars
      call add_default ('um_wetdep2_'//trim(cnst_names(m)), 3, 'A')
      call add_default ('um_wetdep2_'//trim(cnst_names(m)), 1, 'A')

      call addfld      ('um_drydep_'//trim(cnst_names(m)), 'kg/m2/s', 1, 'A', 'gravity + dry:'//trim(cnst_names(m)), phys_decomp) ! must less than 16 chars
      call add_default ('um_drydep_'//trim(cnst_names(m)), 3, 'A')
      call add_default ('um_drydep_'//trim(cnst_names(m)), 1, 'A')
                       
     end do


    !-----------------------------------------------------------------------
    !      temporary fields for test purpose
    !----------------------------------------------------------------------- 
        call addfld      ('um_VAR3D_01', 'unit', pver, 'A', 'test 3D_01', phys_decomp) 
        call add_default ('um_VAR3D_01', 2, 'A') ! added to h1; ' ' is instant; 'A', average
        call addfld      ('um_VAR3D_02', 'unit', pver, 'A', 'test 3D_02', phys_decomp) 
        call add_default ('um_VAR3D_02', 2, 'A') ! added to h1; ' ' is instant; 'A', average
        call addfld      ('um_VAR3D_03', 'unit', pver, 'A', 'test 3D_03', phys_decomp) 
        call add_default ('um_VAR3D_03', 2, 'A') ! added to h1; ' ' is instant; 'A', average
        call addfld      ('um_VAR3D_04', 'unit', pver, 'A', 'test 3D_04', phys_decomp) 
        call add_default ('um_VAR3D_04', 2, 'A') ! added to h1; ' ' is instant; 'A', average

        call addfld      ('um_VAR2D_01', 'unit', 1, 'A', 'test 2D_01', phys_decomp) 
        call add_default ('um_VAR2D_01', 2, 'A') ! added to h1; ' ' is instant; 'A', average
        call addfld      ('um_VAR2D_02', 'unit', 1, 'A', 'test 2D_02', phys_decomp) 
        call add_default ('um_VAR2D_02', 2, 'A') ! added to h1; ' ' is instant; 'A', average
        

    !-----------------------------------------------------------------------
    !     for IMPACT box model required input arguments 
    !----------------------------------------------------------------------- 
        call addfld      ('um_lat', 'degree', 1, 'I', 'latdeg ', phys_decomp) 
        call add_default ('um_lat', 2, 'A') ! added to h1; ' ' is instant; 'A', average

        call addfld      ('um_lon', 'degree', 1, 'I', 'londeg ', phys_decomp) 
        call add_default ('um_lon', 2, 'A') ! added to h1; ' ' is instant; 'A', average
                
        call addfld      ('um_kel', 'K', pver, 'A', 'Temperature', phys_decomp) 
        call add_default ('um_kel', 2, 'A') ! added to h1; ' ' is instant; 'A', average
             
        call addfld      ('um_humidity', 'g/kg', pver, 'A', 'Specific humidity', phys_decomp) 
        call add_default ('um_humidity', 2, 'A') ! added to h1; ' ' is instant; 'A', average
        
        call addfld      ('um_cld', '1', pver, 'A', 'total cloud fraction', phys_decomp) 
        call add_default ('um_cld', 2, 'A') ! added to h1; ' ' is instant; 'A', average       
 
 
!        call addfld      ('um_clwc', 'kg/kg', pver, 'A', 'total IC liquid water content (ICLMRTOT)', phys_decomp) 
!        call add_default ('um_clwc', 2, 'A') ! added to h1; ' ' is instant; 'A', average              
        
        call addfld      ('um_mass', 'kg/m^2/layer', pver, 'A', 'dry air mass of each layer', phys_decomp) 
        call add_default ('um_mass', 2, 'A') ! added to h1; ' ' is instant; 'A', average         
        call add_default ('um_mass', 1, 'A') ! added to h1; ' ' is instant; 'A', average           
       
        call addfld      ('um_O3', 'vmr', pver, 'A', 'O3 (volume mixing ratio)', phys_decomp) 
        call add_default ('um_O3', 2, 'A') ! added to h1; ' ' is instant; 'A', average     
        
        call addfld      ('um_OH', 'vmr', pver, 'A', 'OH (volume mixing ratio)', phys_decomp) 
        call add_default ('um_OH', 2, 'A') ! added to h1; ' ' is instant; 'A', average     
        
        call addfld      ('um_HO2', 'vmr', pver, 'A', 'HO2 (volume mixing ratio)', phys_decomp) 
        call add_default ('um_HO2', 2, 'A') ! added to h1; ' ' is instant; 'A', average             

        call addfld      ('um_NO3', 'vmr', pver, 'A', 'NO3 (volume mixing ratio)', phys_decomp) 
        call add_default ('um_NO3', 2, 'A') ! added to h1; ' ' is instant; 'A', average  

        call addfld      ('um_press3c', 'hPa', pver, 'A', 'pressure at the center of each layer', phys_decomp) 
        call add_default ('um_press3c', 2, 'A') ! added to h1; ' ' is instant; 'A', average                


        call addfld      ('um_IC_tot_liq', 'kg/kg', pver, 'A', 'IC liquid water (stratus clouds only) ', phys_decomp) 
        call add_default ('um_IC_tot_liq', 2, 'A') ! added to h1; ' ' is instant; 'A', average  
        

    !-----------------------------------------------------------------------
    !     for IMPACT  deposition use
    !-----------------------------------------------------------------------      
        call addfld      ('um_con_precip', 'm/s', 1, 'A', 'convective precip', phys_decomp) 
        call add_default ('um_con_precip', 2, 'A') ! added to h1; ' ' is instant; 'A', average

        call addfld      ('um_tot_precip', 'm/s', 1, 'A', 'total precip', phys_decomp) 
        call add_default ('um_tot_precip', 2, 'A') ! added to h1; ' ' is instant; 'A', average
                     
        call addfld      ('um_moistq', 'kg/kg/s', pver, 'A', 'Specific humidity due to moist process', phys_decomp) 
        call add_default ('um_moistq', 2, 'A') ! added to h1; ' ' is instant; 'A', average
        
        call addfld      ('um_zmmu', 'kg/m2/s', pver, 'A', 'Upward cloud mass flux from zm_conv_intr', phys_decomp) 
        call add_default ('um_zmmu', 2, 'A') ! added to h1; ' ' is instant; 'A', average
    
        call addfld      ('um_pblh', 'm', 1, 'A', 'boundary layer height', phys_decomp) 
        call add_default ('um_pblh', 2, 'A') ! added to h1; ' ' is instant; 'A', average
        
        call addfld( 'um_USTAR'       , 'm/s'    , 1      , 'A', 'Surface friction velocity'                         , phys_decomp )
        call add_default ('um_USTAR', 2, 'A') !  'Surface friction velocity from vertical_diffusion.F90   
        
        call addfld( 'um_U10'       , 'm/s'    , 1      , 'A', '10 meter wind speed'                         , phys_decomp )
        call add_default ('um_U10', 2, 'A') !  from cam/cam_diagnostics.F90:1330:    call outfld('U10',      cam_in%u10,       pcols, lchnk)
                     
    !-----------------------------------------------------------------------
    !     for IMPACT box model output 
    !-----------------------------------------------------------------------      
        call addfld      ('um_dms_oh', 'kg/m2/layer/step', pver, 'A', 'SO2 from dms + no3, SO2 mole weight is 64.06', phys_decomp) 
        call add_default ('um_dms_oh', 1, 'A') ! added to h1; ' ' is instant; 'A', average
        
        call addfld      ('um_dms_no3', 'kg/m2/layer/step', pver, 'A', 'SO2 from dms + oh, SO2 mole weight is 64.06', phys_decomp) 
        call add_default ('um_dms_no3', 1, 'A') ! added to h1; ' ' is instant; 'A', average
        
        call addfld      ('um_so2_oh', 'kg/m2/layer/step', pver, 'A', 'SO4 from SO2 + oh, SO4 moleculer weight is 98.08', phys_decomp) 
        call add_default ('um_so2_oh', 1, 'A') ! added to h1; ' ' is instant; 'A', average
        
        call addfld      ('um_so2_h2o2', 'kg/m2/layer/step', pver, 'A', 'SO4 from SO2  + h2o2, SO4 moleculer weight is 98.08', phys_decomp) 
        call add_default ('um_so2_h2o2', 1, 'A') ! added to h1; ' ' is instant; 'A', average
        
        call addfld      ('um_so2_o3', 'kg/m2/layer/step', pver, 'A', 'SO4 from SO2  + o3, SO4 moleculer weight is 98.08', phys_decomp) 
        call add_default ('um_so2_o3', 1, 'A') ! added to h1; ' ' is instant; 'A', average
        
        call addfld      ('um_so2_hete', 'kg/m2/layer/step', pver, 'A', 'SO4 from hetero aqua reaction, SO4 moleculer weight is 98.08', phys_decomp) 
        call add_default ('um_so2_hete', 1, 'A') ! added to h1; ' ' is instant; 'A', average
        
        call addfld      ('um_hox_scaler', '1', 1, 'I', 'HOX diurnal variation scaler', phys_decomp) 
        call add_default ('um_hox_scaler', 3, 'A') ! added to h1; ' ' is instant; 'A', average
        
        call addfld      ('um_nox_scaler', '1', 1, 'I', 'NOX diurnal variation scaler', phys_decomp) 
        call add_default ('um_nox_scaler', 3, 'A') ! added to h1; ' ' is instant; 'A', average
        
        call add_default ('PS', 3, 'A')
  
        call addfld      ('contrail_flag', '1', pver, 'I', 'Schmidt-Appleman criteria, 1:contrail forms; 0: no contrail.', phys_decomp) 
        call add_default ('contrail_flag', 3, 'A') ! added to h1; ' ' is instant; 'A', average
         
  
    !-----------------------------------------------------------------------
    !   Add dsut coating related here
    !-----------------------------------------------------------------------   

        call addfld      ('dust1_so4', '1', pver, 'I', 'Sulfate coating layers on dust1', phys_decomp) 
!        call add_default ('dust1_so4', 4, 'I')
        call add_default ('dust1_so4', 1, 'A')

        call addfld      ('dust2_so4', '1', pver, 'I', 'Sulfate coating layers on dust2', phys_decomp) 
!        call add_default ('dust2_so4', 4, 'I')
        call add_default ('dust2_so4', 1, 'A')

        call addfld      ('dust3_so4', '1', pver, 'I', 'Sulfate coating layers on dust3', phys_decomp) 
 !       call add_default ('dust3_so4', 4, 'I')
        call add_default ('dust3_so4', 1, 'A')                

        call addfld      ('dust4_so4', '1', pver, 'I', 'Sulfate coating layers on dust4', phys_decomp) 
 !       call add_default ('dust4_so4', 4, 'I')
        call add_default ('dust4_so4', 1, 'A')

    !--------------allocate--------------    
    if( .not. allocated(dust_so4_layer)) then
    allocate( dust_so4_layer(begchunk:endchunk,pcols,pver,4) ) !! specific to the input file (begchunk:endchunk, pcols,pver,4 types)
    end if 	
    dust_so4_layer=0.0_r8      
    	          
    	          
    	              
  ! initialize input files
         call um_readin_init      ! land, vegtabtion types
         
         call um_readin_aircraft  ! aircraft emissions           
         
  end subroutine um_aerosol_init
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ !
 
   subroutine um_aerosol_step (state,pbuf,cam_in,fsns) !called by , 1 chunk
!----------------------------------------------------------------------- 
!
! Purpose:  
!
!-----------------------------------------------------------------------

   use cam_history,     only: phys_decomp,outfld
   use physics_types,   only: physics_state
   use physics_buffer,  only: physics_buffer_desc
   use time_manager,    only: get_step_size
   use camsrfexch,      only: cam_in_t
   	
   implicit none
   !input arguments
   type(physics_state), intent(inout) :: state
   type(physics_buffer_desc), pointer :: pbuf(:)	
   type(cam_in_t), target, intent(in) :: cam_in  ! import state	
   real(r8), intent(in) :: fsns(pcols)                   ! Surface solar absorbed flux
   
   ! Local
   integer lchnk,ncol
     
   real(r8)  :: chem_cycle ! number of time steps to cycle chemistry calls on, it appears imp_control.h, should already 1. 
   integer   :: tdt  
   real(r8), dimension(pcols,pver) :: tmp
   		
!--------------------------------------------------------------
 
!  Begin
   		
   	lchnk = state%lchnk
    ncol  = state%ncol


! some parameter
      tdt =  get_step_size()
      dtime =  tdt ! from integer to real
      chem_cycle  = 1.0d0
      
      
!------------------- do not change order const array related part ----      
!  prepare fields   
    call um_allocate_fld (ncol) ! allocate arrays used in both um_met_fld_prep  and um_const_prep
    call um_met_fld_prep (state,pbuf) !! set j2=ncol inside this subroutine
    call um_const_prep   (state,pbuf)
 
!  1 step forward, do gas/aqua phase reaction, do aerosol microphysics

            
       dms_oh   (:,:)=0.0_r8 ! unit is kg/m2/layer/step
       dms_no3  (:,:)=0.0_r8
       so2_oh   (:,:)=0.0_r8
       so2_h2o2 (:,:)=0.0_r8
       so2_o3   (:,:)=0.0_r8
       so2_hete (:,:)=0.0_r8
      	
      !call outfld('um_VAR3D_01', const(:ncol,:,ISO4N3) , ncol, lchnk)
        		
      call Update_Chem (dtime, chem_cycle, londeg, latdeg, kel, const, humidity, semiss, mcor, qjimp, qkimp, moistq, cmf, press3c, grid_height, ispb, mass, dms_oh, dms_no3, so2_oh, so2_h2o2, so2_o3, so2_hete,IC_tot_liq,cld,ncol,lchnk) 
       
      !call outfld('um_VAR3D_02', const(:ncol,:,ISO4N3) , ncol, lchnk)
      	 
! output the production rate   
      call outfld('um_dms_oh'  , dms_oh   (:ncol,pver:1:-1) , ncol, lchnk) ! unit is kg/m2/layer/step
      call outfld('um_dms_no3' , dms_no3  (:ncol,pver:1:-1) , ncol, lchnk)
      call outfld('um_so2_oh'  , so2_oh   (:ncol,pver:1:-1) , ncol, lchnk)
      call outfld('um_so2_h2o2', so2_h2o2 (:ncol,pver:1:-1) , ncol, lchnk)
      call outfld('um_so2_o3'  , so2_o3   (:ncol,pver:1:-1) , ncol, lchnk)
      call outfld('um_so2_hete', so2_hete (:ncol,pver:1:-1) , ncol, lchnk)
! end chem / aerosol microphysics       

!  Now do wet deposition       
       call um_wetdepos_intr(state,cam_in)

!  Now do gravitational settling and dry deposition
       call um_grav_drydep_intr(state,cam_in,fsns)
       
       
!       call um_depos_stub(state)

!  update state%q
    call um_const_2_q(state)
    call um_deallocate_fld()     
!------------------- do not change order above ----  
 
 
 ! update non sulf related surf emission, sulf related are done in the box model., does not need const
    call um_update_emis(state)    
      
      
      

       		
  
  end subroutine um_aerosol_step
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ !    
  
 
   subroutine um_met_fld_prep (state,pbuf) !called by , 1 chunk
!----------------------------------------------------------------------- 
!
! Purpose: get necessary met field for IMPACT model
!
!-----------------------------------------------------------------------

   use cam_history,     only: phys_decomp,outfld
   use physics_types,   only: physics_state
   use physics_buffer,  only: physics_buffer_desc, pbuf_get_field, pbuf_get_index  
   use physconst,       only: pi,gravit
   use time_manager,    only: get_nstep, get_curr_date
   use constituents,    only: cnst_get_ind
	
!----   
   use h_imp_control,   only: nhms, nymd, num_time_steps	
   use h_imp_dims,      only: j2,j2_gl,jhi,ilat,itloop	
   use um_readin,       only: um_get_qj	
   implicit none
   
   type(physics_state), intent(in) :: state
   type(physics_buffer_desc), pointer :: pbuf(:)	
   	
   ! Local
   integer m,idx, ixcldliq
   integer lchnk,ncol

   real(r8), dimension(pcols,pver)   :: tmp3d
   real(r8), dimension(pcols,pver+1) :: tmp3d1
   real(r8), pointer :: tmp2d_ptr(:), tmp3d_ptr(:,:)  !  used to read variable in pbuf, it has be to pointer type.

   ! for IMPACT nymd, nhms
   integer :: tod               ! time of day relative to current date [seconds]
   integer :: yr, mon, day      ! year, month, day components of a date
   	
!--------------------------------------------------------------
 
!  Begin
   		
   	lchnk = state%lchnk
    ncol  = state%ncol


!---- set end subscripts used in the IMPACT box model -------      	
! have to assign them to ncol for each chunk,starting indices are 1 assigned in h_init      
      j2    =  ncol !from CAM
      j2_gl =  ncol !from CAM
      jhi   =  ncol !from CAM
      
!     ilong = 1      ! number of longitudes
      ilat = ncol   ! number of latitudes
!     ivert= k2 - k1 + 1  ! number of vertical layers
!     itloop = ilat * ilong * ivert
      itloop = ncol * pver    

!----set time information in h_imp_control, nhms, nymd, num_time_steps
!       nhms = 10000           ! current hour/min/sec   (HHMMSS)
!       nymd = 20101           ! current year/month/day (YYMMDD)     
       call get_curr_date(yr, mon, day, tod)
       nymd =   MOD(yr,100)*10000   +                     mon*100 + day
       nhms = INT(tod/3600)*10000   +   INT(MOD(tod,3600)/60)*100 + MOD(tod,60)

       num_time_steps = get_nstep()    ! number of current time steps, used to determine wheteher to call aqua phase reaction.      
       
!       if (masterproc) then 
!       	  print*, 'calling um_met_fld_prep, nstep, nymd,nhms,dtime', num_time_steps, nymd, nhms, dtime
!       end if
!-----------------------------  

    
! get lat and lon. In IMPACT lat [-90,90], lon [0,360], while in CAM lat[-pi/2, pi/2], lon[0,2pi]
      latdeg = state%lat/pi*180.0_r8  ! convert to degree for IMPACT
      londeg = state%lon/pi*180.0_r8  ! convert to degree for IMPACT
      call outfld('um_lat', latdeg, pcols, lchnk)  
      call outfld('um_lon', londeg, pcols, lchnk)  

! get kel
      tmp3d = state%t(:,:)
      call outfld('um_kel', tmp3d, pcols, lchnk) 
      kel = tmp3d(:ncol,pver:1:-1)

! get humidity, in CAM5 the unit is kg/kg, in IMPACT it is g/kg
      tmp3d=state%q(:,:,1)*1000.0_r8 
      call outfld('um_humidity', tmp3d, pcols, lchnk)   
      humidity=tmp3d(:ncol,pver:1:-1)   
 
! get total cloud fraction from pbuf, cld has be defined as pointer.
      idx = pbuf_get_index('CLD')   
      call pbuf_get_field(pbuf,idx,tmp3d_ptr)
      call outfld('um_cld', tmp3d_ptr , pcols, lchnk)
      cld = tmp3d_ptr(:ncol,pver:1:-1)
      
! get the mass of each layer, kg/m^2      
      tmp3d = state%pdeldry(:,:)/gravit ! convert from Pa to Kg
      call outfld('um_mass', tmp3d , pcols, lchnk)
      mass = tmp3d(:ncol,pver:1:-1)  !reverse direction, from top down to bottom up

! get the pressure at the center of each layer, use hPa for IMPACT      	
      tmp3d = state%pmid/100.0_r8
      call outfld('um_press3c', tmp3d , pcols, lchnk)
      press3c = tmp3d(:ncol,pver:1:-1)  !reverse direction, from top down to bottom up

! get the pressure at the edge of each layer, use hPa for IMPACT      	
      tmp3d1 = state%pint/100.0_r8
      press3e = tmp3d1(:ncol,pver+1:1:-1)  !reverse direction, from top down to bottom up
      	
! get clwc2, total in-cloud liquid water content  !! Maybe I should use cldw(:ncol,k) = state%q(:ncol,k,ixcldliq) + state%q(:ncol,k,ixcldice) as in chemistry.F90
      idx = pbuf_get_index('ICLWP')  ! is the total in cloud liquid water in each layer, calculated in micro_mg_cam.F90
      call pbuf_get_field(pbuf,idx,tmp3d_ptr)      
      tmp3d = tmp3d_ptr / (state%pdel / gravit) !convert from kg/box to kg/kg
      call outfld('um_IC_tot_liq', tmp3d , pcols, lchnk)
      IC_tot_liq = tmp3d(:ncol,pver:1:-1)

!      call cnst_get_ind('CLDLIQ', ixcldliq)
!      tmp3d = state%q(:ncol,:,ixcldliq) 
!      IC_tot_liq = tmp3d(:ncol,pver:1:-1)
!      IC_tot_liq = IC_tot_liq / max(0.001d0,cld) 
!      call outfld('um_IC_tot_liq', IC_tot_liq(:ncol,pver:1:-1) , pcols, lchnk)
      
      
! get  grid_height
       tmp3d(:,:)    = state%zi(:,1:pver) - state%zi(:,2:(pver+1))        ! used to cal sulf_emiss, not necessary anymore, semiss is prepared already.
!       call outfld('um_VAR3D_01', tmp3d(:,:) , pcols, lchnk)
       grid_height = tmp3d(:ncol,pver:1:-1) 	


! get Precipitation rates (multi-process), DO NOT change the addition order !!!
  !   precc(:ncol) = prec_dp(:ncol)  + prec_sh(:ncol)
  !   precl(:ncol) = prec_sed(:ncol) + prec_pcw(:ncol)
  !   prect(:ncol) = precc(:ncol)    + precl(:ncol)
     
        idx  = pbuf_get_index('PREC_DP') 
        call pbuf_get_field(pbuf, idx, tmp2d_ptr)
        con_precip (:ncol) =  tmp2d_ptr(:ncol)
        
        idx  = pbuf_get_index('PREC_SH') 
        call pbuf_get_field(pbuf, idx, tmp2d_ptr)
        tot_precip (:ncol) = con_precip(:ncol) +  tmp2d_ptr(:ncol) ! consider prec_sh as large scale precip since convection in shallow convection is done w/o wet deposition.
        
        idx = pbuf_get_index('PREC_SED')
        call pbuf_get_field(pbuf, idx, tmp2d_ptr)
        tot_precip (:ncol) = tot_precip(:ncol) +  tmp2d_ptr(:ncol)
        
        idx = pbuf_get_index('PREC_PCW')
        call pbuf_get_field(pbuf, idx, tmp2d_ptr)
        tot_precip (:ncol) = tot_precip (:ncol) +  tmp2d_ptr(:ncol)
        
        
        call outfld('um_con_precip', con_precip(:ncol) , ncol, lchnk)
        call outfld('um_tot_precip', tot_precip(:ncol) , ncol, lchnk)
 
        con_precip (:ncol) = con_precip (:ncol) * 86400 * 1000 ! convert from m/s to mm/day
  	    tot_precip (:ncol) = tot_precip (:ncol) * 86400 * 1000 ! convert from m/s to mm/day
  	    	
 ! get dcq or moist q, Q tendency due to moist processes (convective and large scale precipiation)
      idx = pbuf_get_index('DCQ')   
      call pbuf_get_field(pbuf,idx,tmp3d_ptr)
      call outfld('um_moistq', tmp3d_ptr , pcols, lchnk)
      moistq = tmp3d_ptr(:ncol,pver:1:-1) * 1000d0 * 86400d0 ! from kg/kg/s to g/kg/day  	
 
 
 ! get cmf, upward cloud mass flux
      idx = pbuf_get_index('ZMMU')   
      call pbuf_get_field(pbuf,idx,tmp3d_ptr)
      call outfld('um_zmmu', tmp3d_ptr , pcols, lchnk)
      cmf = tmp3d_ptr(:ncol,pver:1:-1)  
 
 ! get pblh, meter 
      idx = pbuf_get_index('pblh')  
      call pbuf_get_field(pbuf, idx,     tmp2d_ptr)
      call outfld('um_pblh', tmp2d_ptr , pcols, lchnk)
      pblh = tmp2d_ptr(:ncol)
 
 ! set ispb , boundary layer flag 1:yes, 0:no, used for pbl nucleation, NEED to take care of late. Now just 0
      ispb(:,:) = 0 ! 
      	
  ! get ustar 
      idx = pbuf_get_index('USTAR')  
      call pbuf_get_field(pbuf, idx,     tmp2d_ptr)
      call outfld('um_USTAR', tmp2d_ptr , pcols, lchnk)	  
      ustar(:ncol) = tmp2d_ptr(:ncol)

      	     	  	       	
 ! qjimp, need to prepare this late, it is modified in Do_Sulf_Solver,need to update everytime step.
!      qjimp   (:, :, :) = 0.134703703555355967d-04 ! for now just this fixed value
      call um_get_qj(qjimp,ncol,lchnk,mon)
      call outfld('um_VAR3D_02', qjimp(:ncol,:) , ncol, lchnk)	
 
 ! semiss, should be in the unit of molecules/cm^3/s, no conversion late, directly pass to box model
      semiss  (:, :, :) = 0.0_r8
 ! srf emission , bottom up : semiss, 1:fSO2, 2:nSO2,3:nDMS, unit of um_emis_srf is #/cm2/s
      semiss  (:ncol, 1, 1) = ( um_emis_srf(lchnk,:ncol,IFSO2)  + &
      	                     	 um_emis_srf(lchnk,:ncol,ISO4M2) + &
      	                     	 um_emis_srf(lchnk,:ncol,ISO4M3))/ &
      	                     	 (grid_height(:ncol,1)*100.0_r8) ! from #/cm2/s to #/cm3/s
      	                     	 	
      semiss  (:ncol, 1, 3) =  um_emis_srf(lchnk,:ncol,INDMS)/ &
      	                       (grid_height(:ncol,1)*100.0_r8) ! from #/cm2/s to #/cm3/s
   ! add vertical emission      
      semiss  (:ncol, :, 1) = semiss  (:ncol, :, 1) + &
      	                     ( um_emis_ver(lchnk,:ncol,:,IFSO2)  + &
      	                     	 um_emis_ver(lchnk,:ncol,:,ISO4M2) + &
      	                     	 um_emis_ver(lchnk,:ncol,:,ISO4M3)) ! unit is already #/cm3/s
     
      	       	
 ! set fixed fields, not used any more
      qkimp   (:, :, :) = 0.0_r8             !! NOT used
 !     moistq  (:,:)     = 0.0_r8             !! NOT used  in chem/aerosl microphysics.
 !     cmf     (:,:)     = 0.0_r8             !! (cmf is used to calculate cumulus cloud fraction, not used in Chemistry anymore)
      mcor    (:)       = 1.0_r8             ! used to cal sulf_emiss, not necessary anymore

 
 
! for test purpose, set fixed value for now
! kel     (:,:)=220.060368855794280d0
!  kel     (:,:)=270d0  
! humidity(:,:)=0.792036810889840126D-02
! press3c (:,:)=198.122577644023607d0 
! mass    (:,:)=18819232098577.9336d0 
! cld(:,:) =0.0_r8	 
 	      
  end subroutine um_met_fld_prep
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ !
   

   subroutine um_const_prep (state,pbuf) !called by , 1 chunk
!----------------------------------------------------------------------- 
!
! Purpose:  prepare the const array for IMPACT box model
!
!-----------------------------------------------------------------------

   use cam_history,     only: phys_decomp,outfld
   use physics_types,   only: physics_state
   use physics_buffer,  only: physics_buffer_desc, pbuf_get_field, pbuf_get_index  
   use tracer_cnst,     only: get_cnst_data 	
   use mo_chem_utls,    only: get_inv_ndx	
   use physconst,       only: pi,gravit

!----   	
   use h_setkin_depos,  only: aerosol ! aerosol indicator; 0: gas; >0, types of aerosol.
   use h_imp_control,   only: mw ! moleculer weight of species in const
   use h_sulfchem,      only: wtmair
   	
   implicit none
   
   type(physics_state), intent(in) :: state
   type(physics_buffer_desc), pointer :: pbuf(:)	
   	
   ! Local
   integer m,idx
   integer lchnk,ncol
   integer i,j
   
   real(r8), dimension(pcols,pver) :: tmp3d
   	
   real(r8), pointer :: tmp3d_ptr(:,:)  !  used to read variable in pbuf, it has be to pointer type.


   real(r8), dimension(NSP) :: const1d
!   data  const1d  /0.496745071007385904D-10,0.551737447528487521D-10,0.475345573874577676D-10,0.691627816376203819D-13,0.212300012039815046D-12,0.246129437715667878D-11,36645317251.6928177d0,0.120249837324739503D-09,7201252137.21691132d0,0.228986875810923717D-10,23646699.9731766172d0,0.551873377592743285D-12,0.507119217632026343D-12,0.203195324410685169D-12,0.283129808647081432D-12,0.609231563576435403D-13,0.298493633983102044D-13,0.875385726717217694D-14,0.230681425528531495D-14,0.466697235734812820D-18,0.768708752885728718D-13,0.939799988455469678D-14,0.307508215354187174D-14,0.157079354832878090D-16,0.897995150147888158D-12,0.745386746222780363D-12,0.315659888878634094D-12,0.387214425884492357D-12,0.962948802481420205D-13,0.222100591138171584D-12,0.167191411505181386D-11,0.181875677361016792D-11,0.265329529649887674D-14,0.491626408429170409D-12,0.189385696611154894D-11,0.221367556525885836D-11,0.550564645092384795D-13,0.915436899617816380D-07,0.183296073089610212D-12,0.970117107305934478D-12,0.471486117834506169D-13,0.652085827312838758D+19/ 

!--------------------------------------------------------------
 
!  Begin
   		
   	lchnk = state%lchnk
    ncol  = state%ncol
    

! copy advective species from state%q to const, reverse vertical direction
	 do m = 1, NACT
     tmp3d = state%q(:,:,cnst_idx_q (m))
     	
     	! convert from mmr to vmr for gases, in CAM5, all are mmr in state%q
      if (aerosol(m) .eq. 0) then
       const (:,:,m) = tmp3d(:ncol,pver:1:-1) * wtmair / mw(m) ! from mmr to vmr
      else
       const (:,:,m) = tmp3d(:ncol,pver:1:-1) 
      end if      		
	 end do

! to make the warnig in cmfca and convect_shallow go away, I apply 1e-10 to SO4n1, SO4n2, SO4n3 when put them in the state%q. 	 
	    const (:,:,ISO4N1) = const (:,:,ISO4N1) *1.0e10_r8 !
      const (:,:,ISO4N2) = const (:,:,ISO4N2) *1.0e10_r8 !
      const (:,:,ISO4N3) = const (:,:,ISO4N3) *1.0e10_r8 !
      	
!  start to prepare const array, O3, OH, HO2, NO3
!    get_cnst_data returns a copy of data, do not use pointer
    inv_o3 = get_inv_ndx('O3') > 0  
    if ( inv_o3 ) then    	
      call get_cnst_data( 'O3',  tmp3d,  pcols, lchnk, pbuf ) ! vmr , must get pcols, if ncols, the data are dis located.      
      call outfld('um_O3', tmp3d  , pcols, lchnk   )       
      const(:,:,IO3) = tmp3d(:ncol,pver:1:-1)
    else
    	call endrun('Failed to read O3 in um_met_fld_prep!')
    end if

    inv_oh = get_inv_ndx('OH') > 0  
    if ( inv_oh ) then    	
      call get_cnst_data( 'OH',  tmp3d,  pcols, lchnk, pbuf ) ! vmr , must get pcols, if ncols, the data are dis located.      
      call outfld('um_OH', tmp3d  , pcols, lchnk   )    
      const(:,:,IOH) = tmp3d(:ncol,pver:1:-1)   
    else
    	call endrun('Failed to read OH in um_met_fld_prep!')
    end if


    inv_ho2 = get_inv_ndx('HO2') > 0  
    if ( inv_ho2 ) then    	
      call get_cnst_data( 'HO2',  tmp3d,  pcols, lchnk, pbuf ) ! vmr , must get pcols, if ncols, the data are dis located.      
      call outfld('um_HO2', tmp3d  , pcols, lchnk   )    
      const(:,:,IHO2) = tmp3d(:ncol,pver:1:-1)   
    else
    	call endrun('Failed to read HO2 in um_met_fld_prep!')
    end if


    inv_no3 = get_inv_ndx('NO3') > 0  
    if ( inv_no3 ) then    	
      call get_cnst_data( 'NO3',  tmp3d,  pcols, lchnk, pbuf ) ! vmr , must get pcols, if ncols, the data are dis located.      
      call outfld('um_NO3', tmp3d  , pcols, lchnk   )   
      const(:,:,INO3) = tmp3d(:ncol,pver:1:-1)    
    else
    	call endrun('Failed to read NO3 in um_met_fld_prep!')
    end if

! prepare IMGAS in const, calculate the air density at the center of each grid box (molecules/cm^3). 

! formula from IMPACT
!          const(:,:,:,imgas_num) =
!     &      press3c(i1:i2,ju1:j2,:) * MB2CGS /  !MB2CGS  =   1000.0d0         ! millibars to cgs units (dyne/cm^2)
!     &      (kel (i1:i2,ju1:j2,:) * BOLTZMN_E)  !BOLTZMN_E   =    1.380662d-16,   ! Boltzman constant ! (erg  *degK^-1*mole^-1)
    
    tmp3d = state%pmiddry / 100.0_r8 * 1000.0_r8 / (state%t * 1.380662d-16) !(molecules/cm^3). 
!    call outfld('um_VAR3D_01', tmp3d  , pcols, lchnk   ) 
    const(:,:,IMGAS) = tmp3d(:ncol,pver:1:-1)
 
    const(:,:,IAH2O) = 0.0_r8
 
!! below is for test purpose, set const to this fixed values !! 

!    do i = 1, ncol
!      do j = 1, pver
!       const(i,j,:) = const1d
!      end do
!    end do
    
  end subroutine um_const_prep
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ !        

   subroutine um_const_2_q (state) !called by , 1 chunk
!----------------------------------------------------------------------- 
!
! Purpose:  prepare the const array for IMPACT box model
!
!-----------------------------------------------------------------------
   use physics_types,   only: physics_state
   	
   use h_setkin_depos,  only: aerosol ! aerosol indicator; 0: gas; >0, types of aerosol.
   use h_imp_control,   only: mw ! moleculer weight of species in const
   use h_sulfchem,      only: wtmair
   	
   implicit none
   
   type(physics_state), intent(inout) :: state	
   	
   ! Local
   integer m
   integer lchnk,ncol

!--------------------------------------------------------------
 
!  Begin
   		
   	lchnk = state%lchnk
    ncol  = state%ncol
    
! to make the warnig in cmfca and convect_shallow go away, I apply 1e-10 to SO4n1, SO4n2, SO4n3 when put them in the state%q. 
      const (:,:,ISO4N1) = const (:,:,ISO4N1) * 1.0e-10_r8 !
      const (:,:,ISO4N2) = const (:,:,ISO4N2) * 1.0e-10_r8 !
      const (:,:,ISO4N3) = const (:,:,ISO4N3) * 1.0e-10_r8 !
      	
! copy advective species fsrom const to state%q, reverse vertical direction
	 do m = 1, NACT
     	
     	! convert from vmr to mmr for gases, in CAM5, all are mmr in state%q
      if (aerosol(m) .eq. 0) then
       state%q(:ncol,:,cnst_idx_q (m)) = const (:ncol,pver:1:-1,m) / wtmair * mw(m) ! from vmr to mmr
      else
       state%q(:ncol,:,cnst_idx_q (m)) = const (:ncol,pver:1:-1,m)  
      end if
      	
	 end do
	
    
  end subroutine um_const_2_q
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ !
  
  
  !============================================================================ 
  ! Allocate the fields for each chunk before um_met_fld_prep and um_const_prep, deallocate after.
  ! umaerosol needs to loop all the elements in the input arrays (itloop total),
  ! Use pcols to define array size would cause undefined elements in the arrays.
  ! Use ncol and allocatable arrays only.                                                                          
  !============================================================================    
   subroutine um_allocate_fld (ncol) !1 chunk

!----
! input argument
!----
   integer :: ncol

! local argumnet
   	
   integer :: ierror(100),ierr_count
   	
! begin	

   ierr_count = 0

   ierr_count = ierr_count+1
	 allocate (latdeg(ncol), stat=ierror(ierr_count))

   ierr_count = ierr_count+1
	 allocate (londeg(ncol), stat=ierror(ierr_count))

   ierr_count = ierr_count+1
	 allocate (mcor(ncol), stat=ierror(ierr_count))
	  
   ierr_count = ierr_count+1
	 allocate (humidity(ncol,pver), stat=ierror(ierr_count))
	 
   ierr_count = ierr_count+1
	 allocate (IC_tot_liq(ncol,pver), stat=ierror(ierr_count))
	 
   ierr_count = ierr_count+1
	 allocate (mass(ncol,pver), stat=ierror(ierr_count))
	 
   ierr_count = ierr_count+1
	 allocate (cld(ncol,pver), stat=ierror(ierr_count))
	 
   ierr_count = ierr_count+1
	 allocate (press3c(ncol,pver), stat=ierror(ierr_count))	 

   ierr_count = ierr_count+1
	 allocate (grid_height(ncol,pver), stat=ierror(ierr_count))	 	 

   ierr_count = ierr_count+1
	 allocate (ispb(ncol,pver), stat=ierror(ierr_count))	 	
	 	 	    
   ierr_count = ierr_count+1
	 allocate (kel(ncol,pver), stat=ierror(ierr_count))
	 	 
   ierr_count = ierr_count+1
	 allocate (moistq(ncol,pver), stat=ierror(ierr_count))	
	  
   ierr_count = ierr_count+1
	 allocate (cmf(ncol,pver), stat=ierror(ierr_count))
	 
   ierr_count = ierr_count+1
	 allocate (dms_oh(ncol,pver), stat=ierror(ierr_count))	 
	 
   ierr_count = ierr_count+1
	 allocate (dms_no3(ncol,pver), stat=ierror(ierr_count))
	 
   ierr_count = ierr_count+1
	 allocate (so2_oh(ncol,pver), stat=ierror(ierr_count))
	 
   ierr_count = ierr_count+1
	 allocate (so2_o3(ncol,pver), stat=ierror(ierr_count))	
	  
   ierr_count = ierr_count+1
	 allocate (so2_h2o2(ncol,pver), stat=ierror(ierr_count))	 
	 
   ierr_count = ierr_count+1
	 allocate (so2_hete(ncol,pver), stat=ierror(ierr_count))	 
	 
   ierr_count = ierr_count+1
	 allocate (semiss(ncol,pver,num_emiss), stat=ierror(ierr_count))

   ierr_count = ierr_count+1
	 allocate (qjimp(ncol,pver), stat=ierror(ierr_count))
	 
   ierr_count = ierr_count+1
	 allocate (qkimp(ncol,pver,num_qks), stat=ierror(ierr_count))	
	 
   ierr_count = ierr_count+1
	 allocate (const(ncol,pver,NSP), stat=ierror(ierr_count))
	 
	 
	 ! for deposition by large scale precipitation
	 ierr_count = ierr_count+1
	 allocate (con_precip(ncol), stat=ierror(ierr_count))

	 ierr_count = ierr_count+1
	 allocate (tot_precip(ncol), stat=ierror(ierr_count))
	  	 	  
   ierr_count = ierr_count+1
	 allocate (press3e(ncol,pver+1), stat=ierror(ierr_count))
	 
	 ! for deposition by convective precipitatio
 	 ierr_count = ierr_count+1
	 allocate (pblh(ncol), stat=ierror(ierr_count))
	 
	 if ( any( ierror(1:ierr_count) .ne. 0 ) ) then 
	   print*, ierror(1:ierr_count)
	   call endrun("Error in allocation in um_allocate_fld")
	   
	 end if
	 	
	 end subroutine um_allocate_fld	 
  !============================================================================ !
  !                                                                             !
  !============================================================================ !  
  
 
   subroutine um_deallocate_fld !1 chunk

   	
! begin	

    if(allocated(latdeg)) then
      deallocate(latdeg)
    end if
    	
    if(allocated(londeg)) then
      deallocate(londeg)
    end if	
    	
    if(allocated(mcor)) then
      deallocate(mcor)
    end if

    if(allocated(grid_height)) then
      deallocate(grid_height)
    end if

    if(allocated(ispb)) then
      deallocate(ispb)
    end if    	
    	
    if(allocated(kel)) then
      deallocate(kel)
    end if
    	
    if(allocated(humidity)) then
      deallocate(humidity)
    end if
    	
    if(allocated(IC_tot_liq)) then
      deallocate(IC_tot_liq)
    end if
    	
    if(allocated(mass)) then
      deallocate(mass)
    end if
    	
    if(allocated(cld)) then
      deallocate(cld)
    end if
    	
    if(allocated(press3c)) then
      deallocate(press3c)
    end if
    	
    if(allocated(moistq)) then
      deallocate(moistq)
    end if
    	
    if(allocated(cmf)) then
      deallocate(cmf)
    end if
    	
    if(allocated(dms_oh)) then
      deallocate(dms_oh)
    end if
    	
    if(allocated(dms_no3)) then
      deallocate(dms_no3)
    end if
    	
    if(allocated(so2_oh)) then
      deallocate(so2_oh)
    end if
    	
    if(allocated(so2_h2o2)) then
      deallocate(so2_h2o2)
    end if
    	
    if(allocated(so2_o3)) then
      deallocate(so2_o3)
    end if
    	
    if(allocated(so2_hete)) then
      deallocate(so2_hete)
    end if
    	
    if(allocated(semiss)) then
      deallocate(semiss)
    end if
    	
    if(allocated(qjimp)) then
      deallocate(qjimp)
    end if
    	
    if(allocated(qkimp)) then
      deallocate(qkimp)
    end if
    	
    if(allocated(const)) then
      deallocate(const)
    end if
    	
    if(allocated(con_precip)) then
      deallocate(con_precip)
    end if
    	
    if(allocated(tot_precip)) then
      deallocate(tot_precip)
    end if
    	   	
    if(allocated(press3e)) then
      deallocate(press3e)
    end if
    	
    if(allocated(pblh)) then
      deallocate(pblh)
    end if
    	
    	
!    if(allocated()) then
!       deallocate()
!    end if
!    	
!    if(allocated()) then
!       deallocate()
!    end if
!    	
!    if(allocated()) then
!       deallocate()
!    end if
!    	
!    if(allocated()) then
!       deallocate()
!    end if
!    	
!    if(allocated()) then
!       deallocate()
!    end if
!  
	 	
	 end subroutine um_deallocate_fld
	 
  !============================================================================ !
  !                                                                             !
  !============================================================================ !   	
  
 
 
  integer function um_get_spc_ndx( spc_name ) 
    !-----------------------------------------------------------------------
    !     ... return um spec index, not the index in state%q
    !-----------------------------------------------------------------------

    implicit none

    !-----------------------------------------------------------------------
    !     ... dummy arguments
    !-----------------------------------------------------------------------
    character(len=*), intent(in) :: spc_name

    !-----------------------------------------------------------------------
    !     ... local variables
    !-----------------------------------------------------------------------
    integer :: m

    um_get_spc_ndx = -1
    do m = 1,NACT
       if( trim( spc_name ) == trim(cnst_names(m))) then
          um_get_spc_ndx = m
          return
       end if
    end do
    
    if (um_get_spc_ndx  .lt. 1) then
    	  write (*,*) '** Warning ** specieces '//spc_name//' is not used by um_get_spc_ndx.'
    end if

  end function um_get_spc_ndx
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ ! 
  

  integer function um_get_spc_ndx_in_q( spc_name ) 
    !-----------------------------------------------------------------------
    !     ... return um spec index,  the index in state%q
    !-----------------------------------------------------------------------

    implicit none

    !-----------------------------------------------------------------------
    !     ... dummy arguments
    !-----------------------------------------------------------------------
    character(len=*), intent(in) :: spc_name

    !-----------------------------------------------------------------------
    !     ... local variables
    !-----------------------------------------------------------------------
    integer :: m

    um_get_spc_ndx_in_q = -1
    do m = 1,NACT
       if( trim( spc_name ) == trim(cnst_names(m))) then
          um_get_spc_ndx_in_q =cnst_idx_q (m)
          return
       end if
    end do
    
    if (um_get_spc_ndx_in_q .lt. 1) then
    	  write (*,*) '** Warning ** specieces '//spc_name//' is not used by um_get_spc_ndx_in_q.'
    end if

  end function um_get_spc_ndx_in_q
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ ! 
    
  
  subroutine  um_update_emis(state) !called by , 1 chunk
!----------------------------------------------------------------------- 
! Purpose: 
! Update state%q with surface emissions. Carbon,sulf related surface emission (kg/m2/s) is from mo_srf_emissions.F90;
! verical emissions are from mo_extfrc.F90; sea salt is from um_sslt_intr.F; Dust is from 
! CAUTION,in argrument state has only one chunk.
!-----------------------------------------------------------------------
    use cam_history,        only: outfld
    use physics_types,      only: physics_state
    use physconst,          only: gravit ,mwdry ! Acceleration due to gravity 	
    use um_read_aircraft_intr, only: um_get_aircraft
    use um_contrail_form,   only: um_contrail_flag
!-----------------------------------------------------------------------
    implicit none
!-----------------------------------------------------------------------
!
! Arguments:
!
    type(physics_state), intent(inout) :: state          ! Physics state variables
    	
!----local  arguments
    integer :: lchnk, ncol
    integer :: m, ndx,j
    real(r8):: tmp1(pcols) !working space
    real(r8), dimension(pcols,pver) :: tmp3d, tmp3d_mass_dt
    real(r8), dimension(pcols,pver) ::  contrail_flag 
! begin   	
    lchnk = state%lchnk
    ncol  = state%ncol
    
    
    tmp1         (:ncol)   = dtime * gravit * state%rpdeldry(:ncol,pver) ! 1/(dp/g)*dt ==1/mass (kg) * dt, depending on weather it is dry or wet mixing. Assuming dry mixture. sloppy coding.  rpdel vs. rpdeldry
    tmp3d_mass_dt(:ncol,:) = dtime * gravit * state%rpdeldry(:ncol,:)  ! dt/mass


    tmp3d = state%pmiddry / 100.0_r8 * 1000.0_r8 / (state%t * 1.380662d-16) !(molecules/cm^3). 
    tmp3d = dtime * 1.4_r8 * 12.011_r8 / ( mwdry * tmp3d ) ! dt/ (air #/cm^3 ) times OM/air molecule weight factor

 ! update fBC,fOC,nOC surface emission: kg/m2/s
      state%q(:ncol,pver,cnst_idx_q(IFBC)) = state%q(:ncol,pver,cnst_idx_q(IFBC)) + um_emis_srf(lchnk,:ncol,IFBC) * tmp1(:ncol) 
      state%q(:ncol,pver,cnst_idx_q(IFOC)) = state%q(:ncol,pver,cnst_idx_q(IFOC)) + um_emis_srf(lchnk,:ncol,IFOC) * tmp1(:ncol)
      state%q(:ncol,pver,cnst_idx_q(INOC)) = state%q(:ncol,pver,cnst_idx_q(INOC)) + um_emis_srf(lchnk,:ncol,INOC) * tmp1(:ncol)  

 ! update dust surface emission: kg/m2/s
      state%q(:ncol,pver,cnst_idx_q(IDUST1)) = state%q(:ncol,pver,cnst_idx_q(IDUST1)) + um_emis_srf(lchnk,:ncol,IDUST1) * tmp1(:ncol) 
      state%q(:ncol,pver,cnst_idx_q(IDUST2)) = state%q(:ncol,pver,cnst_idx_q(IDUST2)) + um_emis_srf(lchnk,:ncol,IDUST2) * tmp1(:ncol) 
      state%q(:ncol,pver,cnst_idx_q(IDUST3)) = state%q(:ncol,pver,cnst_idx_q(IDUST3)) + um_emis_srf(lchnk,:ncol,IDUST3) * tmp1(:ncol) 
      state%q(:ncol,pver,cnst_idx_q(IDUST4)) = state%q(:ncol,pver,cnst_idx_q(IDUST4)) + um_emis_srf(lchnk,:ncol,IDUST4) * tmp1(:ncol) 
      	
 ! update sea salt surface emission: kg/m2/s
      state%q(:ncol,pver,cnst_idx_q(ISSLT1)) = state%q(:ncol,pver,cnst_idx_q(ISSLT1)) + um_emis_srf(lchnk,:ncol,ISSLT1) * tmp1(:ncol) 
      state%q(:ncol,pver,cnst_idx_q(ISSLT2)) = state%q(:ncol,pver,cnst_idx_q(ISSLT2)) + um_emis_srf(lchnk,:ncol,ISSLT2) * tmp1(:ncol) 
      state%q(:ncol,pver,cnst_idx_q(ISSLT3)) = state%q(:ncol,pver,cnst_idx_q(ISSLT3)) + um_emis_srf(lchnk,:ncol,ISSLT3) * tmp1(:ncol) 
      state%q(:ncol,pver,cnst_idx_q(ISSLT4)) = state%q(:ncol,pver,cnst_idx_q(ISSLT4)) + um_emis_srf(lchnk,:ncol,ISSLT4) * tmp1(:ncol) 
        	  
 ! update bBC,bOC vertical emission: #/cm3/s, bottom up (was converted from topdown to bottom-up in mo_extfrc.F90).
      state%q(:ncol,:,cnst_idx_q(IBBC)) = state%q(:ncol,:,cnst_idx_q(IBBC)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IBBC) * tmp3d(:ncol,:) 
      state%q(:ncol,:,cnst_idx_q(IBOC)) = state%q(:ncol,:,cnst_idx_q(IBOC)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IBOC) * tmp3d(:ncol,:)

 ! get contrail flag using Schmidt-Appleman criteria
     contrail_flag(:,:) = 0.0_r8
     do m = 1, ncol
        do j = 1, pver
        call um_contrail_flag(state%pmid(m,j),state%t(m,j),state%q(m,j,1),contrail_flag(m,j) )
        end do
     end do   
     call outfld ('contrail_flag', contrail_flag(:ncol,:) , ncol, lchnk) 
     	
     
 ! update aircraft emissions, read in is top down, convert to bottom up in um_emiss_ver and back to top down in q
      um_emis_ver_has(IABC)  = .true.
      um_emis_ver_has(IAPOM) = .true.
      um_emis_ver_has(ICBC)  = .true.
      um_emis_ver_has(ICPOM) = .true.
      um_emis_ver_has(IASO2) = .true.
      um_emis_ver_has(IASO4) = .true.
      call um_get_aircraft(um_emis_ver(lchnk,:ncol,pver:1:-1,IABC), &
      	                   um_emis_ver(lchnk,:ncol,pver:1:-1,IAPOM),&
      	                   um_emis_ver(lchnk,:ncol,pver:1:-1,IASO2),&
      	                   um_emis_ver(lchnk,:ncol,pver:1:-1,IASO4), ncol,lchnk) !kg/m2/s/box
      	                   	
!      state%q(:ncol,:,cnst_idx_q(IABC)) = state%q(:ncol,:,cnst_idx_q(IABC)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IABC) * tmp3d_mass_dt(:ncol,:) 	
!    
!      state%q(:ncol,:,cnst_idx_q(IAPOM)) = state%q(:ncol,:,cnst_idx_q(IAPOM)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IAPOM) * tmp3d_mass_dt(:ncol,:) 	
! 
!      state%q(:ncol,:,cnst_idx_q(IASO2)) = state%q(:ncol,:,cnst_idx_q(IASO2)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IASO2) * tmp3d_mass_dt(:ncol,:) 	
!      state%q(:ncol,:,cnst_idx_q(IFSO2)) = state%q(:ncol,:,cnst_idx_q(IFSO2)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IASO2) * tmp3d_mass_dt(:ncol,:) 	! also added to fSO2
!    
!      state%q(:ncol,:,cnst_idx_q(IASO4))  = state%q(:ncol,:,cnst_idx_q(IASO4))  + um_emis_ver(lchnk,:ncol,pver:1:-1,IASO4) * tmp3d_mass_dt(:ncol,:) 	
!      state%q(:ncol,:,cnst_idx_q(ISO4M2)) = state%q(:ncol,:,cnst_idx_q(ISO4M2)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IASO4) * tmp3d_mass_dt(:ncol,:) ! aSO4 is added to Aitken mode       	
!      state%q(:ncol,:,cnst_idx_q(ISO4N2)) = state%q(:ncol,:,cnst_idx_q(ISO4N2)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IASO4) * tmp3d_mass_dt(:ncol,:) * 2.778e10 ! from kg/kg to 1e10 #/kg.  1kg is 2.778e20 #, however, in q I applied a factor 10^(-10) to make it smaller.see	E:\CESM\CESM11\Code_develop\Aircraft_emission_prepare

      !separate emission to contrail part and non-contrail part
      um_emis_ver(lchnk,:ncol,:,ICBC) =  um_emis_ver(lchnk,:ncol,:,IABC) *  contrail_flag(:ncol,pver:1:-1)  ! where contrail forms, goes to cBC, in emis_ver is bottom up
      um_emis_ver(lchnk,:ncol,:,IABC) =  um_emis_ver(lchnk,:ncol,:,IABC) * (1.0_r8 - contrail_flag(:ncol,pver:1:-1)) ! for non-contrail part           	

      um_emis_ver(lchnk,:ncol,pver:1:-1,ICPOM) =  um_emis_ver(lchnk,:ncol,pver:1:-1,IAPOM) *  contrail_flag
      um_emis_ver(lchnk,:ncol,pver:1:-1,IAPOM) =  um_emis_ver(lchnk,:ncol,pver:1:-1,IAPOM) * (1.0_r8 - contrail_flag) 
      	      	
      state%q(:ncol,:,cnst_idx_q(IABC)) = state%q(:ncol,:,cnst_idx_q(IABC)) +  um_emis_ver(lchnk,:ncol,pver:1:-1,IABC) * tmp3d_mass_dt(:ncol,:) 	
      state%q(:ncol,:,cnst_idx_q(ICBC)) = state%q(:ncol,:,cnst_idx_q(ICBC)) +  um_emis_ver(lchnk,:ncol,pver:1:-1,ICBC) * tmp3d_mass_dt(:ncol,:) 	
      	   
      state%q(:ncol,:,cnst_idx_q(IAPOM)) = state%q(:ncol,:,cnst_idx_q(IAPOM)) +  um_emis_ver(lchnk,:ncol,pver:1:-1,IAPOM) * tmp3d_mass_dt(:ncol,:) 	
      state%q(:ncol,:,cnst_idx_q(ICPOM)) = state%q(:ncol,:,cnst_idx_q(ICPOM)) +  um_emis_ver(lchnk,:ncol,pver:1:-1,ICPOM) * tmp3d_mass_dt(:ncol,:)  
      	
      state%q(:ncol,:,cnst_idx_q(IASO2)) = state%q(:ncol,:,cnst_idx_q(IASO2)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IASO2) * tmp3d_mass_dt(:ncol,:) 	
!     state%q(:ncol,:,cnst_idx_q(IFSO2)) = state%q(:ncol,:,cnst_idx_q(IFSO2)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IASO2) * tmp3d_mass_dt(:ncol,:) 	! also added to fSO2
    
      state%q(:ncol,:,cnst_idx_q(IASO4))  = state%q(:ncol,:,cnst_idx_q(IASO4))  + um_emis_ver(lchnk,:ncol,pver:1:-1,IASO4) * tmp3d_mass_dt(:ncol,:) 	
      state%q(:ncol,:,cnst_idx_q(ISO4M2)) = state%q(:ncol,:,cnst_idx_q(ISO4M2)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IASO4) * tmp3d_mass_dt(:ncol,:) ! aSO4 is added to Aitken mode       	
      state%q(:ncol,:,cnst_idx_q(ISO4N2)) = state%q(:ncol,:,cnst_idx_q(ISO4N2)) + um_emis_ver(lchnk,:ncol,pver:1:-1,IASO4) * tmp3d_mass_dt(:ncol,:) * 2.778e10 ! from kg/kg to 1e10 #/kg.  1kg is 2.778e20 #, however, in q I applied a factor 10^(-10) to make it smaller.see	E:\CESM\CESM11\Code_develop\Aircraft_emission_prepare
      	
      
            	
      	
 ! outfld all emissions   	
    do m = 1, NACT
     if (um_emis_srf_has(m)) then
      call outfld ('emis_srf_'//trim(cnst_names(m)), um_emis_srf(lchnk,:ncol,m), ncol, lchnk)
     end if
     
     if (um_emis_ver_has(m)) then
      call outfld ('emis_ver_'//trim(cnst_names(m)), um_emis_ver(lchnk,:ncol,pver:1:-1,m), ncol, lchnk) ! from bottom up to top down
     end if	
    end do

      	
      	
          	
  end subroutine  um_update_emis
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ ! 
  
    subroutine  um_depos_stub(state) !called by , 1 chunk
!----------------------------------------------------------------------- 
! 
! Purpose: do simple 2-day lifetime depoistion for all specie
! 
!-----------------------------------------------------------------------
    use cam_history,        only: outfld
    use physics_types,      only: physics_state
    use physconst,          only: gravit ,mwdry ! Acceleration due to gravity 	
!-----------------------------------------------------------------------
    implicit none
!-----------------------------------------------------------------------
!
! Arguments:
!
    type(physics_state), intent(inout) :: state          ! Physics state variables
    	
!----local  arguments
    integer :: lchnk, ncol
    integer :: m, ndx ,i
    real(r8):: tmp1(pcols) !working space
    real(r8), dimension(pcols,pver) :: tmp3d
    real(r8), dimension(pcols)      :: tmp2d
 
! begin   	
    lchnk = state%lchnk
    ncol  = state%ncol
    

    tmp3d = state%pdeldry(:ncol,:)/gravit/dtime !mass/dt in kg/s
    tmp3d =  tmp3d 	* ( 1._r8 - exp (-dtime/(4.0d0*86400.0d0)))
    	
! output deposition field
    do m = 1, NACT
       
       do i = 1, ncol
          tmp2d (i) =  sum (state%q(i,:,cnst_idx_q(m)) * tmp3d(i,:))
       end do 

       call outfld ('um_wetdep1_'//trim(cnst_names(m)), tmp2d(:ncol), ncol, lchnk)

    end do

      
 !  deposition stub, 4day lifetime
       state%q(:,:,cnst_idx_q(1):cnst_idx_q(NACT)) =  &
       state%q(:,:,cnst_idx_q(1):cnst_idx_q(NACT)) * exp (-dtime/(2.0d0*86400.0d0))   
       	
       
       		
      	
          	
  end subroutine  um_depos_stub
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ ! 

    subroutine  um_wetdepos_intr(state, cam_in) !called by step , 1 chunk
!----------------------------------------------------------------------- 
! 
! Purpose: do the wet depoistion by large scale precipitation
! 
!-----------------------------------------------------------------------
    use cam_history,        only: outfld
    use physics_types,      only: physics_state
    use physconst,          only: gravit ,mwdry ! Acceleration due to gravity 	
    use camsrfexch,         only: cam_in_t
      	    	
    use h_setkin_mw,        only: mw_data
!-----------------------------------------------------------------------
    implicit none
!-----------------------------------------------------------------------
!
! Arguments:
!
    type(physics_state), intent(in) :: state          ! Physics state variables
    type(cam_in_t), target, intent(in) :: cam_in  ! import state	   
    	 	
!----local  arguments
    integer :: lchnk, ncol
    integer :: m
    real(r8), pointer :: ocnfrac(:), landfrac(:), icefrac(:)
 

! required input arguments 
      integer :: ih2o2_num !IH2O2
      integer :: ihno3_num !IHNO3
      real(r8)  :: mw_in(NSP) !mw
      real(r8)  :: coscen(pcols)
      real(r8)  :: tdt !use dtime
!      real(r8)  :: lwi_flags (pcols)
      real(r8)  :: REL_SCAV_EFF_new   (pcols, pver, NSP) ! used for wet deposition
   	      		
! following are done in um_met_fld_prep
!      real(r8)  :: con_precip (i1:i2,   ju1:j2) 
!      real(r8)  :: tot_precip (i1:i2,   ju1:j2)
!      real(r8)  :: mcor       (i1:i2,   ju1:j2)
!      real(r8)  :: grid_height(i1:i2,   ju1:j2,   k1:k2)
!      real(r8)  :: mass       (i1:i2,   ju1:j2,   k1:k2)
!      real(r8)  :: moistq     (i1:i2,   ju1:j2,   k1:k2)
!      real(r8)  :: kel        (ilo:ihi, julo:jhi, k1:k2)
!      real(r8)  :: press3c    (ilo:ihi, julo:jhi, k1:k2)
!      real(r8)  :: press3e    (ilo:ihi, julo:jhi, k1-1:k2)
!      real(r8)  :: const      (i1:i2,   ju1:j2,   k1:k2, num_species)
!      real(r8)  :: humidity   (i1:i2,   ju1:j2,   k1:k2)

      real(r8)  :: wet_depos      (pcols, NSP)
      real(r8)  :: wet_depos_wash (pcols, NSP)

      

! begin   	
    lchnk = state%lchnk
    ncol  = state%ncol
    
       ih2o2_num = IH2O2
       ihno3_num = IHNO3
       tdt       = dtime 
       mw_in (:NSP)  = mw_data(:NSP)
       coscen(:ncol) = cos(state%lat(:ncol))
       REL_SCAV_EFF_new (:,:,:) = 0.0d0 !  will be set in the convection,Do_Convec_Ncar
     
! generate lwi_flags   !1 for water; 2 for land ; 3 for ice	
       	  ocnfrac  => cam_in%ocnfrac
          landfrac => cam_in%landfrac
          icefrac  => cam_in%icefrac
          
          lwi_flags (:) = 1  
       where  ( landfrac(:) >= ocnfrac(:) .and. landfrac(:) >= icefrac(:))
       	  lwi_flags (:) = 2 
       end where
            
       where  ( icefrac(:) > ocnfrac(:) .and. icefrac(:) > landfrac(:))
       	  lwi_flags (:) = 3
       end where
       
!       call outfld("um_VAR2D_01",lwi_flags(:ncol),ncol,lchnk)
       		
!! do convection and deposition in updrafts     
       wet_depos       (:,:) = 0.0d0
       call Do_Convec_Ncar ( ih2o2_num, ihno3_num, lwi_flags(:ncol), tdt, &
          mw_in, mcor, pblh, cmf, grid_height, mass, wet_depos(:ncol,:), &
          kel, press3e, const, humidity, press3c, REL_SCAV_EFF_new(:ncol,:,:),dust_so4_layer(lchnk,:ncol,:,:))
      
       wet_depos(:ncol,:) = wet_depos(:ncol,:)/tdt
       do m = 1, NACT     
         call outfld ('um_wetdep1_'//trim(cnst_names(m)), wet_depos (:ncol,m), ncol, lchnk)
       end do
       
       call outfld ('dust1_so4', dust_so4_layer(lchnk,:ncol,:,1), ncol, lchnk)
       call outfld ('dust2_so4', dust_so4_layer(lchnk,:ncol,:,2), ncol, lchnk)
       call outfld ('dust3_so4', dust_so4_layer(lchnk,:ncol,:,3), ncol, lchnk)
       call outfld ('dust4_so4', dust_so4_layer(lchnk,:ncol,:,4), ncol, lchnk)		 
        
!        REL_SCAV_EFF_new (:,:,:) = 1.0d0 !  will be set in the convection,Do_Convec_Ncar  
!! do large scale wet deposition 
       wet_depos       (:,:) = 0.0d0
       wet_depos_wash  (:,:) = 0.0d0
      call Update_Wetdep(ih2o2_num, ihno3_num, tdt, mw_in, coscen, con_precip, tot_precip, &  
      mcor, grid_height, mass, moistq, kel, press3c, press3e,  &  
      const, wet_depos_wash(:ncol,:), wet_depos(:ncol,:), humidity, REL_SCAV_EFF_new(:ncol,:,:))      	   
    
      wet_depos(:ncol,:) = wet_depos(:ncol,:)/tdt
      do m = 1, NACT     
        call outfld ('um_wetdep2_'//trim(cnst_names(m)), wet_depos (:ncol,m), ncol, lchnk)
      end do
    
      call outfld('um_VAR3D_03', REL_SCAV_EFF_new(:ncol,:,35) , ncol, lchnk)	    
      call outfld('um_VAR3D_04', REL_SCAV_EFF_new(:ncol,:,40) , ncol, lchnk)	
      	
  end subroutine  um_wetdepos_intr
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ ! 


    subroutine  um_grav_drydep_intr(state,cam_in,fsns) !called by step , 1 chunk
!----------------------------------------------------------------------- 
! 
! Purpose: do the wet depoistion by large scale precipitation
! 
!-----------------------------------------------------------------------
    use cam_history,        only: outfld
    use physics_types,      only: physics_state
    use camsrfexch,         only: cam_in_t
    
    !--
    use um_readin,          only: um_get_veg	
!-----------------------------------------------------------------------
    implicit none
!-----------------------------------------------------------------------
!
! Arguments:
!
    type(physics_state), intent(in) :: state          ! Physics state variables
    type(cam_in_t), target, intent(in) :: cam_in  ! import state	
    real(r8), intent(in) :: fsns(pcols)                   ! Surface solar absorbed flux

!----local  arguments
    integer :: lchnk, ncol
    integer :: m
    integer, parameter :: NTYPE = 15 ! maximum land type used in the dry deposition process.
    	
    real(r8), dimension (pcols, NSP) :: diffaer, s_radius, s_velocity, drydep_vel, dry_depos
    real(r8), dimension (pcols)  ::  surf_rough, box_zm,box_zi,surf_air_temp
    real(r8), pointer :: u10(:)
    	
    	
    integer :: ireg   (pcols)
    integer :: iland  (pcols, 1:NTYPE)
    integer :: iuse   (pcols, 1:NTYPE)
    real(r8):: xlai   (pcols, 1:NTYPE) !leaf area index of land type elements
      	
! begin   	
    lchnk = state%lchnk
    ncol  = state%ncol

! do gravitational settling, no deposition on ground yet, just pass settling speed out    
    call Update_Grav_Settling (dtime,grid_height, humidity, mass, press3e, kel, const, &
     diffaer(:ncol,:), s_radius(:ncol,:), s_velocity(:ncol,:))


! prepare fields for drydepos    
    box_zi (:ncol)       = state%zi(:ncol, pver ) - state%zi(:ncol, pver + 1 )!height of bottom box
    box_zm (:ncol)       = state%zm(:ncol, pver )	
    surf_air_temp(:ncol) = state%t (:ncol, pver)

    u10   => cam_in%u10
    call outfld('um_U10', u10, pcols, lchnk)	
    
    ustar = max (0.01d0,ustar)   ! ustar must be >0, otherwise surf_rough below would be divided by zero...
    surf_rough(:ncol) = 10.0_r8 * exp(-0.4_r8*u10(:ncol)/ustar(:ncol)) ! based on  u10 = ustar/0.4 * ln ( 10/surf_rough) ; ustar only have value over ice/ocean
    surf_rough = max (surf_rough, 1.0d-5)
    ustar = max (0.01d0,ustar) 	 ! has to be >0 required by the dry depostion process


    call um_get_veg(ireg, iland, iuse, xlai, ncol,lchnk, NTYPE)
 
    call outfld("um_VAR2D_01",ireg(:ncol)    * 1.0_r8, ncol,lchnk)
    call outfld("um_VAR2D_02",iland(:ncol,2) * 1.0_r8, ncol,lchnk)
     
!! do dry deposition   
    dry_depos(:,:) = 0.0d0
    call Update_Drydep &   ! for static allocated arrays :ncol has to be attached, for dynamically allocated arrays, :ncol is not necessary but it is safer to attach as well.   
       (dtime,mcor, latdeg, londeg,  cld, cld, mass, const, & 
        fsns(:ncol), surf_air_temp(:ncol), surf_rough(:ncol), ustar(:ncol), lwi_flags(:ncol),box_zm(:ncol),box_zi(:ncol),&
        diffaer(:ncol,:), s_radius(:ncol,:), s_velocity(:ncol,:),dry_depos(:ncol,:),drydep_vel(:ncol,:),&
        ireg(:ncol),iland(:ncol,:),iuse(:ncol,:),xlai(:ncol,:))

      dry_depos(:,:) = dry_depos(:,:)/dtime
      do m = 1, NACT     
        call outfld ('um_drydep_'//trim(cnst_names(m)), dry_depos (:ncol,m), ncol, lchnk)
      end do
     call outfld('um_VAR3D_01', s_velocity(:ncol,1:30) , ncol, lchnk)	
     call outfld('um_VAR3D_02', drydep_vel(:ncol,1:30) , ncol, lchnk)	

          	
  end subroutine  um_grav_drydep_intr
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ !      
  end module  um_aerosol_intr
