  module um_aerosol_number

  !-------------------------------------------------------------------------------------------------------
  ! Purpose: calculate aerosol number.
  !
  ! 
  ! Author: Cheng Zhou, 2013-08-19,Zhouc@umich.edu
  !-------------------------------------------------------------------------------------------------------
  use shr_kind_mod,  only: r8=>shr_kind_r8
  use ppgrid,        only: pver, pcols
  use abortutils,    only: endrun
  use cam_logfile,   only: iulog
  use spmd_utils,    only: masterproc,iam
!----  	

  implicit none
  private
  save
  
  integer  :: idx_SO4n2,idx_SO4n3,idx_aBC, idx_aPOM,idx_cBC, idx_cPOM,idx_fBC,idx_fOC,idx_fBC1,idx_fOC1,idx_fBC2,idx_fOC2,idx_bBC,idx_bOC,idx_dst1,idx_dst2,idx_dst3,idx_dst4
  real(r8),parameter :: re_airsoot = 0.03*1.0d-6, rhoairsoot=2.300*1.0e3 ,  rhoapom =9.0*1.0e2 , &
   re_soot=0.0896*1.0e-6,&  ! Pueschel et al., 1992: rg=0.07um, sig=1.5
   re_dst1=0.302*1.0e-6,& 
   re_dst2=0.764*1.0e-6,& 
   re_dst3=1.631*1.0e-6,& 
   re_dst4=3.114*1.0e-6,& 

   rhoboc =1.000*1.0e3 ,&    ! BC,OC
   rhodst =2.600*1.0e3    ! Dust

   
   
  real(r8) :: pm_aBC, pm_aPOM,pm_soot,pm_dst1,pm_dst2,pm_dst3,pm_dst4 ! particle mass
  	
  public :: um_aerosol_number_init  ! called by physpkg.F90
  public :: um_IN_numbers ! called by microp_aero.F90
  	
  contains
  
  subroutine um_aerosol_number_init ! all chunks
  use cam_history,     only: addfld, add_default, phys_decomp
  use um_aerosol_intr, only: um_get_spc_ndx_in_q

  call addfld      ('um_IN_hetero_dust', '#/cm3', pver, 'I', 'Heterogenous IN number dust', phys_decomp) !
  call add_default ('um_IN_hetero_dust', 1, 'A') 
    	
  call addfld      ('um_IN_hetero_dust1', '#/cm3', pver, 'I', 'Heterogenous IN number dust1', phys_decomp) !
  call add_default ('um_IN_hetero_dust1', 1, 'A')   

  call addfld      ('um_IN_hetero_dust2', '#/cm3', pver, 'I', 'Heterogenous IN number dust2', phys_decomp) !
  call add_default ('um_IN_hetero_dust2', 1, 'A')

  call addfld      ('um_IN_hetero_dust3', '#/cm3', pver, 'I', 'Heterogenous IN number dust3', phys_decomp) !
  call add_default ('um_IN_hetero_dust3', 1, 'A')
  
  call addfld      ('um_IN_hetero_dust4', '#/cm3', pver, 'I', 'Heterogenous IN number dust4', phys_decomp) !
  call add_default ('um_IN_hetero_dust4', 1, 'A')  
  
  call addfld      ('um_IN_hetero_asoot', '#/cm3', pver, 'I', 'Heterogenous IN number asoot', phys_decomp) !
  call add_default ('um_IN_hetero_asoot', 1, 'A') 
   
  call addfld      ('um_IN_hetero_soot', '#/cm3', pver, 'I', 'Heterogenous IN number soot', phys_decomp) !
  call add_default ('um_IN_hetero_soot', 1, 'A') 
   
  call addfld      ('um_IN_homo1', '#/cm3', pver, 'I', 'Homogenenous haze number, Aitken mode SO4', phys_decomp) !
  call add_default ('um_IN_homo1', 1, 'A')

  call addfld      ('um_IN_homo2', '#/cm3', pver, 'I', 'Homogenenous haze number: Accu. mode SO4', phys_decomp) !
  call add_default ('um_IN_homo2', 1, 'A')
    
  
   
  !! calculate particle mass 
  pm_aBC  = 3.141592653_r8 * 4.0_r8 / 3.0_r8 * re_airsoot ** 3 * rhoairsoot 
  pm_aPOM = 3.141592653_r8 * 4.0_r8 / 3.0_r8 * re_airsoot ** 3 * rhoapom
  pm_soot = 3.141592653_r8 * 4.0_r8 / 3.0_r8 * re_soot    ** 3 * rhoboc
  pm_dst1 = 3.141592653_r8 * 4.0_r8 / 3.0_r8 * re_dst1    ** 3 * rhodst
  pm_dst2 = 3.141592653_r8 * 4.0_r8 / 3.0_r8 * re_dst2    ** 3 * rhodst
  pm_dst3 = 3.141592653_r8 * 4.0_r8 / 3.0_r8 * re_dst3    ** 3 * rhodst        
  pm_dst4 = 3.141592653_r8 * 4.0_r8 / 3.0_r8 * re_dst4    ** 3 * rhodst 
  
  !! get index in q
  idx_SO4n2  = um_get_spc_ndx_in_q ('SO4n2')
  idx_SO4n3  = um_get_spc_ndx_in_q ('SO4n3')
      
  idx_aBC  = um_get_spc_ndx_in_q ('aBC')
  idx_aPOM = um_get_spc_ndx_in_q ('aPOM')
  
  idx_cBC  = um_get_spc_ndx_in_q ('cBC')
  idx_cPOM = um_get_spc_ndx_in_q ('cPOM') 
  
  idx_fBC  = um_get_spc_ndx_in_q ('fBC')
  idx_fOC  = um_get_spc_ndx_in_q ('fOC')

  idx_fBC1  = um_get_spc_ndx_in_q ('fBC1')
  idx_fOC1  = um_get_spc_ndx_in_q ('fOC1')

  idx_fBC2  = um_get_spc_ndx_in_q ('fBC2')
  idx_fOC2  = um_get_spc_ndx_in_q ('fOC2')
  
  idx_bBC  = um_get_spc_ndx_in_q ('bBC')
  idx_bOC  = um_get_spc_ndx_in_q ('bOC')
 
  idx_dst1  = um_get_spc_ndx_in_q ('dust1')
  idx_dst2  = um_get_spc_ndx_in_q ('dust2')    
  idx_dst3  = um_get_spc_ndx_in_q ('dust3')
  idx_dst4  = um_get_spc_ndx_in_q ('dust4')
  
  	if (masterproc) print*, 'idx_aBC,idx_aPOM',idx_aBC,idx_aPOM,idx_cBC,idx_cPOM !idx_fBC,idx_fOC !,idx_fBC1,idx_fOC1, idx_bBC,idx_bOC,idx_dst1,idx_dst2,idx_dst3,idx_dst4
  
  end subroutine um_aerosol_number_init
   
  subroutine um_IN_numbers (state, IN_hetero_dust,IN_hetero_asoot,IN_hetero_soot, IN_homo1, IN_homo2) !one chunk
!----------------------------------------------------------------------- 
!
! Purpose: calculate the IN number from aircraft soot
!
!-----------------------------------------------------------------------
   use cam_history,     only: outfld
   use ppgrid,          only: begchunk, endchunk
   use physics_types,   only: physics_state
   use um_aerosol_intr, only: dust_so4_layer
   implicit none
   
  ! Arguments:
   type(physics_state), intent(in) :: state          ! Physics state variables
   real(r8), dimension(pcols,pver) :: IN_hetero_dust,IN_hetero_asoot,IN_hetero_soot, IN_homo1, IN_homo2, INdust1, INdust2,INdust3,INdust4
   	
   	
   ! Local Variables
   integer m,k
   integer lchnk,ncol
   real(r8), dimension(pcols,pver) :: rho


!  Begin
   		
   	lchnk = state%lchnk
    ncol  = state%ncol
    
    rho(:ncol,:) = state%pmid(:ncol,:) / 287.0_r8 / state%t(:ncol,:)  ! p = rho R T
    
    IN_hetero_dust(:ncol,:) = 0.0_r8
    IN_hetero_asoot(:ncol,:) = 0.0_r8
    IN_hetero_soot(:ncol,:) = 0.0_r8
    IN_homo1(:ncol,:) = 0.0_r8
    IN_homo2(:ncol,:) = 0.0_r8
    INdust1(:ncol,:) = 0.0_r8
    INdust2(:ncol,:) = 0.0_r8
    INdust3(:ncol,:) = 0.0_r8
    INdust4(:ncol,:) = 0.0_r8		
    	
    if ( idx_cBC .gt. 0 )	then
      IN_hetero_asoot(:ncol,:) =                state%q(:ncol, :, idx_cBC ) / pm_aBC &
      	                                      + state%q(:ncol, :, idx_cPOM) / pm_aPOM!!! apply rho and 1e-6 late to save time     
    end if
   
 
    if ( idx_fBC .gt. 0 )	  then
     IN_hetero_soot(:ncol,:) = IN_hetero_soot(:ncol,:) + ( state%q(:ncol, :, idx_fBC)  / pm_soot &
     	                                                 +   state%q(:ncol, :, idx_fOC)  / pm_soot )   * 0.0005_r8 ! 0.05% nucleating effciency ,  apply rho and 1e-6 late to save time     
    end if    	  		
   
    if ( idx_fBC1 .gt. 0 )	then
     IN_hetero_soot(:ncol,:) = IN_hetero_soot(:ncol,:)  + ( state%q(:ncol, :, idx_fBC1) / pm_soot &
     	                                                  +   state%q(:ncol, :, idx_fOC1) / pm_soot)    * 0.001_r8 ! 0.1% nucleating effciency!!! apply rho and 1e-6 late to save time     
    end if

    if ( idx_bBC .gt. 0 )	then
     IN_hetero_soot(:ncol,:) = IN_hetero_soot(:ncol,:)  + ( state%q(:ncol, :, idx_bBC) / pm_soot &
     	                                                  +   state%q(:ncol, :, idx_bOC) / pm_soot)    * 0.001_r8 ! 0.1% nucleating effciency!!! apply rho and 1e-6 late to save time     
    end if
    	
!    if ( idx_dst1 .gt. 0 )	then
!     IN_hetero_dust(:ncol,:) =                 ( state%q(:ncol, :, idx_dst1 ) / pm_dst1 &
!     	                                       +   state%q(:ncol, :, idx_dst2 ) / pm_dst2 &
!     	                                       +   state%q(:ncol, :, idx_dst3 ) / pm_dst3 &
!     	                                       +   state%q(:ncol, :, idx_dst4 ) / pm_dst4 )  * 0.1_r8 ! 10% nucleating effciency!!! apply rho and 1e-6 late to save time     
!    end if

     INdust1(:ncol,:) = state%q(:ncol, :, idx_dst1 ) / pm_dst1 * rho(:ncol,:) *1e-6
     INdust2(:ncol,:) = state%q(:ncol, :, idx_dst2 ) / pm_dst2 * rho(:ncol,:) *1e-6
     INdust3(:ncol,:) = state%q(:ncol, :, idx_dst3 ) / pm_dst3 * rho(:ncol,:) *1e-6
     INdust4(:ncol,:) = state%q(:ncol, :, idx_dst4 ) / pm_dst4 * rho(:ncol,:) *1e-6
     
     !apply so4 coating layer mask
     where (dust_so4_layer(lchnk,:,:,1) > 3.0d0) INdust1(:,:) = 0.0d0 	 !dust_so4_layer(:,:,:,:) 	!(begchunk:endchunk,pcols,pver,4)
     where (dust_so4_layer(lchnk,:,:,2) > 3.0d0) INdust2(:,:) = 0.0d0 	 !dust_so4_layer(:,:,:,:) 	!(begchunk:endchunk,pcols,pver,4)
     where (dust_so4_layer(lchnk,:,:,3) > 3.0d0) INdust3(:,:) = 0.0d0 	 !dust_so4_layer(:,:,:,:) 	!(begchunk:endchunk,pcols,pver,4)
     where (dust_so4_layer(lchnk,:,:,4) > 3.0d0) INdust4(:,:) = 0.0d0 	 !dust_so4_layer(:,:,:,:) 	!(begchunk:endchunk,pcols,pver,4)

! haze number for homogenous freezing                                                           ! apply tuning factor 0.5 here
   IN_homo1(:ncol,:) =     state%q(:ncol, :, idx_SO4n2)   * 1e10_r8 !* 0.5 & ! SO4n1/2/3 are multiplied by 1e-10 in um_aerosol_intr/um_const_2_q ,need to restore the magnitude here
   IN_homo2(:ncol,:) =     state%q(:ncol, :, idx_SO4n3)   * 1e10_r8
 !  	                   + ( state%q(:ncol, :, idx_fBC2) / pm_soot  &
 !    	                 +   state%q(:ncol, :, idx_fOC2) / pm_soot)  * 0.1_r8 ! 10% hygroscopic fossil fuel soot !!! apply rho and 1e-6 late to save time     
     	                                       	
   ! apply rho and 1e-6 here
   ! IN_hetero(:ncol,:) = IN_hetero(:ncol,:) * rho(:ncol,:) *1e-6 ! kg/kg to kg/m3 then to (#/m3, done above) then to #/cm3  
   ! IN_homo(:ncol,:)   = IN_homo(:ncol,:)   * rho(:ncol,:) *1e-6 ! kg/kg to kg/m3 then to (#/m3, done above) then to #/cm3      	

    IN_hetero_dust(:ncol,:)  = INdust1(:ncol,:) + INdust2(:ncol,:)+ INdust3(:ncol,:) +INdust4(:ncol,:)
    IN_hetero_asoot(:ncol,:) = IN_hetero_asoot(:ncol,:)* rho(:ncol,:) *1e-6 !*0.0 ! turn off
    IN_hetero_soot(:ncol,:)  = IN_hetero_soot(:ncol,:)* rho(:ncol,:) *1e-6 !*0.0 ! turn off
    IN_homo1(:ncol,:) = IN_homo1(:ncol,:)* rho(:ncol,:) *1e-6
    IN_homo2(:ncol,:) = IN_homo2(:ncol,:)* rho(:ncol,:) *1e-6
    	
!    IN_hetero(:ncol,:) = IN_hetero_dust(:ncol,:)+IN_hetero_asoot(:ncol,:)+IN_hetero_soot(:ncol,:)
!    IN_homo(:ncol,:)   = IN_homo1(:ncol,:)+ IN_homo2(:ncol,:)
!		

	  call outfld('um_IN_hetero_dust' , IN_hetero_dust(:ncol,:) , ncol, lchnk)
	  call outfld('um_IN_hetero_dust1' , INdust1(:ncol,:) , ncol, lchnk)
	  call outfld('um_IN_hetero_dust2' , INdust2(:ncol,:) , ncol, lchnk)
	  call outfld('um_IN_hetero_dust3' , INdust3(:ncol,:) , ncol, lchnk)
	  call outfld('um_IN_hetero_dust4' , INdust4(:ncol,:) , ncol, lchnk)
	  				
	  call outfld('um_IN_hetero_asoot' , IN_hetero_asoot(:ncol,:) , ncol, lchnk)
	 	call outfld('um_IN_hetero_soot' , IN_hetero_soot(:ncol,:) , ncol, lchnk) 	
	  	
	  call outfld('um_IN_homo1' , IN_homo1(:ncol,:) , ncol, lchnk)	
	  call outfld('um_IN_homo2' , IN_homo2(:ncol,:) , ncol, lchnk)	
	  		  	       	   
  end subroutine um_IN_numbers


  !============================================================================ !
  !                                                                             !
  !============================================================================ !
       
  end module  um_aerosol_number
