!---------------------
!Purpose: read in hourly aircraft emissions. 
!Created by Cheng Zhou, zhouc@umich.edu, 2013-03-05
!--------------

   module  um_read_aircraft_intr
   use shr_kind_mod,     only: r8=>shr_kind_r8
   use cam_history,      only: addfld, add_default, phys_decomp
   use ioFileMod,        only: getfil
   use pio
   use cam_pio_utils,    only : cam_pio_openfile
   use phys_grid,        only : get_ncols_p, get_rlat_all_p, get_rlon_all_p
   use interpolate_data, only : lininterp_init, lininterp, lininterp_finish, interp_type
   use ppgrid,           only : begchunk, endchunk, pcols, pver, pverp
   use mo_constants,     only : pi, d2r
   use abortutils,       only : endrun
   use cam_history,      only : outfld		
   use cam_logfile,      only : iulog
   use time_manager,     only : get_curr_date
   		
   implicit none
   private
   save
   
   real(r8), allocatable ::  var_in_4d(:,:,:,:),aBC(:,:,:,:),aPOM(:,:,:,:),aSO2(:,:,:,:),aSO4m(:,:,:,:)  !  	
   real(r8), allocatable :: in_lons(:)
   real(r8), allocatable :: in_lats(:)
   character(256) ::  in_file_base="/glade/work/jialeiz/input_SOA/airsoot_1.9x2.5x30_2006" !hard coded here for now. Change of this line is not necessary if you put the file in the working directory since getfil will always search working directory first
!   character(256) ::  in_file_aSO4m, in_file_aBC,in_file_aPOM,in_file_aSO2
   character(len=2)  ::  mon_name(12)
   data mon_name/"01","02","03","04","05","06","07","08","09","10","11","12"/
    	  	
   
   public  um_readin_aircraft ! readin 
   public  um_get_aircraft  ! 
   
   contains
   
   subroutine  um_readin_aircraft  !! all chunks, called by phys_run1
   implicit none
   integer,save :: readin_mon = -999, readin_day = -999
   
   integer :: tod               ! time of day relative to current date [seconds]
   integer :: yr, mon, day      ! year, month, day components of a date
   	
   		
   call get_curr_date(yr, mon, day, tod)
   
   
   if ( (mon /= readin_mon) .or. (day /= readin_day) ) then 
   	   readin_mon  = mon
   	   readin_day  = day
   	   call um_readin_aBC   (mon,day)
   	   call um_readin_aPOM  (mon,day)
   	   call um_readin_aSO2  (mon,day)
   	   call um_readin_aSO4m (mon,day)
   end if
   	
!       nymd =   MOD(yr,100)*10000   +                     mon*100 + day
!       nhms = INT(tod/3600)*10000   +   INT(MOD(tod,3600)/60)*100 + MOD(tod,60)
!       
       
     
   end subroutine  um_readin_aircraft
   
  !============================================================================ !
  !                                                                             !
  !============================================================================ !   
   subroutine  um_readin_aBC(mon,day) ! read in one day emission, read in all chunks, so should be called outside one chunk
   implicit none
!  input argument
   integer          :: mon,day! current month
  
! local variables
   integer            :: m,irec
   character(256)     :: in_file0,in_file
   integer            :: did, vid, nlat, nlon, nlev, nrec
   type(file_desc_t)  :: ncid
   type(interp_type)  :: lon_wgts, lat_wgts
   real(r8)           :: to_lats(pcols), to_lons(pcols)

   	
   integer             :: c, ncols, ierr
   real(r8), parameter :: zero=0._r8, twopi=2._r8*pi

!
    integer :: start(4)
    integer :: count(4)
    

   	   
 ! Get file name.
   in_file0 = trim(in_file_base)//mon_name(mon)//"_aBC.nc"
   print*, "aircraft emission readin :", in_file0
   call getfil(in_file0, in_file, 0)
   call cam_pio_openfile (ncid, trim(in_file), PIO_NOWRITE)
   
   
! Get input data resolution.
   ierr = pio_inq_dimid( ncid, 'lon', did )
   ierr = pio_inq_dimlen( ncid, did, nlon )

   ierr = pio_inq_dimid( ncid, 'lat', did )
   ierr = pio_inq_dimlen( ncid, did, nlat )

   ierr = pio_inq_dimid( ncid, 'lev', did )
   ierr = pio_inq_dimlen( ncid, did, nlev )
   
   ierr = pio_inq_dimid( ncid, 'time', did )
   ierr = pio_inq_dimlen( ncid, did, nrec )   
 
    if( (nrec > 744) .or. (nlev /= pver) )then
       write(iulog,*) 'um_readin_qj: expecting time <= 144 and lev = pver'
       call endrun
    end if
    	  

   
   if( .not. allocated(var_in_4d)) then !first time
   allocate(var_in_4d (nlon,nlat,nlev,24)) !! specific to the input file, nrec should be 12 and nlev = pver
   allocate(in_lons(nlon))
   allocate(in_lats(nlat))
   end if
   	
   ierr = pio_inq_varid( ncid, 'lon', vid )
   ierr = pio_get_var( ncid, vid, in_lons  )

   ierr = pio_inq_varid( ncid, 'lat', vid )
   ierr = pio_get_var( ncid, vid, in_lats  )

    start = (/ 1, 1, 1, (day-1)*24 + 1 /)
    count = (/ nlon, nlat, nlev, 24 /)
   ierr = pio_inq_varid( ncid, 'aBC', vid ) !! specific to the input file
   ierr = pio_get_var( ncid, vid, start, count, var_in_4d )
 
 

!-----------------------------------------------------------------------
!     	... convert to radians and setup regridding
!-----------------------------------------------------------------------
    in_lats(:) = d2r * in_lats(:)
    in_lons(:) = d2r * in_lons(:)

!--------------allocate emission arrays --------------    
    if( .not. allocated(aBC)) then
    allocate( aBC(begchunk:endchunk,pcols,pver,24) ) !! specific to the input file (begchunk:endchunk, pcols,pver,totmonth)
    end if 	
    	
!-----------------------------------------------------------------------
!     	... regrid ..
!-----------------------------------------------------------------------
    do c=begchunk,endchunk
       ncols = get_ncols_p(c)
       call get_rlat_all_p(c, pcols, to_lats)
       call get_rlon_all_p(c, pcols, to_lons)
       
       call lininterp_init(in_lons, nlon, to_lons, ncols, 2, lon_wgts, zero, twopi)
       call lininterp_init(in_lats, nlat, to_lats, ncols, 1, lat_wgts)
         
         do irec = 1, 24
            do m = 1, nlev
             call lininterp(var_in_4d(:,:,m,irec), nlon,nlat , aBC(c,:,m,irec), ncols, lon_wgts,lat_wgts)
          	end do
         end do  	

       call lininterp_finish(lat_wgts)
       call lininterp_finish(lon_wgts)
    end do

    
!    deallocate( var_in_4d, stat=ierr )
!    if( ierr /= 0 ) then
!       write(iulog,*) 'um_readin_qj: failed',ierr
!       call endrun
!    end if
!   
   end subroutine  um_readin_aBC
  !============================================================================ !
  !                                                                             !
  !============================================================================ !   
   subroutine  um_readin_aPOM(mon,day) ! read in one day emission, read in all chunks, so should be called outside one chunk
   implicit none
!  input argument
   integer          :: mon,day! current month
  
! local variables
   integer            :: m,irec
   character(256)     :: in_file0,in_file
   integer            :: did, vid, nlat, nlon, nlev, nrec
   type(file_desc_t)  :: ncid
   type(interp_type)  :: lon_wgts, lat_wgts
   real(r8)           :: to_lats(pcols), to_lons(pcols)
   	

   	
   integer             :: c, ncols, ierr
   real(r8), parameter :: zero=0._r8, twopi=2._r8*pi

!
    integer :: start(4)
    integer :: count(4)
    

   	   
 ! Get file name.
   in_file0 = trim(in_file_base)//mon_name(mon)//"_aPOM.nc"
   print*, "aircraft emission readin :", in_file0
   call getfil(in_file0, in_file, 0)
   call cam_pio_openfile (ncid, trim(in_file), PIO_NOWRITE)
   
   
! Get input data resolution.
   ierr = pio_inq_dimid( ncid, 'lon', did )
   ierr = pio_inq_dimlen( ncid, did, nlon )

   ierr = pio_inq_dimid( ncid, 'lat', did )
   ierr = pio_inq_dimlen( ncid, did, nlat )

   ierr = pio_inq_dimid( ncid, 'lev', did )
   ierr = pio_inq_dimlen( ncid, did, nlev )
   
   ierr = pio_inq_dimid( ncid, 'time', did )
   ierr = pio_inq_dimlen( ncid, did, nrec )   
 
    if( (nrec > 744) .or. (nlev /= pver) )then
       write(iulog,*) 'um_readin_qj: expecting time <= 144 and lev = pver'
       call endrun
    end if
    	  

   
   if( .not. allocated(var_in_4d)) then !first time
   allocate(var_in_4d (nlon,nlat,nlev,24)) !! specific to the input file, nrec should be 12 and nlev = pver
   allocate(in_lons(nlon))
   allocate(in_lats(nlat))
   end if
   	
   ierr = pio_inq_varid( ncid, 'lon', vid )
   ierr = pio_get_var( ncid, vid, in_lons  )

   ierr = pio_inq_varid( ncid, 'lat', vid )
   ierr = pio_get_var( ncid, vid, in_lats  )

    start = (/ 1, 1, 1, (day-1)*24 + 1 /)
    count = (/ nlon, nlat, nlev, 24 /)
   ierr = pio_inq_varid( ncid, 'aPOM', vid ) !! specific to the input file
   ierr = pio_get_var( ncid, vid, start, count, var_in_4d )
 
 

!-----------------------------------------------------------------------
!     	... convert to radians and setup regridding
!-----------------------------------------------------------------------
    in_lats(:) = d2r * in_lats(:)
    in_lons(:) = d2r * in_lons(:)

!--------------allocate emission arrays --------------    
    if( .not. allocated(aPOM)) then
    allocate( aPOM(begchunk:endchunk,pcols,pver,24) ) !! specific to the input file (begchunk:endchunk, pcols,pver,totmonth)
    end if 	
    	
!-----------------------------------------------------------------------
!     	... regrid ..
!-----------------------------------------------------------------------
    do c=begchunk,endchunk
       ncols = get_ncols_p(c)
       call get_rlat_all_p(c, pcols, to_lats)
       call get_rlon_all_p(c, pcols, to_lons)
       
       call lininterp_init(in_lons, nlon, to_lons, ncols, 2, lon_wgts, zero, twopi)
       call lininterp_init(in_lats, nlat, to_lats, ncols, 1, lat_wgts)
         
         do irec = 1, 24
            do m = 1, nlev
             call lininterp(var_in_4d(:,:,m,irec), nlon,nlat , aPOM(c,:,m,irec), ncols, lon_wgts,lat_wgts)
          	end do
         end do  	

       call lininterp_finish(lat_wgts)
       call lininterp_finish(lon_wgts)
    end do

    
!    deallocate( var_in_4d, stat=ierr )
!    if( ierr /= 0 ) then
!       write(iulog,*) 'um_readin_qj: failed',ierr
!       call endrun
!    end if
!   
   end subroutine  um_readin_aPOM
!============================================================================ !
  !                                                                             !
  !============================================================================ !   
   subroutine  um_readin_aSO2(mon,day) ! read in one day emission, read in all chunks, so should be called outside one chunk
   implicit none
!  input argument
   integer          :: mon,day! current month
  
! local variables
   integer            :: m,irec
   character(256)     :: in_file0,in_file
   integer            :: did, vid, nlat, nlon, nlev, nrec
   type(file_desc_t)  :: ncid
   type(interp_type)  :: lon_wgts, lat_wgts
   real(r8)           :: to_lats(pcols), to_lons(pcols)
   	

   integer             :: c, ncols, ierr
   real(r8), parameter :: zero=0._r8, twopi=2._r8*pi

!
    integer :: start(4)
    integer :: count(4)
    

   	   
 ! Get file name.
   in_file0 = trim(in_file_base)//mon_name(mon)//"_aSO2.nc"
   print*, "aircraft emission readin :", in_file0
   call getfil(in_file0, in_file, 0)
   call cam_pio_openfile (ncid, trim(in_file), PIO_NOWRITE)
   
   
! Get input data resolution.
   ierr = pio_inq_dimid( ncid, 'lon', did )
   ierr = pio_inq_dimlen( ncid, did, nlon )

   ierr = pio_inq_dimid( ncid, 'lat', did )
   ierr = pio_inq_dimlen( ncid, did, nlat )

   ierr = pio_inq_dimid( ncid, 'lev', did )
   ierr = pio_inq_dimlen( ncid, did, nlev )
   
   ierr = pio_inq_dimid( ncid, 'time', did )
   ierr = pio_inq_dimlen( ncid, did, nrec )   
 
    if( (nrec > 744) .or. (nlev /= pver) )then
       write(iulog,*) 'um_readin_qj: expecting time <= 144 and lev = pver'
       call endrun
    end if
    	  

   
   if( .not. allocated(var_in_4d)) then !first time
   allocate(var_in_4d (nlon,nlat,nlev,24)) !! specific to the input file, nrec should be 12 and nlev = pver
   allocate(in_lons(nlon))
   allocate(in_lats(nlat))
   end if
   	
   ierr = pio_inq_varid( ncid, 'lon', vid )
   ierr = pio_get_var( ncid, vid, in_lons  )

   ierr = pio_inq_varid( ncid, 'lat', vid )
   ierr = pio_get_var( ncid, vid, in_lats  )

    start = (/ 1, 1, 1, (day-1)*24 + 1 /)
    count = (/ nlon, nlat, nlev, 24 /)
   ierr = pio_inq_varid( ncid, 'aSO2', vid ) !! specific to the input file
   ierr = pio_get_var( ncid, vid, start, count, var_in_4d )
 
 

!-----------------------------------------------------------------------
!     	... convert to radians and setup regridding
!-----------------------------------------------------------------------
    in_lats(:) = d2r * in_lats(:)
    in_lons(:) = d2r * in_lons(:)

!--------------allocate emission arrays --------------    
    if( .not. allocated(aSO2)) then
    allocate( aSO2(begchunk:endchunk,pcols,pver,24) ) !! specific to the input file (begchunk:endchunk, pcols,pver,totmonth)
    end if 	
    	
!-----------------------------------------------------------------------
!     	... regrid ..
!-----------------------------------------------------------------------
    do c=begchunk,endchunk
       ncols = get_ncols_p(c)
       call get_rlat_all_p(c, pcols, to_lats)
       call get_rlon_all_p(c, pcols, to_lons)
       
       call lininterp_init(in_lons, nlon, to_lons, ncols, 2, lon_wgts, zero, twopi)
       call lininterp_init(in_lats, nlat, to_lats, ncols, 1, lat_wgts)
         
         do irec = 1, 24
            do m = 1, nlev
             call lininterp(var_in_4d(:,:,m,irec), nlon,nlat , aSO2(c,:,m,irec), ncols, lon_wgts,lat_wgts)
          	end do
         end do  	

       call lininterp_finish(lat_wgts)
       call lininterp_finish(lon_wgts)
    end do

    
!    deallocate( var_in_4d, stat=ierr )
!    if( ierr /= 0 ) then
!       write(iulog,*) 'um_readin_qj: failed',ierr
!       call endrun
!    end if
!   
   end subroutine  um_readin_aSO2
!============================================================================ !
  !                                                                             !
  !============================================================================ !   
   subroutine  um_readin_aSO4m(mon,day) ! read in one day emission, read in all chunks, so should be called outside one chunk
   implicit none
!  input argument
   integer          :: mon,day! current month
  
! local variables
   integer            :: m,irec
   character(256)     :: in_file0,in_file
   integer            :: did, vid, nlat, nlon, nlev, nrec
   type(file_desc_t)  :: ncid
   type(interp_type)  :: lon_wgts, lat_wgts
   real(r8)           :: to_lats(pcols), to_lons(pcols)
   	
   	
   integer             :: c, ncols, ierr
   real(r8), parameter :: zero=0._r8, twopi=2._r8*pi

!
    integer :: start(4)
    integer :: count(4)
    

   	   
 ! Get file name.
   in_file0 = trim(in_file_base)//mon_name(mon)//"_aSO4m.nc"
   print*, "aircraft emission readin :", in_file0
   call getfil(in_file0, in_file, 0)
   call cam_pio_openfile (ncid, trim(in_file), PIO_NOWRITE)
   
   
! Get input data resolution.
   ierr = pio_inq_dimid( ncid, 'lon', did )
   ierr = pio_inq_dimlen( ncid, did, nlon )

   ierr = pio_inq_dimid( ncid, 'lat', did )
   ierr = pio_inq_dimlen( ncid, did, nlat )

   ierr = pio_inq_dimid( ncid, 'lev', did )
   ierr = pio_inq_dimlen( ncid, did, nlev )
   
   ierr = pio_inq_dimid( ncid, 'time', did )
   ierr = pio_inq_dimlen( ncid, did, nrec )   
 
    if( (nrec > 744) .or. (nlev /= pver) )then
       write(iulog,*) 'um_readin_qj: expecting time <= 144 and lev = pver'
       call endrun
    end if
    	  

   
   if( .not. allocated(var_in_4d)) then !first time
   allocate(var_in_4d (nlon,nlat,nlev,24)) !! specific to the input file, nrec should be 12 and nlev = pver
   allocate(in_lons(nlon))
   allocate(in_lats(nlat))
   end if
   	
   ierr = pio_inq_varid( ncid, 'lon', vid )
   ierr = pio_get_var( ncid, vid, in_lons  )

   ierr = pio_inq_varid( ncid, 'lat', vid )
   ierr = pio_get_var( ncid, vid, in_lats  )

    start = (/ 1, 1, 1, (day-1)*24 + 1 /)
    count = (/ nlon, nlat, nlev, 24 /)
   ierr = pio_inq_varid( ncid, 'aSO4m', vid ) !! specific to the input file
   ierr = pio_get_var( ncid, vid, start, count, var_in_4d )
 
 

!-----------------------------------------------------------------------
!     	... convert to radians and setup regridding
!-----------------------------------------------------------------------
    in_lats(:) = d2r * in_lats(:)
    in_lons(:) = d2r * in_lons(:)

!--------------allocate emission arrays --------------    
    if( .not. allocated(aSO4m)) then
    allocate( aSO4m(begchunk:endchunk,pcols,pver,24) ) !! specific to the input file (begchunk:endchunk, pcols,pver,totmonth)
    end if 	
    	
!-----------------------------------------------------------------------
!     	... regrid ..
!-----------------------------------------------------------------------
    do c=begchunk,endchunk
       ncols = get_ncols_p(c)
       call get_rlat_all_p(c, pcols, to_lats)
       call get_rlon_all_p(c, pcols, to_lons)
       
       call lininterp_init(in_lons, nlon, to_lons, ncols, 2, lon_wgts, zero, twopi)
       call lininterp_init(in_lats, nlat, to_lats, ncols, 1, lat_wgts)
         
         do irec = 1, 24
            do m = 1, nlev
             call lininterp(var_in_4d(:,:,m,irec), nlon,nlat , aSO4m(c,:,m,irec), ncols, lon_wgts,lat_wgts)
          	end do
         end do  	

       call lininterp_finish(lat_wgts)
       call lininterp_finish(lon_wgts)
    end do

    
!    deallocate( var_in_4d, stat=ierr )
!    if( ierr /= 0 ) then
!       write(iulog,*) 'um_readin_qj: failed',ierr
!       call endrun
!    end if
!   
   end subroutine  um_readin_aSO4m
  !============================================================================ !
  !                                                                             !
  !============================================================================ !  
   subroutine um_get_aircraft(aBC_emis,aPOM_emis,aSO2_emis,aSO4m_emis, ncol,lchnk) ! one chunk
   implicit none
   
   integer  :: ncol,lchnk
   real(r8) :: aBC_emis(ncol,pver),aSO2_emis(ncol,pver),aPOM_emis(ncol,pver),aSO4m_emis(ncol,pver)

  !local

   integer :: tod               ! time of day relative to current date [seconds]
   integer :: yr, mon, day      ! year, month, day components of a date
   integer :: curr_h            ! current hour
   		
   call get_curr_date(yr, mon, day, tod)
   
   curr_h = INT(tod/3600) + 1
   !print*, 'mon_day =', mon*100 + day , 'tod=',tod
   aBC_emis   (:ncol,:pver) = aBC  (lchnk,:ncol,:pver, curr_h)
   aPOM_emis  (:ncol,:pver) = aPOM (lchnk,:ncol,:pver, curr_h)
   aSO2_emis  (:ncol,:pver) = aSO2 (lchnk,:ncol,:pver, curr_h)
   aSO4m_emis (:ncol,:pver) = aSO4m(lchnk,:ncol,:pver, curr_h)
   				
   end subroutine um_get_aircraft
  !============================================================================ !
  !                                                                             !
  !============================================================================ !    

   
   
  end module um_read_aircraft_intr
  
