   module  um_readin
   use shr_kind_mod,     only: r8=>shr_kind_r8
   use cam_history,      only: addfld, add_default, phys_decomp
   use ioFileMod,        only: getfil
   use pio
   use cam_pio_utils,    only : cam_pio_openfile
   use phys_grid,        only : get_ncols_p, get_rlat_all_p, get_rlon_all_p
   use interpolate_data, only : lininterp_init, lininterp, lininterp_finish, interp_type
   use ppgrid,           only : begchunk, endchunk, pcols, pver, pverp
   use mo_constants,     only : pi, d2r
   use abortutils,       only : endrun
   use cam_history,      only : outfld		
   use cam_logfile,      only : iulog
   	
   implicit none
   private
   save
   
   real(r8), allocatable ::  qjh2o2(:,:,:,:) , var_in_4d(:,:,:,:)  !  	
   character(256) :: in_file_qj="/glade/work/jialeiz/input_SOA/qjh2o2_12x30x96x144.nc" !hard coded here for now. Change of this line is not necessary if you put the file in the working directory since getfil will always search working directory first

   real(r8), allocatable ::  iland(:,:,:), iuse(:,:,:),ixlai(:,:,:,:)
   integer,  allocatable ::  var_in_land(:,:,:), var_in_use(:,:,:), var_in_xlai(:,:,:,:)  ! 	
   character(256) :: in_file_veg="/glade/work/jialeiz/input_SOA/vegtype_leaf_index.nc" !hard coded here for now. Change of this line is not necessary if you put the file in the working directory since getfil will always search working directory first
    	  	
   
   public  um_readin_init ! readin 
   public  um_get_qj  ! get qj
   public  um_get_veg  ! get qj
   
   contains
   
   subroutine  um_readin_init
     call um_readin_qj
     call um_readin_veg
   end subroutine  um_readin_init
   
  !============================================================================ !
  !                                                                             !
  !============================================================================ !   
   subroutine  um_readin_qj()
   implicit none
! local variables
   integer            :: m,irec
   character(256)      :: in_file
   integer            :: did, vid, nlat, nlon, nlev, nrec
   type(file_desc_t)  :: ncid
   type(interp_type)  :: lon_wgts, lat_wgts
   real(r8)           :: to_lats(pcols), to_lons(pcols)
   	
   real(r8), allocatable :: in_lons(:)
   real(r8), allocatable :: in_lats(:)
   	
   integer             :: c, ncols, ierr
   real(r8), parameter :: zero=0._r8, twopi=2._r8*pi

   	   
 ! Get file name.
   call getfil(in_file_qj, in_file, 0)
   call cam_pio_openfile (ncid, trim(in_file), PIO_NOWRITE)
   
   
! Get input data resolution.
   ierr = pio_inq_dimid( ncid, 'lon', did )
   ierr = pio_inq_dimlen( ncid, did, nlon )

   ierr = pio_inq_dimid( ncid, 'lat', did )
   ierr = pio_inq_dimlen( ncid, did, nlat )

   ierr = pio_inq_dimid( ncid, 'lev', did )
   ierr = pio_inq_dimlen( ncid, did, nlev )
   
   ierr = pio_inq_dimid( ncid, 'time', did )
   ierr = pio_inq_dimlen( ncid, did, nrec )   
 
    if( (nrec /= 12) .or. (nlev /= pver) )then
       write(iulog,*) 'um_readin_qj: expecting time = 12 and lev = pver'
       call endrun
    end if
    	  
   allocate(in_lons(nlon))
   allocate(in_lats(nlat))
   allocate(var_in_4d (nlon,nlat,nlev,nrec)) !! specific to the input file, nrec should be 12 and nlev = pver

   ierr = pio_inq_varid( ncid, 'lon', vid )
   ierr = pio_get_var( ncid, vid, in_lons  )

   ierr = pio_inq_varid( ncid, 'lat', vid )
   ierr = pio_get_var( ncid, vid, in_lats  )

   ierr = pio_inq_varid( ncid, 'qjh2o2', vid ) !! specific to the input file
   ierr = pio_get_var( ncid, vid, var_in_4d ) !! read in


!-----------------------------------------------------------------------
!     	... convert to radians and setup regridding
!-----------------------------------------------------------------------
    in_lats(:) = d2r * in_lats(:)
    in_lons(:) = d2r * in_lons(:)

    allocate( qjh2o2(begchunk:endchunk,pcols,pver,12) ) !! specific to the input file (begchunk:endchunk, pcols,pver,totmonth)
!-----------------------------------------------------------------------
!     	... regrid ..
!-----------------------------------------------------------------------
    do c=begchunk,endchunk
       ncols = get_ncols_p(c)
       call get_rlat_all_p(c, pcols, to_lats)
       call get_rlon_all_p(c, pcols, to_lons)
       
       call lininterp_init(in_lons, nlon, to_lons, ncols, 2, lon_wgts, zero, twopi)
       call lininterp_init(in_lats, nlat, to_lats, ncols, 1, lat_wgts)
         
         do irec = 1, nrec
            do m = 1, nlev
             call lininterp(var_in_4d(:,:,m,irec), nlon,nlat , qjh2o2(c,:,m,irec), ncols, lon_wgts,lat_wgts)
          	end do
         end do  	

       call lininterp_finish(lat_wgts)
       call lininterp_finish(lon_wgts)
    end do
    
    deallocate( var_in_4d, stat=ierr )
    if( ierr /= 0 ) then
       write(iulog,*) 'um_readin_qj: failed',ierr
       call endrun
    end if
   
   end subroutine  um_readin_qj
   !============================================================================ !
  !                                                                             !
  !============================================================================ !  
   subroutine um_get_qj(qjimp,ncol,lchnk,imon)
   implicit none
   
   integer  :: ncol,lchnk,imon
   real(r8) :: qjimp(ncol,pver)
   	
   qjimp (:ncol,:pver) = qjh2o2(lchnk,:ncol,:pver,imon)
   	
   end subroutine um_get_qj
  !============================================================================ !
  !                                                                             !
  !============================================================================ !    
   subroutine  um_readin_veg()
   implicit none
   
! local variables
   integer            :: m,irec
   character(256)      :: in_file
   integer            :: did, vid, nlat, nlon, nlev, nrec, nmonth
   type(file_desc_t)  :: ncid
   type(interp_type)  :: lon_wgts, lat_wgts
   real(r8)           :: to_lats(pcols), to_lons(pcols)
   	
   real(r8), allocatable :: in_lons(:)
   real(r8), allocatable :: in_lats(:)
   	
   integer             :: c, ncols, ierr
   real(r8), parameter :: zero=0._r8, twopi=2._r8*pi

   	   
 ! Get file name.
   call getfil(in_file_veg, in_file, 0)
   call cam_pio_openfile (ncid, trim(in_file), PIO_NOWRITE)
   
   
! Get input data resolution.
   ierr = pio_inq_dimid( ncid, 'lon', did )
   ierr = pio_inq_dimlen( ncid, did, nlon )

   ierr = pio_inq_dimid( ncid, 'lat', did )
   ierr = pio_inq_dimlen( ncid, did, nlat )
   
   ierr = pio_inq_dimid( ncid, 'type', did )
   ierr = pio_inq_dimlen( ncid, did, nrec )   

   ierr = pio_inq_dimid( ncid, 'month', did )
   ierr = pio_inq_dimlen( ncid, did, nmonth ) 
    
    if( (nrec /= 16) .or. (nmonth /= 12) )then
       write(iulog,*) 'um_readin_veg: expecting month = 12 and type = 16'
       call endrun
    end if
    	  
   allocate(in_lons(nlon))
   allocate(in_lats(nlat))
   allocate(var_in_land (nlon,nlat,nrec)) !! 
   allocate(var_in_use  (nlon,nlat,nrec)) !! 
   allocate(var_in_xlai (nlon,nlat,nrec,nmonth)) !! 
   
   ierr = pio_inq_varid( ncid, 'lon', vid )
   ierr = pio_get_var( ncid, vid, in_lons  )

   ierr = pio_inq_varid( ncid, 'lat', vid )
   ierr = pio_get_var( ncid, vid, in_lats  )

   ierr = pio_inq_varid( ncid, 'land', vid ) !! 
   ierr = pio_get_var( ncid, vid, var_in_land ) !! 

   ierr = pio_inq_varid( ncid, 'use', vid ) !! 
   ierr = pio_get_var( ncid, vid, var_in_use ) !! 
   
   ierr = pio_inq_varid( ncid, 'xlai', vid ) !! 
   ierr = pio_get_var( ncid, vid, var_in_xlai ) !! 

!-----------------------------------------------------------------------
!     	... convert to radians and setup regridding
!-----------------------------------------------------------------------
    in_lats(:) = d2r * in_lats(:)
    in_lons(:) = d2r * in_lons(:)

    allocate( iland(begchunk:endchunk,pcols,nrec) ) !! 
    allocate( iuse (begchunk:endchunk,pcols,nrec) ) !! 
    allocate( ixlai(begchunk:endchunk,pcols,nrec,nmonth) ) !!  	
!-----------------------------------------------------------------------
!     	... regrid ..
!-----------------------------------------------------------------------
    do c=begchunk,endchunk
       ncols = get_ncols_p(c)
       call get_rlat_all_p(c, pcols, to_lats)
       call get_rlon_all_p(c, pcols, to_lons)
       
       call lininterp_init(in_lons, nlon, to_lons, ncols, 2, lon_wgts, zero, twopi)
       call lininterp_init(in_lats, nlat, to_lats, ncols, 1, lat_wgts)
         
         do irec = 1, nrec
             call lininterp(var_in_land(:,:,irec) * 1.0_r8, nlon,nlat , iland(c,:,irec), ncols, lon_wgts,lat_wgts)
             call lininterp(var_in_use (:,:,irec) * 1.0_r8, nlon,nlat , iuse (c,:,irec), ncols, lon_wgts,lat_wgts)	
             	do m = 1, nmonth
             	 call lininterp(var_in_xlai (:,:,irec,m) * 1.0_r8, nlon,nlat , ixlai (c,:,irec,m), ncols, lon_wgts,lat_wgts)	
             	end do
         end do  	

       call lininterp_finish(lat_wgts)
       call lininterp_finish(lon_wgts)
    end do
    
    deallocate( var_in_land, stat=ierr )
    deallocate( var_in_use, stat=ierr )
    deallocate( var_in_xlai, stat=ierr )

    if( ierr /= 0 ) then
       write(iulog,*) 'um_readin_veg: failed to deallocate soil_erod_in, ierr = ',ierr
       call endrun
    end if
   
   end subroutine  um_readin_veg
  !============================================================================ !
  !                                                                             !
  !============================================================================ !   
   subroutine um_get_veg(ireg, iland_out, iuse_out, ixlai_out, ncol,lchnk,ntype)
   use time_manager, only : get_curr_date
   
   implicit none
   	
   integer  :: ncol,lchnk,ntype
   real(r8) :: ixlai_out(pcols,ntype)
   integer  :: iland_out(pcols,ntype), iuse_out(pcols,ntype), ireg(pcols)
   
   ! local
   integer  :: yr, mon, day, tod
   	
  
   
   ireg      (:ncol       ) = IDNINT(iland(lchnk,:ncol,1)) !rounding
   iland_out (:ncol,:ntype) = IDNINT(iland(lchnk,:ncol,2:(ntype+1))) !rounding
   iuse_out  (:ncol,:ntype) = IDNINT(iuse (lchnk,:ncol,2:(ntype+1)))	
   	
   call get_curr_date(yr, mon, day, tod)	
   ixlai_out (:ncol,:ntype) = ixlai(lchnk,:ncol,2:(ntype+1),mon) *0.1_r8 ! the xlai in the input netcdf was applied a factor of 10 to it integer.
   		  	
   end subroutine um_get_veg
   
   end module um_readin
   
   
   
!       integer :: start(3)
!    integer :: count(3)
!    
!    start = (/ 1, 1, 1 /)
!    count = (/ nlon, nlat, ntimes /)
!    
!    ierr = pio_get_var( pio_id, vid, start, count, tropp_p_in )
