      module um_sslt_intr
      use shr_kind_mod,      only: r8 => shr_kind_r8
      use spmd_utils,        only: masterproc
      use ppgrid,            only: pcols, pver, pverp
!----      
      use um_aerosol_intr,   only: um_emis_srf, um_emis_srf_has       
      
      implicit none
      private          ! Make default type private to the module
      save

      public um_sslt_emiss





      contains 
c-----------------------------------------------------------------------------
c
c ROUTINE
c   um_sslt_emiss !added by Cheng zhou, surface emission only
c   
c DESCRIPTION
c   This routine updates const based on sea salt emissions.
c   when the dry particle diameter < 2.8 um, use Martensson et al 2003 scheme. Not good for low temperature.
C        When the temperature is < 0C, the flux could be negative according to this formula. See Martensson et al 2003 [27] for its applicability. 
c   when the dry particle diameter > 2.8 um, use Gong1997(essentially Monahan1986) scheme.
c   In CAM5 3mode version, the range of sslt is  0.02um<dry particle diameter<10um
c   In this version of Impact, the range is 0.05um [dry particle radius] to 10um [dry particle radius]


c-----------------------------------------------------------------------------

      subroutine um_sslt_emiss(state, cam_in) ! one chunk 

      use cam_history,   only: outfld
      use physics_types, only: physics_state
      use camsrfexch,    only: cam_in_t


!-----
      use h_imp_aerosol, only: AER_DENSITY,SIZE_RANGE_HI,SIZE_RANGE_LO,C1,C2,C3,C4
      use h_setkin_depos,only: aerosol
      use h_setkin_par,  only: ISSLT1,ISSLT2,ISSLT3,ISSLT4
      use h_imp_phys_constants, only: IMP_PI,CMPM,MPCM
!-----

      
c     ----------------------
c     Argument declarations.
c     ----------------------
      type(physics_state),    intent(in)   :: state   ! Physics state variables
      type(cam_in_t), target, intent(in)    :: cam_in  ! import state
   
c     -----------------------
c     Parameter declarations.
c     -----------------------

      real(r8) , parameter :: UMPCM = 1.0d4  ! micrometers per centimeter
      integer  , parameter :: NN    = 10  ! number of sub-bin in each bin
      real(r8) , parameter :: zo    = 0.0001_r8  ! follow CAM5, progseasalts_intr.F90
c     ----------------------
c     Local Variable declarations.
c     ----------------------
      integer   :: lchnk, ncol
      integer   :: ic, icol, ir

      real(r8), pointer :: ocnfrc(:), sst(:)

      real(r8)  :: w10m
      real(r8)  :: White ! white cap are fraction   
      real(r8)  :: delr
      real(r8)  :: relh
      real(r8)  :: radius_lo_dry, diameter_lo_dry
      real(r8)  :: radius_hi_dry, diameter_hi_dry
      real(r8)  :: radius_dry,radius_m, diameter_dry, diameter_dry_um
      real(r8)  :: pmass
      real(r8)  :: c3t
      real(r8)  :: size_lo !mosit
      real(r8)  :: size_hi !mosit
      real(r8)  :: del_size !moist
      real(r8)  :: bb,aa,Ak,Bk !add aa by Cheng Zhou
      real(r8)  :: salt_source
     
      real(r8)  :: temp1
      
c     ----------------
c     Begin execution.
c     ----------------
          lchnk = state%lchnk
          ncol  = state%ncol
          
          ocnfrc => cam_in%ocnfrac
          sst    => cam_in%sst
          
          um_emis_srf_has (ISSLT1:ISSLT4) = .true. ! always true 
          
       do icol = 1, ncol
           
        if (ocnfrc(icol) .gt. 0.0_r8) then 
          


c               ------------------------------------------
c               Wind speed at 10 meters above the surface.
c               ------------------------------------------      
                !zo = 0.0001_r8   ! follow CAM5, progseasalts_intr.F90, assume 
                w10m = sqrt(state%u(icol,pver)**2+state%v(icol,pver)**2) !wind speed at the center of lowest grid
                w10m = w10m * Log (10.0d0 / zo)/Log(state%zm(icol,pver)/zo) ! move the winds to 10m high from the midpoint of the gridbox
                
                White = 3.84d-6 * w10m**3.41 !fraction of the white cap area, divided by 100. Original W has a unit of percent.
 
c               ------------------------------------------------
c               Set surface relative humidity between 80% - 95%.
c               ------------------------------------------------
!                relh = Max (relhum(il,ij,k1), 0.80d0)
!                relh = Min (relh, 0.95d0)
                relh = 0.80 ! assume bubble emissions at 80% RH
          
                                
           
         do ic = ISSLT1, ISSLT4
                c3t = C3(aerosol(ic)) * (1.0d0 + 0.004d0 * (298.0d0 - sst(icol)))     
c         -------------------
c         Number of sub_bins.
c         -------------------

          delr = (SIZE_RANGE_HI(aerosol(ic)) - SIZE_RANGE_LO(aerosol(ic))) / DBLE(NN) ! in meter

          salt_source = 0.0d0

c           === loop over all sub bins ==================
          do ir = 1, NN
c               =====================

c - dry radius and diameter
                  radius_lo_dry = CMPM  * (SIZE_RANGE_LO(aerosol(ic)) + float (ir-1) * delr) ! from m to cm
                  radius_hi_dry = CMPM  * (SIZE_RANGE_LO(aerosol(ic)) + float (ir) * delr)  !  from m to cm
                  radius_dry = 0.5d0 * (radius_lo_dry + radius_hi_dry)* MPCM ! back to  meter

                  diameter_dry    = radius_dry    * 2.0d0         !meter
                  diameter_lo_dry = radius_lo_dry * 2.0d0 * MPCM  !meter
                  diameter_hi_dry = radius_hi_dry * 2.0d0 * MPCM  !meter
                  diameter_dry_um = diameter_dry * 1.0d6          !micron meter               
c                 ----------------------
c                 Mass of sea salt (kg).
c                 ----------------------
                  pmass = 4.0d0 * IMP_PI / 3.0d0 * radius_dry**3 * AER_DENSITY(aerosol(ic))


!------------------------------start to add,  4 ranges -----

c  d<0.145 , Use Martensson 2003
                  if (diameter_dry_um .le. 0.145) then
                    Ak = (-2.576)*10.**35*diameter_dry**4 + 5.932*10.**28* diameter_dry**3 + 
     &                   (-2.867)*10.**21*diameter_dry**2 + (-3.003)*10.**13*diameter_dry  +
     &                   (-2.881)*10.**6
                    Bk = 7.188*10.**37* diameter_dry**4   + (-1.616)*10.**31*diameter_dry**3 + 
     &                   6.791*10.**23* diameter_dry**2   + 1.829*10.**16*diameter_dry   + 
     &                   7.609*10.**8    
     
                    temp1 =    
     &              White *
     &              (Ak*sst(icol) + Bk) *
     &              log10(diameter_hi_dry/diameter_lo_dry) * ! dlog(D)
     &              pmass  
   
                    if (temp1 .gt. 0)    then 
                    salt_source = salt_source + temp1           
                    end if  
                                         
                  end if

c  0.145<d<0.419, Use Martensson 2003
                  if ((diameter_dry_um .gt. 0.145).and.(diameter_dry_um .le. 0.419)) then
                    Ak = (-2.452)*10.**33*diameter_dry**4 + 2.404*10.**27 * diameter_dry**3 + 
     &                   (-8.148)*10.**20*diameter_dry**2 + (1.183)*10.**14* diameter_dry   +
     &                   (-6.743)*10.**6
                    Bk = 7.368*10.**35*diameter_dry**4    + (-7.310)*10.**29*diameter_dry**3 +
     &                   2.528*10.**23*diameter_dry**2    + (-3.787)*10.**16*diameter_dry    +
     &                   2.279*10.**9
     
                    temp1 =    
     &              White *
     &              (Ak*sst(icol) + Bk) *
     &              log10(diameter_hi_dry/diameter_lo_dry) * ! dlog(D)
     &              pmass   
   
                    if (temp1 .gt. 0 )    then 
                    salt_source = salt_source + temp1           
                    end if  
                    

                  end if
                  
c  0.419<d<2.8, Use Martensson 2003
                  if ((diameter_dry_um .gt. 0.419).and.(diameter_dry_um .le. 2.8)) then
                    Ak = (1.085)*10.**29*diameter_dry**4  + (-9.841)*10.**23 * diameter_dry**3 + 
     &                   (3.132)*10.**18*diameter_dry**2  + (-4.165)*10.**12 * diameter_dry    +
     &                   (2.181)*10.**6
                    Bk = (-2.859)*10.**31* diameter_dry**4 + (2.601)*10.**26*diameter_dry**3   + 
     &                   (-8.297)*10.**20* diameter_dry**2 + (1.105)*10.**15*diameter_dry +
     &                   (-5.800)*10.**8
     
                    temp1 =    
     &              White *
     &              (Ak*sst(icol) + Bk) *
     &              log10(diameter_hi_dry/diameter_lo_dry) * ! dlog(D)
     &              pmass   
   
                    if (temp1 .gt.0 )    then 
                    salt_source = salt_source + temp1           
                    end if  
                    
                  end if

                                                      
c d > 2.8, use  Gong1997 scheme
                 if (diameter_dry_um .gt. 2.8) then

c                 ------------------------------------
c ----           Swelling of aerosol due to humidity, used for Gong1997 scheme
c                 ------------------------------------
                  
                  size_lo = UMPCM *
     &              (C1(aerosol(ic)) * radius_lo_dry**C2(aerosol(ic)) /
     &              (c3t * radius_lo_dry**C4(aerosol(ic)) -
     &              Log10 (relh)) + radius_lo_dry**3)**(1.0d0/3.0d0)  !from cm to um

                  size_hi = UMPCM *
     &              (C1(aerosol(ic)) * radius_hi_dry**C2(aerosol(ic)) /
     &              (c3t * radius_hi_dry**C4(aerosol(ic)) -
     &              Log10 (relh)) + radius_hi_dry**3)**(1.0d0/3.0d0)  !from cm to um

                  radius_m = 0.5d0 * (size_lo + size_hi)
                  
                  del_size = size_hi - size_lo !moist delta r in um
                  
                                    
                  bb = (0.38d0 - Log10 (radius_m)) / 0.65d0

                  temp1 =
     &              (1.373d0 * w10m**3.41d0 * radius_m**(-3.0d0) *
     &              (1.0d0 + 0.057d0 * radius_m**1.05d0) *
     &              10.0d0**(1.19d0 * exp(-bb * bb))) *
     &              del_size  * pmass 

                  salt_source = salt_source + temp1

                  end if
                  
                 
          
          end do ! subbin ir
             
          um_emis_srf(lchnk,icol,ic) = salt_source * ocnfrc(icol)
           
         end do ! isslt
           
 ! not over ocean, ocnfrc(icol) = 0
        else 
            um_emis_srf(lchnk,icol,ISSLT1:ISSLT4) = 0.0_r8
        end if
          
       end do  !ncol





       end subroutine um_sslt_emiss    


       end module um_sslt_intr