c=============================================================================
c
c $Id: gas_sulfchem.F,v 1.2 2003/06/30 22:50:11 dan Exp $
c
c CODE DEVELOPER
c   originated from GRANTOUR programmed by John Walton
c   modified by Xiaohong Liu
c
c FILE
c   gas_sulfchem.F
c
c ROUTINES
c   Do_Gas_Sulfchem
c
c=============================================================================

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Do_Gas_Sulfchem
c
c DESCRIPTION
c   This is the routine for the gas-chemistry
c
c ARGUMENTS
c   dtclc     : time step for clear air chemistry (s)
c   itloop    : # of zones (ilong * ilat * ivert)
c   cp        : species concentration (molecules/cm3 for gas-species, 
c               and g/g for aerosols)
c   dcp       : species concentration change
c   cfac      : total cloud fraction [0 - 1]
c   zmp       : air density (molecules/cm^3)
c   semiss    : array of sulfur emissions (molecules/cm^3/s)
c   oh, ho2, h2o, no3
c             : oh, ho2, h2o, no3 species concentrations (molecules/cm3)
c   qh1p, qh2p, qh3p, qj4p, qh5p, qh8p, qh9p
c             : gas phase reaction rate coefficient
c
c-----------------------------------------------------------------------------

c Change note by Cheng Zhou
c 1) change include to use

      subroutine Do_Gas_Sulfchem
     &  (dtclc, itloop, cp, dcp, cfac, zmp, semiss, srcgas,
     &   src_aso2, oh, ho2, h2o, no3,
     &   massc, pr_sulf_src, dms_oh, dms_no3, so2_oh,
     &   qh1p, qh2p, qh3p, qj4p, qh5p, qh8p, qh9p)


      use h_setkin_par
      use h_sulfchem, only: ijk
      
      implicit none

c#     include "setkin_par.h"
c#     include "sulfchem.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop

      real*8  :: dtclc

      real*8  :: oh (itloop)
      real*8  :: ho2(itloop)
      real*8  :: h2o(itloop)
      real*8  :: no3(itloop)

      real*8  :: qh1p(itloop)
      real*8  :: qh2p(itloop)
      real*8  :: qh3p(itloop)
      real*8  :: qj4p(itloop)
      real*8  :: qh5p(itloop)
      real*8  :: qh8p(itloop)
      real*8  :: qh9p(itloop)

      real*8  :: cp (itloop, NSP)
      real*8  :: dcp(itloop, NSP)

      real*8  :: cfac(itloop)
      real*8  :: zmp (itloop)
      real*8  :: semiss(itloop, NSP)
      real*8  :: srcgas(itloop)
      real*8  :: src_aso2(itloop)   !added by Roy
      
      real*8  :: massc(itloop)
      logical :: pr_sulf_src
      real*8  :: dms_oh (itloop)
      real*8  :: dms_no3(itloop)
      real*8  :: so2_oh (itloop)

c -------------------------------------------------------------
c  *** LOCAL  VARIABLES WERE DECLARED HERE.
c -------------------------------------------------------------

      real*8 :: alh2o2(itloop)
      real*8 :: exh2o2(itloop)
      real*8 :: ah2o2 (itloop)
      real*8 :: aldms (itloop)
      real*8 :: exdms (itloop)
      real*8 :: rdms  (itloop)
      real*8 :: also2 (itloop)
      real*8 :: exso2 (itloop)
      real*8 :: rso2  (itloop)

      alh2o2 = 0.0d0
      exh2o2 = 0.0d0
      ah2o2  = 0.0d0
      aldms  = 0.0d0
      exdms  = 0.0d0
      rdms   = 0.0d0
      also2  = 0.0d0
      exso2  = 0.0d0
      rso2   = 0.0d0


c.... compute the analytic integration terms
c
c          ======
      call chem10
c          ======
     &  (dtclc, itloop,
     &   oh, ho2, h2o, no3,
     &   qh1p, qh2p, qh3p, qj4p, qh5p, qh8p, qh9p,
     &   alh2o2, exh2o2, ah2o2,
     &   aldms, exdms, rdms,
     &   also2, exso2, rso2)

c.... compute species changes due to clear region chemistry over dtclc
c
c          ======
      call chem11
c          ======
     &  (dtclc, itloop, cp, dcp,
     &   oh, cfac, zmp, semiss, srcgas, src_aso2,
     &   massc, pr_sulf_src, dms_oh, dms_no3, so2_oh,
     &   alh2o2, exh2o2, ah2o2,
     &   aldms, exdms, rdms,
     &   also2, exso2, rso2)


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   chem10
c
c DESCRIPTION
c
c   compute the decay and "asymptotic" terms for the chemisty
c   analytic solutions in clear regions
c
c   dtclc  :  time step for clear air chemistry (s)
c   ohp, ho2p, h2op, no3p
c          :  oh, ho2, h2o, no3 species concentrations (molecules/cm3)
c   qh1p, qh2p, qh3p, qj4p, qh5p, qh8p, qh9p
c          :  gas phase reaction rate coefficient
c
c.... hydrogen peroxide
c     *     alh2o2 = qh4 + qh5 * OH
c     *     exh2o2 = exp( - alh2o2 * dtclc )
c     *     ah2o2  = (qh3 + qh8 * H2O) * HO2**2 / alh2o2
c
c.... dms
c     *     aldms = qh1 * OH + qh9 * NO3
c     *     exdms = exp( - aldms * dtclc )
c     *     rdms  = qh1 * OH / aldms
c
c.... sulfur dioxide
c     *     also2 = qh2 * OH
c     *     exso2 = exp( - also2 * dtclc )
c     *     rso2  = aldms / ( also2 - aldms )
c     *     aso2  = so2sor / also2                   not used
c
c----------------------------------------------------------------------

      subroutine chem10
     &  (dtclc, itloop,
     &   ohp, ho2p, h2op, no3p,
     &   qh1p, qh2p, qh3p, qj4p, qh5p, qh8p, qh9p,
     &   alh2o2, exh2o2, ah2o2,
     &   aldms, exdms, rdms,
     &   also2, exso2, rso2)
      
      use h_sulfchem, only: ijk
      
      implicit none

c#     include "sulfchem.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop

      real*8  :: dtclc

      real*8  :: ohp (itloop)
      real*8  :: ho2p(itloop)
      real*8  :: h2op(itloop)
      real*8  :: no3p(itloop)

      real*8  :: qh1p(itloop)
      real*8  :: qh2p(itloop)
      real*8  :: qh3p(itloop)
      real*8  :: qj4p(itloop)
      real*8  :: qh5p(itloop)
      real*8  :: qh8p(itloop)
      real*8  :: qh9p(itloop)

      real*8  :: alh2o2(itloop)
      real*8  :: exh2o2(itloop)
      real*8  :: ah2o2 (itloop)
      real*8  :: aldms (itloop)
      real*8  :: exdms (itloop)
      real*8  :: rdms  (itloop)
      real*8  :: also2 (itloop)
      real*8  :: exso2 (itloop)
      real*8  :: rso2  (itloop)


c     *****************************************************************
c     * (4) H2O2
c     *****************************************************************

      do 100 ijk = 1, itloop

        alh2o2(ijk) = qj4p(ijk) + qh5p(ijk) * ohp(ijk)
        exh2o2(ijk) = exp( - alh2o2(ijk) * dtclc )
        ah2o2(ijk)  = (qh3p(ijk) + qh8p(ijk) * h2op(ijk))
     &              * ho2p(ijk) * ho2p(ijk) / alh2o2(ijk)

  100 continue


c     *****************************************************************
c     * (3) DMS
c     *****************************************************************

      where( ohp(:) .gt. 10. ) 
        aldms(:) = qh1p(:) * ohp(:) + qh9p(:) * no3p(:)
        exdms(:) = exp( - aldms(:) * dtclc )
        rdms (:) = qh1p(:) * ohp(:) / aldms(:)
      elsewhere
        aldms(:) = 0.
        exdms(:) = 0.
        rdms (:) = 0.
      end where


c     *****************************************************************
c     * (2) SO2
c     *****************************************************************

      do 300 ijk = 1, itloop

        also2(ijk) = qh2p(ijk) * ohp(ijk)
        exso2(ijk) = exp( - also2(ijk) * dtclc )
        rso2(ijk)  = aldms(ijk) / ( also2(ijk) - aldms(ijk) )

  300 continue


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   chem11
c
c DESCRIPTION
c
c   advances the gas phase species using analytic expressions
c
c     ********** comments:
c
c     * species changes are computed first and stored in dcp
c       these changes are added to those from aqueous chemistry,
c       then applied to the cp
c
c     *     cp  : species concentration (# cm-3 for gas-species)
c     *     dcp : storage for conc. change
c
c     *     The following species are categorized by the type
c           of source driving them.
c
c     *     fso4a      ==  H2SO4,  fossil - aqueous
c     *     fso4n      ==  H2SO4,  fossil - clear sky
c     *     fso2       ==  SO2  ,    "
c
c     *     nso4a      ==  H2SO4,  natural - aqueous
c     *     nso4n      ==  H2SO4,     "    - clear sky
c     *     nso2       ==  SO2  ,     "
c     *     ndms       ==  DMS  ,     "
c
c     *     bso4a      ==  H2SO4,  biomass - aqueous
c     *     bso4n      ==  H2SO4,     "    - clear sky
c     *     bso2       ==  SO2  ,     "
c
c     *     h2o2       ==  H2O2
c
c     *     also2  = qh2 * OH
c     *     exso2  = exp( - also2 * dtclc )
c     *     rso2   = aldms / (  also2 - aldms )
c     *     aldms  = qh1 * OH + qh9 * NO3
c     *     exdms  = exp( - aldms * dtclc )
c     *     rdms   = qh1 * OH / aldms
c     *     alh2o2 = qh4 + qh5 * OH
c     *     exh2o2 = exp( - alh2o2 * dtclc )
c     *     ah2o2  = qh3 * HO2**2 / alh2o2
c
c
c     ********** output:
c
c     *     Species changes due to clear sky reactions.
c
c     *     dfso4n    ==  H2SO4,  fossil  - clear sky
c     *     dfso2     ==  SO2  ,    "
c
c     *     dnso4n    ==  H2SO4,  natural - clear sky
c     *     dnso2     ==  SO2  ,     "
c     *     dndms     ==  DMS  ,     "
c
c     *     dbso4n    ==  H2SO4,  biomass - clear sky
c     *     dbso2     ==  SO2  ,     "
c
c     *     dh2o2     ==  H2O2
c
c     *     dcp       ==  clear air species changes, then
c                         the clear air contribution to 
c                         species change, i.e.
c                         ( 1 - cfp ) * dcp.
c                         In chem30 there will be added to this
c                         the contribution due to aqueous chem
c                         in cloud, i.e.
c                         cfp * [ (new aqueous) - (old species) ].
c                         This will be imposed upon the grid points
c                         in chem40.
c
c-----------------------------------------------------------------------------

      subroutine chem11
     &  (dtclc, itloop, cp, dcp,
     &   ohp, cfp, zmp, semiss, srcgas, src_aso2,
     &   massc, pr_sulf_src, dms_oh, dms_no3, so2_oh,
     &   alh2o2, exh2o2, ah2o2,
     &   aldms, exdms, rdms,
     &   also2, exso2, rso2)
     
      use h_setkin_par
      use h_sulfchem, only:ijk,wtmso4,wtmso2,wtmair,conv
      
      implicit none

c#     include "setkin_par.h"
c#     include "sulfchem.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop

      real*8  :: dtclc

      real*8  :: cp    (itloop, NSP)
      real*8  :: dcp   (itloop, NSP)
      real*8  :: ohp   (itloop)
      real*8  :: cfp   (itloop)
      real*8  :: zmp   (itloop)
      real*8  :: semiss(itloop, NSP)
      real*8  :: srcgas(itloop)
      real*8  :: src_aso2(itloop)  ! added by Roy      

      real*8  :: massc (itloop)
      logical :: pr_sulf_src
      real*8  :: dms_oh (itloop)
      real*8  :: dms_no3(itloop)
      real*8  :: so2_oh (itloop)

      real*8  :: alh2o2(itloop)
      real*8  :: exh2o2(itloop)
      real*8  :: ah2o2 (itloop)
      real*8  :: aldms (itloop)
      real*8  :: exdms (itloop)
      real*8  :: rdms  (itloop)
      real*8  :: also2 (itloop)
      real*8  :: exso2 (itloop)
      real*8  :: rso2  (itloop)

c -------------------------------------------------------------
c  *** LOCAL  VARIABLES WERE DECLARED HERE.
c -------------------------------------------------------------

      real*8 :: h2o2  (itloop)
      real*8 :: dh2o2 (itloop)
      real*8 :: ndms  (itloop)
      real*8 :: dndms (itloop)
      real*8 :: nso2  (itloop)
      real*8 :: dnso2 (itloop)
      real*8 :: dnso4n(itloop)
      real*8 :: fso2  (itloop)
      real*8 :: dfso2 (itloop)
      real*8 :: dfso4n(itloop)

c++by Roy
      real*8 :: aso2  (itloop)
      real*8 :: daso2 (itloop)
      real*8 :: daso4n(itloop)
      real*8 :: saso2 (itloop)
c--

      real*8 :: sfso2(itloop)
      real*8 :: snso2(itloop)
      real*8 :: sndms(itloop)

      h2o2   = 0.0d0
      dh2o2  = 0.0d0
      ndms   = 0.0d0
      dndms  = 0.0d0
      nso2   = 0.0d0
      dnso2  = 0.0d0
      dnso4n = 0.0d0
      fso2   = 0.0d0
      dfso2  = 0.0d0
      dfso4n = 0.0d0
c++Roy
      aso2   = 0.0d0
      daso2  = 0.0d0
      daso4n = 0.0d0
      saso2  = 0.0d0
c--
      sfso2  = 0.0d0
      snso2  = 0.0d0
      sndms  = 0.0d0


      h2o2(:) = cp(:, IH2O2)
      ndms(:) = cp(:, INDMS)
      nso2(:) = cp(:, INSO2)
      fso2(:) = cp(:, IFSO2)

c++Roy
      aso2(:) = cp(:, IASO2)
      saso2(:) = semiss(:, IASO2)
c--

      sfso2(:) = semiss(:, IFSO2)
c     &         * 0.98d0           ! 98% of anthropogenic sulfur emission as SO2, now emission has been timed the fraction before calling this.
      snso2(:) = semiss(:, INSO2)
      sndms(:) = semiss(:, INDMS)



c     *****************************************************************
c     * (11) H2O2
c     *****************************************************************


        dh2o2(:) = ( ah2o2(:) - h2o2(:) )
     &             * ( 1.d0 - exh2o2(:) ) 
        where ( abs( dh2o2(:) ) < 1.d0 ) dh2o2(:) = 0.d0



c     *****************************************************************
c     * (10) biomass SO2
c     *****************************************************************
c                         
c           dbso2(:) = ( sbso2(:) / also2(:) - bso2(:) ) 
c     &              * ( 1.d0 - exso2(:) )
c           where ( abs( dbso2(:) ) < 1. ) dbso2(:) = 0.d0
c                   
c     *****************************************************************
c     * (9) biomass H2SO4
c     *****************************************************************
c                              
c           dbso4n(:) = sbso2(:) * dtclc - dbso2(:) 
c           where ( abs( dbso4n(:) ) < 1.d0 ) dbso4n(:) = 0.d0
c                          


c     *****************************************************************
c     * (7) natural DMS
c     *****************************************************************

        where ( ohp(:) .gt. 10.d0 )

          dndms(:) = ( sndms(:) / aldms(:) - ndms(:) ) 
     &               * ( 1.d0 - exdms(:) ) 

        elsewhere

          dndms(:) = sndms(:) * dtclc

        end where

        where ( abs( dndms(:) ) < 1.d0 ) dndms(:) = 0.d0


c     *****************************************************************
c     * (6) natural SO2
c     *****************************************************************

        where ( ohp(:) .gt. 10.d0 )

          dnso2(:) = ( ( sndms(:) + snso2(:) )
     &               / also2(:) - nso2(:) )
     &               * ( 1.d0 - exso2(:) )
     &               - ( sndms(:) / aldms(:) -  ndms(:) )
     &               * ( exdms(:) - exso2(:) ) * rso2(:)

        elsewhere

          dnso2(:) = ( snso2(:) / also2(:) - nso2(:) )
     &               * ( 1.d0 - exso2(:) )

        end where

        where ( abs( dnso2(:) ) < 1.d0 ) dnso2(:) = 0.d0


c     *****************************************************************
c     * (5) natural H2SO4
c     *****************************************************************

        where ( ohp(:) .gt. 10.d0 )

          dnso4n(:) = ( sndms(:) + snso2(:) ) * dtclc 
     &                - dnso2(:) - dndms(:)

        elsewhere

          dnso4n(:) = snso2(:) * dtclc - dnso2(:)

        end where

c       where ( abs( dnso4n(:) ) < 1.d0 ) dnso4n(:) = 0.d0
        where (  dnso4n(:)  < 1.d0 ) dnso4n(:) = 0.d0 !modified by  Cheng

        

c     *****************************************************************
c     * (3) fossil SO2
c     *****************************************************************

        dfso2(:) = ( sfso2(:) / also2(:) - fso2(:) ) 
     &             * ( 1.d0 - exso2(:) )

        where ( abs( dfso2(:) ) < 1.d0 ) dfso2(:) = 0.d0



        dfso4n(:) = sfso2(:) * dtclc - dfso2(:)

c        where ( abs( dfso4n(:) ) < 1.d0 ) dfso4n(:) = 0.d0

c modified by Cheng to make sure dfso4>0
        where  ( dfso4n(:) < 1.d0 ) 
           dfso4n(:) = 0.d0
           dfso2(:)  = sfso2(:) * dtclc ! all emission goes to fSO2
        end where

c     *****************************************************************
c     * (3) Airsoot SO2  added by Roy
c     *****************************************************************

        daso2(:) = ( saso2(:) / also2(:) - aso2(:) )
     &             * ( 1.d0 - exso2(:) )

        where ( abs( daso2(:) ) < 1.d0 ) daso2(:) = 0.d0



        daso4n(:) = saso2(:) * dtclc - daso2(:)

        where ( abs( daso4n(:) ) < 1.d0 ) daso4n(:) = 0.d0


c     *****************************************************************
c     * Compute the clear sky contribution to species change.
c     *****************************************************************
 
c     conv: convert SO4 concentration from # cm-3 to gm gm-1
      conv = wtmair / wtmso4

c.... H2O2
        dcp(:, IH2O2)   = ( 1.0d0 - cfp(:) ) * dh2o2(:)

c.... fossil
        dcp(:, IFSO2)   = ( 1.0d0 - cfp(:) ) * dfso2(:)
c++Roy
        dcp(:, IASO2)   = ( 1.0d0 - cfp(:) ) * daso2(:)
c--
c       dcp(:, IFSO4N1) = ( 1.0d0 - cfp(:) ) * dfso4n(:)
c    &                    * ffra_so4(1) / (conv * zmp(:))  ! unit: g/g
c       dcp(:, IFSO4N2) = ( 1.0d0 - cfp(:) ) * dfso4n(:)
c    &                    * ffra_so4(2) / (conv * zmp(:))
c       dcp(:, IFSO4N3) = ( 1.0d0 - cfp(:) ) * dfso4n(:)
c    &                    * ffra_so4(3) / (conv * zmp(:))

c.... natural
        dcp(:, INDMS)   = ( 1.0d0 - cfp(:) ) * dndms(:)
        dcp(:, INSO2)   = ( 1.0d0 - cfp(:) ) * dnso2(:)

c       dcp(:, INSO4N1) = ( 1.0d0 - cfp(:) ) * dnso4n(:)
c    &                    * nfra_so4(1) / (conv * zmp(:))  ! unit: g/g
c       dcp(:, INSO4N2) = ( 1.0d0 - cfp(:) ) * dnso4n(:)
c    &                    * nfra_so4(2) / (conv * zmp(:))
c       dcp(:, INSO4N3) = ( 1.0d0 - cfp(:) ) * dnso4n(:)
c    &                    * nfra_so4(3) / (conv * zmp(:))

        srcgas(:) = ( 1.0d0 - cfp(:) ) *
     &              ( dfso4n(:) + dnso4n(:) ) / dtclc
     &              * 1.0d+6                               ! unit: molecules/m3/s
c added by Roy
        src_aso2(:) = ( 1.0d0 - cfp(:) ) * daso4n(:) / dtclc
     &              * 1.0d+6                ! unit: molecules/m3/s
        dcp(:, IASO4)   = (1.0d0 - cfp(:)) * daso4n(:) /
     &               (conv * zmp(:))        ! unit: g/g
     
c.... biomass
c       dbso2(:)  = ( 1.0d0 - cfp(:) ) * dbso2(:)
c       dbso4n(:) = ( 1.0d0 - cfp(:) ) * dbso4n(:)


        if (pr_sulf_src) then
 
          dms_oh(:) =
     &      dms_oh(:) +
     &      ( 1.0d0 - cfp(:) ) *
     &      max( (sndms(:) * dtclc - dndms(:)), 0.0d0) *
     &      rdms(:) *
     &      massc(:) / zmp(:) * wtmso2 / wtmair

          dms_no3(:) =
     &      dms_no3(:) +
     &      ( 1.0d0 - cfp(:) ) *
     &      max( (sndms(:) * dtclc - dndms(:)), 0.0d0) * 
     &      ( 1.0d0 - rdms(:)) *
     &      massc(:) / zmp(:) * wtmso2 / wtmair

          so2_oh(:) =
     &      so2_oh(:) +
     &      ( 1.0d0 - cfp(:) ) *
     &      (dfso4n(:) + dnso4n(:) + daso4n(:)) *
     &      massc(:) / zmp(:) * wtmso4 / wtmair

    
        end if


      return

      end

