      module h_imp_control
      implicit none
      public 
c=============================================================================
c
c $Id: imp_control.h,v 1.45 2003/07/14 17:22:06 dan Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL
c   jrt@llnl.gov
c
c FILE
c   imp_control.h
c
c DESCRIPTION
c   This include file contains declarations of a number of control items.
c   This data needs to be set prior to allocating any memory in a given
c   processor.
c
c   NOTE THAT IF A DIFFERENT PARAMETER IS USED IN DETERMINING THE SIZE OF
c   ANY OF THE imp_cn_*_array's BELOW, BE SURE TO CHANGE THE PARAMETER USED
c   IN THE Imp_Control_Mes ROUTINE AS WELL.
c
c=============================================================================


c     ----------
c     Constants.
c     ----------

      integer, parameter ::
     &  SIZCNSML =  255,  ! small  common block size
     &  SIZCNMED = 1023,  ! medium common block size
     &  SIZCNBIG = 4095   ! big    common block size


c     --------------------------------------------------------
c     The following MIN/MAX parameters need to match the range
c     used for ACTM messages in msg_numbers.h!
c     --------------------------------------------------------
 
      integer, parameter ::
     &  MIN_MSG_NUM = 10000, ! minimum (i.e., smallest) message number
     &  MAX_MSG_NUM = 14999  ! maximum (i.e., biggest)  message number

 
c     -------------------------------------------------------------
c     IF ANY OF THE FOLLOWING PARAMETERS IS INCREASED, BE SURE THAT
c     THE COMMON BLOCK SIZE CONSTANTS ABOVE ARE LARGE ENOUGH!
c
c     IF MAX_NUM_CONST IS INCREASED, BE SURE THAT THERE ARE ENOUGH
c     MESSAGE NUMBERS IN msg_numbers.h!
c     -------------------------------------------------------------
 
      integer, parameter ::
     &  MAX_NUM_CGRP      =    2,  ! maximum number of chemical groups
     &  MAX_NUM_CONST     =  185,  ! maximum number of const species
     &  MAX_NUM_OPT_DEPTH =   20,  ! maximum number of optical depths
     &  MAX_NUM_RAD       =   20,  ! maximum number of rad species
     &  MAX_NUM_RAD_MAP   =   19,  ! maximum number of mapped rad species
     &  MAX_NUM_RAD_MAPA  =   15,  ! maximum number of mapped rad aerosol spec
     &  MAX_NUM_RXNR      = 1000,  ! maximum number of reaction rates
     &  MAX_NUM_SMARRAY   =   10   ! maximum number of small array elements

 
c     ------------------
c     Character commons.
c     ------------------
 
      character (len=1) ::
     &  met_grid_type  ! met grid type, A or C

      character (len=1) :: imp_cn_cdum1
      character (len=1) :: imp_cn_c1_array(SIZCNSML)
 
cc     ====================
c      common  / impcn_c1 /
cc     ====================
c     &  imp_cn_cdum1,
c     &  met_grid_type
c 
cc     ===========
c      equivalence (imp_cn_c1_array, imp_cn_cdum1)
cc     ===========


c     ----------------
c     Logical commons.
c     ----------------

      logical ::
     &  do_ftiming          ! do fine timing?

      logical ::
     &  do_aqu_chem,        ! do aqueous chem  (sulfur)?
     &  do_chem_grp,        ! do chemical groups?
     &  do_full_chem,       ! do full chemistry?
     &  do_smv_reord,       ! reorder grid boxes in order of stiffness?
     &  do_wetchem          ! do wet chemistry?

      logical ::
     &  do_clear_sky,       ! do clear sky photolysis?
     &  do_dust_emiss,      ! do the dust emissions using Ginoux's algorithm?
     &  do_grav_set,        ! do gravitational settling of aerosols?
     &  do_rxnr_adjust,     ! adjust reaction rates?
     &  do_semiss_inchem,   ! do surface emissions inside chemistry solver?
     &  do_var_adv_tstp,    ! take variable advection time steps?
     &  do_wind_pole        ! make winds blow over Pole?

      logical ::
     &  do_drydep,          ! do dry deposition?
     &  do_simpledep,       ! do simple deposition?
     &  do_wetdep           ! do wet deposition?

      logical ::
     &  do_nodoz,           ! do Nodoz?
     &  do_synoz            ! do Synoz?

      logical ::
     &  do_opt_depth,       ! do aerosol optical depth?
     &  do_rad,             ! do radiation?
     &  use_bc_cld,          ! BC absorption effect within drop?
     &  do_um_aod           ! do UM AOD calculation? added by Cheng Zhou

      logical ::
     &  do_timinterp_met,   ! time interpolate met  fields, except winds?
     &  do_timinterp_winds  ! time interpolate wind fields?

      logical :: imp_cn_ldum1
      logical :: imp_cn_l1_array(SIZCNMED)

cc     ====================
c      common  / impcn_l1 /
cc     ====================
c     &  imp_cn_ldum1,
c     &  do_ftiming,
c     &  do_aqu_chem,      do_chem_grp,
c     &  do_full_chem,
c     &  do_smv_reord,     do_wetchem,
c     &  do_clear_sky,     do_dust_emiss,     do_grav_set,
c     &  do_rxnr_adjust,   do_semiss_inchem,
c     &  do_var_adv_tstp,  do_wind_pole,
c     &  do_drydep,        do_simpledep,      do_wetdep,
c     &  do_nodoz,         do_synoz,
c     &  do_opt_depth,     do_rad,            use_bc_cld, do_um_aod,
c     &  do_timinterp_met, do_timinterp_winds
c 
cc     ===========
c      equivalence (imp_cn_l1_array, imp_cn_ldum1)
cc     ===========
c 

c     ----------------
c     Integer commons.
c     ----------------

      integer ::
     &  advec_opt,        ! advection              option
     &  chem_opt,         ! chemistry              option
     &  const_opt,        ! constituent            option
     &  convec_opt,       ! convection             option
     &  diffu_opt,        ! diffusion              option
     &  emiss_opt,        ! emission               option
     &  met_opt,          ! met                    option
     &  phot_opt,         ! photolysis             option
     &  sad_opt,          ! surface area density   option
     &  trans_opt         ! transport              option

      integer ::
     &  advec_consrv_opt, ! advection conserve     option
     &  be_opt,           ! beryllium star table   option
     &  emiss_in_opt,     ! emission input         option
     &  emiss_aero_opt,   ! sulfur aerosol emiss   option
     &  emiss_dust_opt,   ! sulfur dust    emiss   option
     &  forc_bc_opt,      ! forcing boundary cond. option
     &  h2oclim_opt,      ! water climatology      option
     &  lbssad_opt,       ! liquid binary sulfate  option
     &  loss_freq_opt,    ! loss frequency         option
     &  pmet2_opt,        ! pmet2                  option
     &  press_fix_opt,    ! pressure fixer         option
     &  sfalbedo_opt,     ! surface albedo         option
     &  uvalbedo_opt      ! UV albedo              option

      integer ::
     &  iacetone_num,     ! const array index for acetone  (C3H6O)
     &  ico_num,          ! const array index for CO
     &  iisoprene_num,    ! const array index for isoprene (C5H8)
     &  ipropene_num,     ! const array index for propene  (C3H6)
     &  ino_num           ! const array index for NO

      integer ::
     &  ibrono2_num,      ! const array index for BRONO2
     &  ich4_num,         ! const array index for CH4
     &  in2o_num,         ! const array index for N2O
     &  idehyd_num,       ! const array index for dehydration data
     &  ih2o_num,         ! const array index for water
     &  ih2o2_num,        ! const array index for H2O2
     &  ih2oaircr_num,    ! const array index for aircraft water
     &  ihcl_num,         ! const array index for HCl
     &  ihno3_num,        ! const array index for HNO3
     &  imgas_num,        ! const array index for air density data
     &  initrogen_num,    ! const array index for nitrogen
     &  ioxygen_num,      ! const array index for oxygen
     &  io3_num,          ! const array index for ozone
     &  isynoz_num        ! const array index for synoz

      integer :: imp_cn_idum1
      integer :: imp_cn_i1_array(SIZCNMED)

cc     ====================
c      common  / impcn_i1 /
cc     ====================
c     &  imp_cn_idum1,
c     &  advec_opt,        chem_opt,
c     &  const_opt,        convec_opt,
c     &  diffu_opt,        emiss_opt,
c     &  met_opt,          phot_opt,
c     &  sad_opt,          trans_opt,
c     &  advec_consrv_opt, be_opt,
c     &  emiss_in_opt,     emiss_aero_opt,
c     &  emiss_dust_opt,   forc_bc_opt,
c     &  h2oclim_opt,      lbssad_opt,
c     &  loss_freq_opt,    pmet2_opt,
c     &  press_fix_opt,    sfalbedo_opt,
c     &  uvalbedo_opt,
c     &  iacetone_num,     ico_num,
c     &  iisoprene_num,    ipropene_num,
c     &  ino_num,
c     &  ibrono2_num,      ich4_num,     in2o_num,
c     &  idehyd_num,
c     &  ih2o_num,         ih2o2_num,    ih2oaircr_num,
c     &  ihcl_num,         ihno3_num,
c     &  imgas_num,
c     &  initrogen_num,    ioxygen_num,
c     &  io3_num,          isynoz_num
c 
cc     ===========
c      equivalence (imp_cn_i1_array, imp_cn_idum1)
cc     ===========
c

      integer ::
     &  chem_mask_khi,        ! do not do chemistry at k levels above this
     &  chem_mask_klo         ! do not do chemistry at k levels below this

      integer ::
     &  forc_bc_kmax,         ! maximum vertical index at which forcing occurs
     &  forc_bc_kmin,         ! minimum vertical index at which forcing occurs
     &  forc_bc_start_num     ! forcing boundary condition start number; index
                              ! for year to use

      integer ::
     &  kmax_loss,            ! maximum vertical index at which loss occurs
     &  kmin_loss,            ! minimum vertical index at which loss occurs
     &  ktr                   ! number of vertical tropospheric levels

      integer ::
     &  ndt,                  ! model time step (s)
     &  num_adv_time_steps,   ! number of advection time steps
     &  num_time_steps        ! number of           time steps

      integer ::
     &  nhms,                 ! current hour/min/sec   (HHMMSS)
     &  nymd,                 ! current year/month/day (YYMMDD)
     &  start_hms,            ! starting hour/min/sec   (HHMMSS)
     &  start_ymd             ! starting year/month/day (YYMMDD)

      integer ::
     &  num_ks_sbc,           ! number of vertical layers in which to reset
                              ! surface boundary conditions
     &  num_ks_sdep,          ! number of vertical layers to apply 2 day loss
                              ! factor to in simple deposition
     &  num_nox,              ! number of Nodoz NOx species
     &  num_noy,              ! number of Nodoz NOy species
     &  num_spc_sbc           ! number of species that need to have their
                              ! surface boundary conditions reset

      integer ::
     &  opt_depth_bcbb,       ! points to biomass black carbon,     used in
                              ! optical depth
     &  opt_depth_bcfos       ! points to fossil fuel black carbon, used in
                              ! optical depth


c     ---------------------------------------------------------------------
c     advec_flag       : array of flags that indicate whether or not to
c                        advect a given species
c
c     rad_di_flag      : flag for direct   radiative impact by rad species
c     rad_in_flag      : flag for indirect radiative impact by rad species
c     rad_tt_flag      : flag for (direct + indirect) radiative effect
c
c     num_cgrp_elem    : number of chemical group elements
c
c     emiss_map        : mapping of emission      number to const species #
c     emiss_map_aero   : mapping of aerosol emiss number to const species #
c     emiss_map_dust   : mapping of dust    emiss number to const species #
c     emiss_map_tfunc  : mapping of temperature function to const_species #
c     fixed_const_map  : mapping of fixed const   number to const species #
c     forc_bc_map      : mapping of forcing bc    number to const species #
c     surf_bc_map      : mapping of surface boundary condition number to
c                        const species number
c
c     nox_map          : mapping of NOx species   number to const species #
c     noy_map          : mapping of NOy species   number to const species #
c
c     opt_depth_map    : mapping of optical depth number to const species #
c
c     rad_map          : mapping of rad           number to const species #
c
c     rxnr_adjust_map  : mapping of reaction rate adjustment number to
c                        reaction rate number
c
c     cgrp_elem_map    : mapping of chemical group element   number to
c                        const species number
c     ---------------------------------------------------------------------

      integer :: advec_flag     (MAX_NUM_CONST)

      integer :: rad_di_flag    (MAX_NUM_RAD_MAP, MAX_NUM_RAD)
      integer :: rad_in_flag    (MAX_NUM_RAD_MAP, MAX_NUM_RAD)
      integer :: rad_tt_flag    (MAX_NUM_RAD_MAP, MAX_NUM_RAD)

      integer :: num_cgrp_elem  (MAX_NUM_CGRP)

      integer :: emiss_map      (MAX_NUM_CONST)
      integer :: emiss_map_aero (MAX_NUM_CONST)
      integer :: emiss_map_dust (MAX_NUM_CONST)
      integer :: emiss_map_tfunc(MAX_NUM_CONST)

      integer :: fixed_const_map(MAX_NUM_CONST)
      integer :: forc_bc_map    (MAX_NUM_CONST)
      integer :: surf_bc_map    (MAX_NUM_CONST)

      integer :: nox_map        (MAX_NUM_SMARRAY)
      integer :: noy_map        (MAX_NUM_SMARRAY)

      integer :: opt_depth_map  (MAX_NUM_OPT_DEPTH)

      integer :: rad_map        (MAX_NUM_RAD_MAP)

      integer :: rxnr_adjust_map(MAX_NUM_RXNR)

      integer :: cgrp_elem_map  (MAX_NUM_SMARRAY, MAX_NUM_CGRP)

      integer :: imp_cn_idum2
      integer :: imp_cn_i2_array(SIZCNBIG)
c
cc     ====================
c      common  / impcn_i2 /
cc     ====================
c     &  imp_cn_idum2,
c     &  chem_mask_khi,      chem_mask_klo,
c     &  forc_bc_kmax,       forc_bc_kmin,
c     &  forc_bc_start_num,
c     &  kmax_loss,          kmin_loss,
c     &  ktr,
c     &  ndt,
c     &  num_adv_time_steps,
c     &  num_time_steps,
c     &  nhms,               nymd,
c     &  start_hms,          start_ymd,
c     &  num_ks_sbc,         num_ks_sdep,
c     &  num_nox,            num_noy,
c     &  num_spc_sbc,
c     &  opt_depth_bcbb,     opt_depth_bcfos,
c     &  advec_flag,
c     &  rad_di_flag,        rad_in_flag,
c     &  rad_tt_flag,
c     &  num_cgrp_elem,
c     &  emiss_map,          emiss_map_aero,
c     &  emiss_map_dust,     emiss_map_tfunc,   fixed_const_map,
c     &  forc_bc_map,        surf_bc_map,
c     &  nox_map,            noy_map,
c     &  opt_depth_map,      rad_map,
c     &  rxnr_adjust_map,
c     &  cgrp_elem_map 

cc     ===========
c      equivalence (imp_cn_i2_array, imp_cn_idum2)
cc     ===========
c

c     -------------
c     Real commons.
c     -------------

      real*8  ::
     &  chem_cycle,        ! number of time steps to cycle chemistry calls on
     &  dehydmin,          ! minimum dehyd value (mixing ratio)
     &  fastj_offset_sec,  ! offset time at which to do fastj photolysis (s)
     &  geofac_pc,         ! special geometrical factor (geofac) for Polar cap
     &  synoz_threshold,   ! chemistry turned off where synoz > this threshold
     &  t_cloud_ice,       ! temperature of ice formation in clouds (degK)
     &  vert_diffu_coef    ! scalar vertical diffusion coefficient  (m^2/s)


      real*8  ::
     &  forc_bc_incrpyr,   ! forcing boundary cond. emission increase per year
     &  forc_bc_lz_val     ! value to which lower zones are forced
                           ! (mixing ratio)

      real*8  ::
     &  imp_sec,           ! total Impact seconds (s)
     &  tdt                ! model time step (s)

      real*8  ::
     &  pt,                ! pressure = (am * pt) + (bm * psx) (mb)
     &  ptop               ! pressure at top edge of top zone  (mb)

      real*8  ::
     &  t_half_be7,        ! half life of     Beryllium-7,  or other cosmogenic
                           ! radionuclide (days)
     &  t_half_be10        ! half life of     Beryllium-10, or other cosmogenic
                           ! radionuclide (days)

      real*8  ::
     &  yield_be7,         ! yield factor for Beryllium-7,  or other cosmogenic
                           ! radionuclide (unitless)
     &  yield_be10         ! yield factor for Beryllium-10, or other cosmogenic
                           ! radionuclide (unitless)

c     ------------------------------------------------------------------
c     isop_scale : array of monthly isoprene scaling coefficients 
c     dust_scale : array of dust scaling factors for each species
c     mw         : array of species' molecular weights (g/mol)
c     wetdep_eff : array of wet deposition (scavenging) efficiencies
c                  (a fraction between 0.0 and 1.0)
c
c     cgrp_fac   : number of atomic elements in a chemical group element
c     ------------------------------------------------------------------

      real*8  :: isop_scale (12)

      real*8  :: dust_scale(MAX_NUM_CONST)
      real*8  :: mw        (MAX_NUM_CONST)
      real*8  :: wetdep_eff(MAX_NUM_CONST)

      real*8  :: cgrp_fac  (MAX_NUM_SMARRAY, MAX_NUM_CGRP)

      real*8  :: imp_cn_rdum1
      real*8  :: imp_cn_r1_array(SIZCNBIG)

cc     ====================
c      common  / impcn_r1 /
cc     ====================
c     &  imp_cn_rdum1,
c     &  chem_cycle,
c     &  dehydmin,
c     &  fastj_offset_sec,
c     &  geofac_pc,
c     &  synoz_threshold,
c     &  t_cloud_ice,
c     &  vert_diffu_coef,
c     &  forc_bc_incrpyr, forc_bc_lz_val,
c     &  imp_sec,         tdt,
c     &  pt,              ptop,
c     &  t_half_be7,      t_half_be10,
c     &  yield_be7,       yield_be10,
c     &  isop_scale,
c     &  dust_scale,
c     &  mw,              wetdep_eff,
c     &  cgrp_fac
c
cc     ===========
c      equivalence (imp_cn_r1_array, imp_cn_rdum1)
cc     ===========

      end module h_imp_control