      module h_imp_dims
c=============================================================================
c
c $Id: imp_dims.h,v 1.16 2003/05/27 21:14:25 jrt Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL
c   jrt@llnl.gov
c   Cheng Zhou, zhouc@umich.edu
c FILE
c   imp_dims.h -> h_imp_dims.F
c
c DESCRIPTION
c   This include file primarily contains declarations of the dimensions that
c   are used to declare the dynamic arrays.  They are set by the master
c   processor and communicated to the slaves as necessary.  This data needs
c   to be set prior to dynamically allocating any arrays on a given processor.
c Modification by Cheng Zhou
c   comment out all commons and equivalence, and implicit non, public ,end module
c=============================================================================
      implicit none
      public 

c     ----------
c     Constants.
c     ----------

      integer, parameter ::
     &  SIZDIM = 100  ! used to set the size of communication arrays below


c     ===============================================================
c     Variables that don't need to be communicated by message passing
c     (see actm_init1):
c     ===============================================================

c     ----------------
c     Logical commons.
c     ----------------

      logical ::
     &  one_proc    ! if true  => single processor mode,
                    ! if false => multi-processor  mode

cc     =====================
c      common  / impdm_l1n /
cc     =====================
c     &  one_proc


c     ----------------
c     Integer commons.
c     ----------------

      integer ::
     &  glob_proc,  ! global processor number
     &  loc_proc    ! local  processor number

      integer ::
     &  NPIJ_actm   ! number of processors used by actm

cc     =====================
c      common  / impdm_i1n /
cc     =====================
c     &  glob_proc, loc_proc,
c     &  NPIJ_actm


c     =============================================================
c     Variables that do need to be communicated by message passing:
c     =============================================================

c     ----------------
c     Logical commons.
c     ----------------

      logical ::
     &  pr_diag  ! print some diagnostic output to screen?

      logical :: imp_dm_ldum1
      logical :: imp_dm_l1_array(SIZDIM)

cc     =====================
c      common  / impdm_l1y /
cc     =====================
c     &  imp_dm_ldum1,
c     &  pr_diag
c 
cc     ===========
c      equivalence (imp_dm_l1_array, imp_dm_ldum1)
cc     ===========


c     ----------------
c     Integer commons.
c     ----------------

c     ---------------------------------------------------------------
c     i1_gl   : index of first global longitude      (no ghost zones)
c     i2_gl   : index of last  global longitude      (no ghost zones)
c     ju1_gl  : index of first global "u"   latitude (no ghost zones)
c     jv1_gl  : index of first global "v"   latitude (no ghost zones)
c     j2_gl   : index of last  global "u&v" latitude (no ghost zones)
c     k1_gl   : index of first global altitude       (no ghost zones)
c     k2_gl   : index of last  global altitude       (no ghost zones)
c
c     ilo_gl  : i1_gl  - imp_nborder (has ghost zones)
c     ihi_gl  : i2_gl  + imp_nborder (has ghost zones)
c     julo_gl : ju1_gl - imp_nborder (has ghost zones)
c     jvlo_gl : jv1_gl - imp_nborder (has ghost zones)
c     jhi_gl  : j2_gl  + imp_nborder (has ghost zones)
c     ---------------------------------------------------------------

      integer :: i1_gl,  i2_gl,  ju1_gl,  jv1_gl,  j2_gl, k1_gl, k2_gl
      integer :: ilo_gl, ihi_gl, julo_gl, jvlo_gl, jhi_gl

      integer :: imp_dm_idum1
      integer :: imp_dm_i1_array(SIZDIM)
c
cc     =====================
c      common  / impdm_i1y /
cc     =====================
c     &  imp_dm_idum1,
c     &  i1_gl,  i2_gl,  ju1_gl,  jv1_gl,  j2_gl, k1_gl, k2_gl,
c     &  ilo_gl, ihi_gl, julo_gl, jvlo_gl, jhi_gl
c 
cc     ===========
c      equivalence (imp_dm_i1_array, imp_dm_idum1)
cc     ===========


c     -----------------------------------------------------------
c     i1   : index of first local longitude      (no ghost zones)
c     i2   : index of last  local longitude      (no ghost zones)
c     ju1  : index of first local "u"   latitude (no ghost zones)
c     jv1  : index of first local "v"   latitude (no ghost zones)
c     j2   : index of last  local "u&v" latitude (no ghost zones)
c     k1   : index of first local altitude       (no ghost zones)
c     k2   : index of last  local altitude       (no ghost zones)
c
c     ilo  : i1  - imp_nborder (has ghost zones)
c     ihi  : i2  + imp_nborder (has ghost zones)
c     julo : ju1 - imp_nborder (has ghost zones)
c     jvlo : jv1 - imp_nborder (has ghost zones)
c     jhi  : j2  + imp_nborder (has ghost zones)
c     -----------------------------------------------------------

      integer :: i1,  i2,  ju1,  jv1,  j2,  k1,  k2
      integer :: ilo, ihi, julo, jvlo, jhi

      integer :: imp_dm_idum2
      integer :: imp_dm_i2_array(SIZDIM)

cc     =====================
c      common  / impdm_i2y /
cc     =====================
c     &  imp_dm_idum2,
c     &  i1,  i2,  ju1,  jv1,  j2,  k1,  k2,
c     &  ilo, ihi, julo, jvlo, jhi
c
cc     ===========
c      equivalence (imp_dm_i2_array, imp_dm_idum2)
cc     ===========


      integer ::
     &  ilat,   ! number of latitudes
     &  ilong,  ! number of longitudes
     &  ivert,  ! number of vertical layers
     &  itloop  ! number of zones (ilat * ilong * ivert)

      integer :: imp_dm_idum3
      integer :: imp_dm_i3_array(SIZDIM)
 
cc     =====================
c      common  / impdm_i3y /
cc     =====================
c     &  imp_dm_idum3,
c     &  ilat, ilong, ivert,
c     &  itloop
c 
cc     ===========
c      equivalence (imp_dm_i3_array, imp_dm_idum3)
cc     ===========


      integer ::
     &  emiss_timpyr,       ! emission              times per year
                            ! (1 => yearly, 12 => monthly)
     &  fixed_const_timpyr, ! fixed const           times per year
                            ! (1 => yearly, 12 => monthly)
     &  h2oclim_timpyr,     ! water climatology     times per year
                            ! (1 => yearly, 12 => monthly)
     &  lbssad_timpyr,      ! liquid binary sulfate times per year
                            ! (1 => yearly, 12 => monthly)
     &  rxnr_adjust_timpyr  ! reac rate adjustment  times per year
                            ! (1 => yearly, 12 => monthly)

      integer ::
     &  fbc_j1,             ! forcing boundary condition j1 (i.e., low  lat.)
     &  fbc_j2,             ! forcing boundary condition j2 (i.e., high lat.)
     &  forc_bc_num,        ! number of species with forced boundary conditions
     &  forc_bc_years      ! number of years of forced boundary condition data

      integer ::
     &  imp_nborder         ! number of longitude and latitude ghost zones

      integer ::
     &  j1p,                ! determines size of the Polar cap
     &  j2p                 ! j2_gl - j1p + 1

      integer ::
     &  NPI_actm,           ! number of processors in the i (i.e., longitude)
                            ! direction
     &  NPJ_actm            ! number of processors in the j (i.e., latitude)
                            ! direction

cc?
      integer ::            
     &  naero,              ! number of aerosols ???
     &  ndust,              ! number of dust     ???
     &  nst_dust,           ! starting index for dust ???
     &  nt_dust             ! ending   index for dust ???

      integer,parameter ::
     &  num_emiss=3,          ! number of emissions of Sulf related, nSO2,fSO2, DMS
     &  num_qjs=1,            ! number of qjs (i.e., photolysis rate constants)
     &  num_qks=8            ! number of qks (i.e., thermal    rate constants)! not used actually

      integer ::
     &  num_active,         ! number of active chemical species
     &  num_cgrp,           ! number of        chemical groups
     &  num_chem,           ! number of        chemical species
c     &  num_emiss,          ! number of emissions
     &  num_fixed_const,    ! number of fixed consts
     &  num_molefrac,       ! number of species for which conversion from mole
                            ! fraction to number density is required going
                            ! into chemistry (and back after chemistry)
     &  num_opt_depth,      ! number of optical depths to calculate
     &  num_qjo,            ! number of qjs for output file
c     &  num_qjs,            ! number of qjs (i.e., photolysis rate constants)
c     &  num_qks,            ! number of qks (i.e., thermal    rate constants)
     &  num_rad,            ! number of total rad calculations
     &  num_rad_di,         ! number of rad calculations for direct   effect
     &  num_rad_in,         ! number of rad calculations for indirect effect
     &  num_rad_tt,         ! number of rad calculations for direct + indirect
                            ! effect
     &  num_rad_map,        ! number of mapped rad species
     &  num_rad_mapa,       ! number of mapped rad aerosol species
     &  num_rxnr_adjust,    ! number of reaction rate adjustments
     &  num_sad,            ! number of sads (i.e., surface area densities)
     &  num_species,        ! number of species
     &  num_wave_aer        ! number of wavelengths in optical depth calculation

      integer ::
     &  num_const_inrecs,   ! number of species to input 
     &  num_const_outrecs   ! number of species to output 

      integer :: imp_dm_idum4
      integer :: imp_dm_i4_array(SIZDIM)

cc     =====================
c      common  / impdm_i4y /
cc     =====================
c     &  imp_dm_idum4,
c     &  emiss_timpyr, fixed_const_timpyr,
c     &  h2oclim_timpyr, lbssad_timpyr,
c     &  rxnr_adjust_timpyr,
c     &  fbc_j1, fbc_j2,
c     &  forc_bc_num, forc_bc_years,
c     &  imp_nborder,
c     &  j1p, j2p,
c     &  NPI_actm, NPJ_actm,
c     &  naero, ndust,
c     &  nst_dust, nt_dust,
c     &  num_active,
c     &  num_cgrp,
c     &  num_chem,
c     &  num_emiss, num_fixed_const,
c     &  num_molefrac,
c     &  num_opt_depth,
c     &  num_qjo, num_qjs, num_qks,
c     &  num_rad,
c     &  num_rad_di, num_rad_in, num_rad_tt,
c     &  num_rad_map, num_rad_mapa,
c     &  num_rxnr_adjust,
c     &  num_sad, num_species,
c     &  num_wave_aer,
c     &  num_const_inrecs, num_const_outrecs
c 
cc     ===========
c      equivalence (imp_dm_i4_array, imp_dm_idum4)
cc     ===========

      end module h_imp_dims 