      module h_imp_gen_io
      implicit none
      public 
c=============================================================================
c
c $Id: imp_gen_io.h,v 1.30 2003/07/14 17:22:06 dan Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL
c   jrt@llnl.gov
c
c FILE
c   imp_gen_io.h
c
c DESCRIPTION
c   This include file contains declarations of a number of general io items.
c   This data needs to be set prior to allocating any memory in a given
c   processor.
c
c   NOTE THAT IF A DIFFERENT PARAMETER IS USED IN DETERMINING THE SIZE OF
c   ANY OF THE imp_gio_*_array's BELOW, BE SURE TO CHANGE THE PARAMETER USED
c   IN THE Imp_Control_Mes ROUTINE AS WELL.
c
c=============================================================================


c     ----------
c     Constants.
c     ----------

      integer, parameter ::
     &  SIZGIOSML =      255,  ! small  common block size
     &  SIZGIOMED =     1023,  ! medium common block size
     &  SIZGIOBIG =     4095,  ! big    common block size
     &  SIZGIOBG2 = 1023*127   ! bigger common block size


c     ----------------------------------------------------------------
c     IF ANY OF THE FOLLOWING PARAMETERS IS INCREASED, BE SURE THAT
c     THE COMMON BLOCK SIZE CONSTANTS ABOVE ARE LARGE ENOUGH!
c
c     IF MAX_NUM_CONST_GIO IS INCREASED, BE SURE THAT THERE ARE ENOUGH
c     MESSAGE NUMBERS IN msg_numbers.h!
c     ----------------------------------------------------------------

      integer, parameter ::
     &  MAX_COL_DIAG_PRES  =   50,  ! maximum number of pressures for column
                                    ! diagnostic output
     &  MAX_COL_DIAG_SITES =  200,  ! maximum number of column diagnostic sites
     &  MAX_INFILE_NAMES   =  366,  ! maximum number of input file names
     &  MAX_NUM_CONST_GIO  =  185,  ! maximum number of const gen. I/O species
     &  MAX_NUM_QJ         =  200,  ! maximum number of qj (photolysis) rxns
     &  MAX_NUM_QK         =  500   ! maximum number of qk (thermal)    rxns


c     ------------------
c     Character commons.
c     ------------------

      character (len=128) ::
     &  imp_deckname,        ! name of namelist file
     &  imp_problem_name     ! problem name from namelist file

      character (len=80)  ::
     &  imp_data_dir         ! directory where input data files are located

      character (len=50)  ::
     &  metdata_name         ! met data netcdf file attribute "Met_Dat_Name",
                             ! e.g., "NCAR_MATCH_4x5x52"

      character (len=16)  ::
     &  metdata_name_org,    ! first  part of metdata_name, e.g., "NCAR"
     &  metdata_name_model,  ! second part of metdata_name, e.g., "MATCH"
     &  metdata_name_dims    ! third  part of metdata_name, e.g., "4x5x52"

      character (len=32)  ::
     &  hdf_dim_name,             ! NetCDF header        dimension  name
     &  lat_dim_name,             ! NetCDF latitude      dimension  name
     &  lon_dim_name,             ! NetCDF longitude     dimension  name
     &  prs_dim_name,             ! NetCDF pressure      dimension  name
     &  qj_dim_name,              ! NetCDF qj            dimension  name
     &  qk_dim_name,              ! NetCDF qk            dimension  name
     &  qqj_dim_name,             ! NetCDF qqj           dimension  name
     &  qqk_dim_name,             ! NetCDF qqk           dimension  name
     &  rec_dim_name,             ! NetCDF record        dimension  name
     &  sad_dim_name,             ! NetCDF sad           dimension  name
     &  spc_dim_name,             ! NetCDF species       dimension  name
     &  tim_dim_name              ! NetCDF time          dimension  name

      character (len=32)  ::
     &  const_var_name,           ! NetCDF const         variable   name
     &  dust_var_name,            ! NetCDF dust          variable   name
     &  emiss_var_name,           ! NetCDF emiss         variable   name
     &  flux_var_name,            ! NetCDF flux          variable   name
     &  hdr_var_name,             ! NetCDF header        variable   name
     &  qj_var_name,              ! NetCDF qj            variable   name
     &  qk_var_name,              ! NetCDF qk            variable   name
     &  qqj_var_name,             ! NetCDF qqj           variable   name
     &  qqk_var_name,             ! NetCDF qqk           variable   name
     &  rxnr_adjust_var_name,     ! NetCDF reaction rate adjustment
                                  !                      variable   name
     &  sad_var_name              ! NetCDF sad           variable   name

      character (len=128) ::
     &  aqua_infile_name,         ! aqua sulfur chem     input file name
     &  chem_infile_name,         ! sulfur chemistry     input file name
     &  const_infile_name,        ! constituent          input file name
     &  dust_source_infile_name,  ! dust source          input file name
     &  emiss_aero_infile_name,   ! aerosol (sulf. code) input file name
     &  emiss_dust_infile_name,   ! dust    (sulf. code) input file name
     &  emiss_infile_name,        ! emission             input file name
     &  fertscal_infile_name,     ! fertilizer scale     input file name
     &  fixed_const_infile_name,  ! fixed const          input file name
     &  forc_bc_infile_name,      ! forcing bc           input file name
     &  h2oclim_infile_name,      ! water climatology    input file name
     &  isopconv_infile_name,     ! isoprene convert     input file name
     &  lai_infile_name,          ! leaf area index      input file name
     &  lbssad_infile_name,       ! liq bin sul sad      input file name
     &  light_infile_name,        ! light                input file name
     &  loss_data_infile_name,    ! loss data            input file name
     &  monotconv_infile_name,    ! monoterpene convert  input file name
     &  precip_infile_name,       ! precipitation        input file name
     &  qj_infile_name,           ! qj                   input file name
     &  restart_infile_name,      ! restart              input file name
     &  rxnr_adjust_infile_name,  ! reaction rate adjustment
                                  !                      input file name
     &  sfalbedo_infile_name,     ! surface albedo       input file name
     &  soil_infile_name,         ! soil type            input file name
     &  uvalbedo_infile_name,     ! uv albedo            input file name
     &  veg_infile_name           ! vegetation type      input file name

c     -----------------------------------------------------------------
c     col_diag_site    : names of site locations for column diagnostics
c
c     const_labels     : constituent string labels
c
c     met_infile_names : met input file names
c
c     qj_labels        : qj (photolysis) string labels
c     qk_labels        : qk (thermal)    string labels
c     -----------------------------------------------------------------

      character (len=24)  ::
     &  col_diag_site(MAX_COL_DIAG_SITES)

      character (len=16)  ::
     &  const_labels (MAX_NUM_CONST_GIO)

      character (len=128) ::
     &  met_infile_names(MAX_INFILE_NAMES)

      character (len=80)  ::
     &  qj_labels(MAX_NUM_QJ), qk_labels(MAX_NUM_QK)

cc    character (len=1)   :: imp_gio_cdum1
      real*8        :: imp_gio_cdum1(4)
      character (len=1)   :: imp_gio_c1_array(SIZGIOBG2)

cc     =====================
c      common  / imp_gioc1 /
cc     =====================
c     &  imp_gio_cdum1,
c     &  imp_deckname,      imp_problem_name,
c     &  imp_data_dir,
c     &  metdata_name,
c     &  metdata_name_org,  metdata_name_model,
c     &  metdata_name_dims,
c     &  hdf_dim_name,      lat_dim_name,
c     &  lon_dim_name,      prs_dim_name,
c     &  qj_dim_name,       qk_dim_name,
c     &  qqj_dim_name,      qqk_dim_name,
c     &  rec_dim_name,      sad_dim_name,
c     &  spc_dim_name,      tim_dim_name,
c     &  const_var_name,    dust_var_name, 
c     &  emiss_var_name,
c     &  flux_var_name,     hdr_var_name,
c     &  qj_var_name,       qk_var_name,
c     &  qqj_var_name,      qqk_var_name,
c     &  rxnr_adjust_var_name,
c     &  sad_var_name,
c     &  aqua_infile_name,
c     &  chem_infile_name,
c     &  const_infile_name,
c     &  dust_source_infile_name,
c     &  emiss_aero_infile_name,
c     &  emiss_dust_infile_name,
c     &  emiss_infile_name,
c     &  fertscal_infile_name,
c     &  fixed_const_infile_name,
c     &  forc_bc_infile_name,
c     &  h2oclim_infile_name,
c     &  isopconv_infile_name,
c     &  lai_infile_name,
c     &  lbssad_infile_name,
c     &  light_infile_name,
c     &  loss_data_infile_name,
c     &  monotconv_infile_name,
c     &  precip_infile_name,
c     &  qj_infile_name,
c     &  restart_infile_name,
c     &  rxnr_adjust_infile_name,
c     &  sfalbedo_infile_name,
c     &  soil_infile_name,
c     &  uvalbedo_infile_name,
c     &  veg_infile_name,
c     &  col_diag_site,
c     &  const_labels,
c     &  qj_labels, qk_labels,
c     &  met_infile_names
c
cc     ===========
c      equivalence (imp_gio_c1_array, imp_gio_cdum1)
cc     ===========
c 

c     ----------------
c     Logical commons.
c     ----------------

      logical ::
     &  do_cycle_met,    ! cycle met data?
     &  do_flux_reset,   ! reset flux accumulators (i.e., flux output
                         ! occurred on previous time step)?
     &  do_mean,         ! should means or current values be put in periodic
                         ! output files?
     &  do_noon_rate,    ! should rates and rate constants be local noon
                         ! averages?
     &  do_overwrt_rst,  ! should the restart file be over-written?
     &  do_qqjk_inchem,  ! if pr_qqjk is on, should qqj's & qqk's be
                         ! determined inside chemistry solver?
     &  do_qqjk_reset    ! reset qqjk accumulators (i.e., qqjk output
                         ! occurred on previous time step)?

      logical :: 
     &  noon_metwater    ! should the meteorological water be put out at
                         ! local noon time?
      logical ::
     &  pr_ascii,        ! should ASCII output file be written at all?
     &  pr_ascii1,       ! should first  section of ASCII output file be
                         ! written (species mass data)?
     &  pr_ascii2,       ! should second section of ASCII output file be
                         ! written (species concentration data)?
     &  pr_ascii3,       ! should third  section of ASCII output file be
                         ! written (species concentration min/maxs)?
     &  pr_ascii4,       ! should fourth section of ASCII output file be
                         ! written (total mass of each species)?
     &  pr_ascii5        ! should fifth  section of ASCII output file be
                         ! written (total production and loss of each species)?

      logical ::
     &  pr_col_diag,     ! should the periodic column   diagnostics be written?
     &  pr_flux,         ! should the periodic flux     diagnostics be written?
     &  pr_tend          ! should the periodic tendency diagnostics be written?

      logical ::
     &  pr_netcdf,       ! should any periodic output files be written at all?
     &  pr_const,        ! should the periodic species concentrations file be
                         ! written?
     &  pr_psf,          ! should surface pressures also be written to periodic
                         ! const output file?
     &  pr_kel,          ! should temperatures      also be written to periodic
                         ! const output file?
     &  pr_mass,         ! should mass              also be written to periodic
                         ! const output file?
     &  pr_metwater,     ! should met water         also be written to periodic
                         ! const output file?
     &  pr_dry_depos,    ! should dry depositions   also be written to periodic
                         ! const output file?
     &  pr_wet_depos, pr_wet_depos_3type,   ! should wet depositions   also be written to periodic
                         ! const output file?
     &  pr_sulf_src,     ! should sulfur budgets    also be written to periodic
                         ! const output file?
     &  pr_surf_emiss    ! should surface emissions also be written to periodic
                         ! const output file?

      logical ::
     &  pr_nl            ! should all the namelist variables used be written to
                         ! an output file?

      logical ::
     &  pr_qj,           ! should the periodic qj   output file be written?
     &  pr_qj_o3_o1d,    ! should the o1d reaction be added to the end of 
                         ! the rate constants?
     &  pr_qj_opt_depth, ! should the optical depths be added to the end of
                         ! the rate constants?
     &  pr_qk,           ! should the periodic qk   output file be written?
     &  pr_qqjk,         ! should the periodic qqjk output file be written?
     &  pr_rad,          ! should the periodic rad  output file be written?
     &  pr_sad           ! should the periodic sad  output file be written?

      logical ::
     &  pr_restart,      ! should a restart file be written?
     &  pr_smv2,         ! should the SmvgearII output file be written
                         ! (non-parallel mode only)?
     &  pr_time          ! should the time be printed to the terminal screen
                         ! each time step?

      logical ::
     &  rd_restart       ! should a restart file be read?

      logical :: imp_gio_ldum1
      logical :: imp_gio_l1_array(SIZGIOSML)

cc     =====================
c      common  / impgio_l1 /
cc     =====================
c     &  imp_gio_ldum1,
c     &  do_cycle_met,   do_flux_reset,
c     &  do_mean,        do_noon_rate,
c     &  do_overwrt_rst,
c     &  do_qqjk_inchem, do_qqjk_reset,
c     &  noon_metwater,
c     &  pr_ascii,
c     &  pr_ascii1,      pr_ascii2,
c     &  pr_ascii3,      pr_ascii4,
c     &  pr_ascii5,
c     &  pr_col_diag,    pr_flux,
c     &  pr_tend,
c     &  pr_netcdf,
c     &  pr_const,
c     &  pr_psf,         pr_kel,
c     &  pr_mass,        pr_metwater,
c     &  pr_dry_depos,   pr_wet_depos, pr_wet_depos_3type, 
c     &  pr_sulf_src,    pr_surf_emiss,
c     &  pr_nl,
c     &  pr_qj,
c     &  pr_qj_o3_o1d,   pr_qj_opt_depth,
c     &  pr_qk,          pr_qqjk,
c     &  pr_rad,         pr_sad,
c     &  pr_restart,     pr_smv2,
c     &  pr_time,
c     &  rd_restart
c 
cc     ===========
c      equivalence (imp_gio_l1_array, imp_gio_ldum1)
cc     ===========
c 

c     ----------------
c     Integer commons.
c     ----------------

      integer ::
     &  ascii_out_i,      ! longitude index to use in ASCII     output file
     &  ascii_out_n       ! species   index to use in ASCII     output file

      integer ::
     &  asclun            ! logical unit number for   ASCII     output file

      integer ::
     &  col_diag_num,         ! number of different locations for column diag.
     &  col_diag_pres_num,    ! number of pressure levels     for column diag.
     &  col_diag_species_num  ! number of species             for column diag.

      integer ::
     &  emiss_conv_flag   ! emission conversion flag

      integer ::
     &  m1num_recs,       ! number of NetCDF met1 records in file
     &  m2num_recs        ! number of NetCDF met2 records in file

      integer ::
     &  m1rnum_in,        ! next NetCDF met1 record to read
     &  m2rnum_in         ! next NetCDF met2 record to read

      integer ::
     &  met_num_infiles   ! total number of met input files

      integer ::
     &  met1_infile_num,  ! index of current met1 input file
     &  met2_infile_num   ! index of current met2 input file

      integer ::
     &  ncid_met1,        ! NetCDF met1     input  file id
     &  ncid_met2         ! NetCDF met2     input  file id

      integer ::
     &  ncid_con,         ! NetCDF const    output file id
     &  ncid_flx,         ! NetCDF flux     output file id
     &  ncid_qj,          ! NetCDF qj       output file id
     &  ncid_qk,          ! NetCDF qk       output file id
     &  ncid_qqjk,        ! NetCDF qqjk     output file id
     &  ncid_rad,         ! NetCDF rad      output file id
     &  ncid_rst,         ! NetCDF restart  output file id
     &  ncid_sad,         ! NetCDF sad      output file id
     &  ncid_tnd          ! NetCDF tendency output file id

      integer ::
     &  noon_species_num

      integer ::
     &  pr_ascii_step_interval  ! interval for ASCII output

      integer ::
     &  restart_inrec     ! record number in restart input file to read from

      integer ::
     &  rnum_out,         ! next Netcdf output record to write
     &  rnum_out_col,     ! next column diag.  record to write
     &  rnum_out_qqjk,    ! next qqjk          record to write
     &  rnum_out_rad,     ! next rad           record to write
     &  rnum_out_rst      ! next restart       record to write

c     ---------------------------------------------------------------------
c     col_diag_species : should species be included in column diagnostics?
c
c     ncid_col         : NetCDF column diagnostic output files ids
c
c     const_outrec_map : mapping of const species output number to const
c                        species input number

c     noon_species     : array with non-zero entries indicating the species
c                        means should only be accumulated at approximately
c                        noon local time
c     ---------------------------------------------------------------------

      integer :: col_diag_species(MAX_NUM_CONST_GIO)

      integer :: ncid_col        (MAX_COL_DIAG_SITES)

      integer :: const_outrec_map(MAX_NUM_CONST_GIO)

      integer :: noon_species    (MAX_NUM_CONST_GIO)


      integer :: imp_gio_idum1
      integer :: imp_gio_i1_array(SIZGIOBIG)

cc     =====================
c      common  / impgio_i1 /
cc     =====================
c     &  imp_gio_idum1,
c     &  ascii_out_i,       ascii_out_n,
c     &  asclun,
c     &  col_diag_num,
c     &  col_diag_pres_num,
c     &  col_diag_species_num,
c     &  emiss_conv_flag,
c     &  m1num_recs,        m2num_recs,
c     &  m1rnum_in,         m2rnum_in,
c     &  met_num_infiles,
c     &  met1_infile_num,   met2_infile_num,
c     &  ncid_met1,         ncid_met2,
c     &  ncid_con,          ncid_flx,
c     &  ncid_qj,           ncid_qk,
c     &  ncid_qqjk,         ncid_rad,
c     &  ncid_rst,          ncid_sad,
c     &  ncid_tnd,
c     &  noon_species_num,
c     &  pr_ascii_step_interval,
c     &  restart_inrec,
c     &  rnum_out,
c     &  rnum_out_col,      rnum_out_qqjk,
c     &  rnum_out_rad,      rnum_out_rst,
c     &  col_diag_species,
c     &  ncid_col,
c     &  const_outrec_map,  noon_species
c
cc     ===========
c      equivalence (imp_gio_i1_array, imp_gio_idum1)
cc     ===========

 
c     -------------
c     Real commons.
c     -------------

      real*8  ::
     &  col_diag_period,   ! column diagnostics output period (s)
     &  emiss_conv_fac,    ! emission conversion factor (s^-1)
     &  emiss_delta_temp,  ! delta temperature used for emiss functions (deg k)
     &  mdt                ! time increment for reading new met data (s)

      real*8  ::
     &  ch4clim_init_val,  ! value to initialize ch4clim  array to
     &  emiss_init_val,    ! value to initialize emiss    array to (kg/s)
     &  forc_bc_init_val,  ! value to initialize forc_bc  array to (ppmv)
     &  h2oclim_init_val,  ! value to initialize h2oclim  array to
     &  lbssad_init_val,   ! value to initialize lbssad   array to (cm^-1)
     &  loss_init_val,     ! value to initialize loss     array to (s^-1)
     &  qj_init_val,       ! value to initialize qj       array to (s^-1)
     &  saldif_init_val,   ! value to initialize saldif   array to
     &  saldir_init_val,   ! value to initialize saldir   array to
     &  sasdif_init_val,   ! value to initialize sasdif   array to
     &  sasdir_init_val,   ! value to initialize sasdir   array to
     &  uvalbedo_init_val  ! value to initialize uvalbedo array to

      real*8  ::
     &  pr_nc_period,      ! NetCDF  output period
     &  pr_rad_period,     ! rad     output period
     &  pr_rst_period      ! restart output period

      real*8  ::
     &  tmet1_1,           ! time tag of first  met1 record (s)
     &  tmet1_2            ! time tag of second met1 record (s)

      real*8  ::
     &  tmet2_1,           ! time tag of first  met2 record (s)
     &  tmet2_2            ! time tag of second met2 record (s)

c     -------------------------------------------------------------------
c     col_diag_pres    : pressure levels for column diagnostics
c
c     const_init_val   : array of values to initialize each const species
c                        to (mixing ratio)
c
c     col_diag_lat_lon : lat/lon location of each column diagnostics site
c     -------------------------------------------------------------------

      real*8  :: col_diag_pres   (MAX_COL_DIAG_PRES)

      real*8  :: const_init_val  (MAX_NUM_CONST_GIO)

      real*8  :: col_diag_lat_lon(2, MAX_COL_DIAG_SITES)

      real*8  :: imp_gio_rdum1
      real*8  :: imp_gio_r1_array(SIZGIOBIG)

cc     =====================
c      common  / impgio_r1 /
cc     =====================
c     &  imp_gio_rdum1,
c     &  col_diag_period,
c     &  emiss_conv_fac,   emiss_delta_temp,
c     &  mdt,
c     &  ch4clim_init_val, emiss_init_val,
c     &  forc_bc_init_val, h2oclim_init_val,
c     &  lbssad_init_val,  loss_init_val,
c     &  qj_init_val,
c     &  saldif_init_val,  saldir_init_val,
c     &  sasdif_init_val,  sasdir_init_val,
c     &  uvalbedo_init_val,
c     &  pr_nc_period,     pr_rad_period,
c     &  pr_rst_period,
c     &  tmet1_1,          tmet1_2,
c     &  tmet2_1,          tmet2_2,
c     &  col_diag_pres,
c     &  const_init_val,
c     &  col_diag_lat_lon
c
cc     ===========
c      equivalence (imp_gio_r1_array, imp_gio_rdum1)
cc     ===========
 
      end module h_imp_gen_io