
c=============================================================================
c
c $Id: imp_time_utils.F,v 1.6 2003/04/02 17:43:31 jrt Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL
c   jrt@llnl.gov
c
c FILE
c   imp_time_utils.F
c
c ROUTINES
c   Imp_Tick
c   Get_Sec_Jan1
c   Get_Day_Jan1
c   Inc_Ymd
c   Is_Leap_Year
c   Convert_Nhms_Nsec
c   Convert_Nsec_Nhms
c   Convert_Nymd_Nsec
c   Split_Time_Flds
c
c=============================================================================

C change note by Cheng Zhou
c 1) change include to use
c 2) change call Do_Err_Out to    stop err_msg
c-----------------------------------------------------------------------------
c
c ROUTINE
c   Imp_Tick
c
c DESCRIPTION
c   This routine steps the Impact clock one time step.
c
c ARGUMENTS
c   num_time_steps : number of time steps
c   imp_sec : total Impact seconds (s)
c   nymd    : year/month/day (YYMMDD)
c   nhms    : hour/min/sec   (HHMMSS)
c   tdt     : model time step (s)
c
c-----------------------------------------------------------------------------

      subroutine Imp_Tick
     &  (num_time_steps, imp_sec, nymd, nhms, tdt)
      use h_imp_time_constants
      implicit none

c#     include "imp_time_constants.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: num_time_steps
      integer :: nymd
      integer :: nhms
      real*8  :: imp_sec
      real*8  :: tdt


c     ----------------------
c     Function declarations.
c     ----------------------

      integer, external :: Convert_Nhms_Nsec
      integer, external :: Convert_Nsec_Nhms
      integer, external :: Inc_Ymd


c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: nsec


c     ----------------
c     Begin execution.
c     ----------------
 
      num_time_steps = num_time_steps + 1
      imp_sec        = num_time_steps * tdt


      nsec = Convert_Nhms_Nsec (nhms) + Nint (tdt)
 

      if (nsec >= SECPDY) then
 
        nsec = nsec - SECPDY
        nymd = Inc_Ymd (nymd, 1)
 
      end if
 
 
      if (nsec < 00000) then
 
        nsec = SECPDY + nsec
        nymd = Inc_Ymd (nymd, -1)
 
      end if
 
 
      nhms = Convert_Nsec_Nhms (nsec)
 
 
      return
 
      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Get_Sec_Jan1
c
c DESCRIPTION
c   This routine returns the offset in seconds from Jan. 1st, given any
c   nymd/nhms.
c
c ARGUMENTS
c   nsec_jan1 : seconds from Jan. 1st (s)
c   nymd      : year/month/day (YYMMDD)
c   nhms      : hour/min/sec   (HHMMSS)
c
c-----------------------------------------------------------------------------

      subroutine Get_Sec_Jan1
     &  (nsec_jan1, nymd, nhms)
 
      implicit none


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: nsec_jan1
      integer :: nymd
      integer :: nhms

 
c     ----------------------
c     Function declarations.
c     ----------------------

      integer, external :: Convert_Nymd_Nsec
      integer, external :: Convert_Nhms_Nsec


c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: nsec_ymd
      integer :: nsec_hms


c     ----------------
c     Begin execution.
c     ----------------

      nsec_ymd = Convert_Nymd_Nsec (nymd)

      nsec_hms = Convert_Nhms_Nsec (nhms) 


      nsec_jan1 = nsec_ymd + nsec_hms


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Get_Day_Jan1
c
c DESCRIPTION
c   This routine returns the offset in days from Jan. 1st, given any
c   nymd/nhms.
c
c ARGUMENTS
c   nday_jan1 : days from Jan. 1st (days)
c   nymd      : year/month/day (YYMMDD)
c
c-----------------------------------------------------------------------------

      subroutine Get_Day_Jan1
     &  (nday_jan1, nymd)
      use h_imp_time_constants
      implicit none

c#     include "imp_time_constants.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: nday_jan1
      integer :: nymd

 
c     ----------------------
c     Function declarations.
c     ----------------------

      integer, external :: Convert_Nymd_Nsec


c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: nsec_ymd


c     ----------------
c     Begin execution.
c     ----------------

      nsec_ymd  = Convert_Nymd_Nsec (nymd)


      nday_jan1 = (nsec_ymd / SECPDY) + 1


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Inc_Ymd
c
c DESCRIPTION
c   This routine increments/decrements nymd by one day.
c
c ARGUMENTS
c   nymd  : year/month/day (YYMMDD)
c   dyinc : the day increment/decrement (1 or -1)
c
c-----------------------------------------------------------------------------

      function Inc_Ymd
     &  (nymd, dyinc)
      use h_imp_time_constants
      implicit none

c#     include "imp_time_constants.h"

 
c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: nymd
      integer :: dyinc


c     ----------------------
c     Function declarations.
c     ----------------------

      integer :: Inc_Ymd
 
      logical, external :: Is_Leap_Year


c     ----------------------
c     Variable declarations.
c     ----------------------

      character (len=75) :: err_msg

      logical :: is_ldy
      logical :: is_lyr

      integer :: ndy, nmon, nyr
 

c     ----------------
c     Begin execution.
c     ----------------
 
      if ((dyinc /= -1) .and. (dyinc /= 1)) then
        err_msg = 'dyinc range error in Inc_Ymd.'
c        call Do_Err_Out (err_msg, .true., 1, dyinc, 0, 0, 0.0d0, 0.0d0)
        stop 'dyinc range error in Inc_Ymd.'
      end if


      nyr  = nymd / 10000
      nmon = Mod (nymd, 10000) / 100
      ndy  = Mod (nymd, 100) + dyinc
 
 
      if (ndy == 0) then
 
c       -------------------------
c       Return to previous month.
c       -------------------------

        nmon = nmon - 1
 
        if (nmon == 0) then
          nmon = MONTHS_PER_YEAR
          if (nyr == 0) then
            nyr = 99
          else
            nyr = nyr - 1
          end if
        end if
 
        is_lyr = Is_Leap_Year (nyr)

        if (is_lyr) then
          ndy = DAYS_PER_MONTH_LY(nmon)
        else
          ndy = DAYS_PER_MONTH(nmon)
        end if

      else

        is_lyr = Is_Leap_Year (nyr)

        if (is_lyr .and. (nmon == 2) .and.
     &      (ndy == DAYS_PER_MONTH_LY(nmon))) then
          is_ldy = .true.
        else
          is_ldy = .false.
        end if

        if (.not. is_ldy) then

          if (ndy > DAYS_PER_MONTH(nmon)) then

c           ----------------------
c           Proceed to next month.
c           ----------------------
 
            ndy  = 1
            nmon = nmon + 1
 
            if (nmon > MONTHS_PER_YEAR) then
              nmon = 1
              if (nyr == 99) then
                nyr = 0
              else
                nyr = nyr + 1
              end if
            end if
 
          end if

        end if

      end if

 
      Inc_Ymd = (nyr * 10000) + (nmon * 100) + ndy
 
 
      return
 
      end
 

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Is_Leap_Year
c
c DESCRIPTION
c   This routine always returns false if leap_year_flag < 0,
c                always returns true  if leap_year_flag > 0,
c                and calculates whether or not it is really a leap year if
c                  leap_year_flag = 0.
c
c ARGUMENTS
c   the_year : the year to check (last 2 digits)
c
c-----------------------------------------------------------------------------

      function Is_Leap_Year (the_year)
 
      use  h_imp_time_constants
      use  h_imp_time_utils
      implicit none

c#     include "imp_time_constants.h"
c#     include "imp_time_utils.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: the_year

 
c     ----------------------
c     Function declarations.
c     ----------------------

      logical :: Is_Leap_Year


c     ----------------------
c     Variable declarations.
c     ----------------------

cc    integer, save :: the_year00 = 1900
      integer, save :: the_year00 = 2000


c     ----------------
c     Begin execution.
c     ----------------

      Is_Leap_Year = .false.


      if (leap_year_flag > 0) then

        Is_Leap_Year = .true.

      else if (leap_year_flag == 0) then

        if (Mod (the_year, 4) == 0) then

          if ((the_year /= 0) .or.
     &        (Mod (the_year00, 400) == 0)) then

            Is_Leap_Year = .true.

          end if

        end if

      end if
        

      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Convert_Nhms_Nsec
c
c DESCRIPTION
c   This routine converts nhms to total seconds.
c
c ARGUMENTS
c   nhms : hour/min/sec (HHMMSS)
c
c-----------------------------------------------------------------------------

      function Convert_Nhms_Nsec (nhms)
      use h_imp_time_constants
      implicit none

c#     include "imp_time_constants.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: nhms

 
c     ----------------------
c     Function declarations.
c     ----------------------

      integer :: Convert_Nhms_Nsec


c     ----------------------
c     Variable declarations.
c     ----------------------

      character (len=75) :: err_msg

      integer :: nsec


c     ----------------
c     Begin execution.
c     ----------------

      nsec = (nhms / 10000) * SECPHR +
     &       (Mod (nhms, 10000) / 100) * SECPMN +
     &       Mod (nhms, 100)


      if (nsec >= SECPDY) then

        err_msg = 'nsec too big in Convert_Nhms_Nsec.'
c        call Do_Err_Out (err_msg, .true., 1, nsec, 0, 0, 0.0d0, 0.0d0)
        stop 'nsec too big in Convert_Nhms_Nsec.'
      end if


      Convert_Nhms_Nsec = nsec


      return

      end
 

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Convert_Nsec_Nhms
c
c DESCRIPTION
c   This routine converts total seconds to hour/min/sec.
c
c ARGUMENTS
c   nsec : total seconds
c
c-----------------------------------------------------------------------------

      function Convert_Nsec_Nhms (nsec)
      use h_imp_time_constants
      implicit none

c#     include "imp_time_constants.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: nsec

 
c     ----------------------
c     Function declarations.
c     ----------------------

      integer :: Convert_Nsec_Nhms


c     ----------------------
c     Variable declarations.
c     ----------------------

      character (len=75) :: err_msg

      integer :: nhms


c     ----------------
c     Begin execution.
c     ----------------

      if (nsec >= SECPDY) then

        err_msg = 'nsec too big in Convert_Nsec_Nhms.'
c        call Do_Err_Out (err_msg, .true., 1, nsec, 0, 0, 0.0d0, 0.0d0)
        stop 'nsec too big in Convert_Nsec_Nhms.'
      end if


      nhms = (nsec / SECPHR) * 10000 +
     &       (Mod (nsec, SECPHR) / SECPMN) * 100 +
     &       Mod (nsec, SECPMN)


      Convert_Nsec_Nhms = nhms


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Convert_Nymd_Nsec
c
c DESCRIPTION
c   This routine converts nymd to total seconds.
c
c ARGUMENTS
c   nymd : year/month/day (YYMMDD)
c
c-----------------------------------------------------------------------------

      function Convert_Nymd_Nsec
     &  (nymd)
      use h_imp_time_constants
      implicit none

c#     include "imp_time_constants.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: nymd

 
c     ----------------------
c     Function declarations.
c     ----------------------

      integer :: Convert_Nymd_Nsec

      logical, external :: Is_Leap_Year


c     ----------------------
c     Variable declarations.
c     ----------------------

      character (len=75) :: err_msg

      logical :: is_lyr

      integer :: idays, isecs
      integer :: iyrsec
      integer :: ndy, nmon, nyr


c     ----------------
c     Begin execution.
c     ----------------

      nyr  = nymd / 10000
      nmon = Mod (nymd, 10000) / 100
      ndy  = Mod (nymd, 100)
 
 
      is_lyr = Is_Leap_Year (nyr)

      if (is_lyr) then
        idays  = (START_DAY_OF_MONTH_LY(nmon) - 1) + (ndy - 1)
        iyrsec = SECPYR_LY
      else
        idays  = (START_DAY_OF_MONTH(nmon)    - 1) + (ndy - 1)
        iyrsec = SECPYR
      end if
 
      isecs = idays * SECPDY
 

      if (isecs >= iyrsec) then
        err_msg = 'isecs too big in Convert_Nymd_Nsec.'
c        call Do_Err_Out (err_msg, .true., 1, isecs, 0, 0, 0.0d0, 0.0d0)
        stop  'isecs too big in Convert_Nymd_Nsec.'
      end if


      Convert_Nymd_Nsec = isecs


      return

      end
 

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Split_Time_Flds
c
c DESCRIPTION
c   This routine extracts the three fields from either nymd (year, month,
c   day) or nhms (hours, minutes, seconds).
c
c ARGUMENTS
c   nlmr    : nymd or nhms ("lmr" = "left" "middle" "right")
c             (YYMMDD or HHMMSS)
c   left2   : left   field (year  or hours)
c   middle2 : middle field (month or minutes)
c   right2  : right  field (day   or seconds)
c
c-----------------------------------------------------------------------------

      subroutine Split_Time_Flds
     &  (nlmr, left2, middle2, right2)
 
      implicit none

 
c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: nlmr
      integer :: left2
      integer :: middle2
      integer :: right2


c     ----------------
c     Begin execution.
c     ----------------
 
      left2   = nlmr / 10000

      middle2 = Mod (nlmr, 10000) / 100

      right2  = Mod (nlmr, 100)


      return

      end
 
