      module modbox_umaer
c-----------------------------------------------------------------------
c---->define accuracy
c-----------------------------------------------------------------------
      use precision
      implicit real(kdef) (a-h,o-z), integer (i-n)
c-----------------------------------------------------------------------
c---->parcel quantities unchanged during time integration
c
c     nvec          vector/loop length
c
c     h2osat        h2o saturation pressure               [#molec/m3]
c     h2ogas        h2o gas concentration                 [#molec/m3]
c     so4sat        h2so4 saturation pressure             [#molec/m3]
c     so4mfrac      so4 mass fraction in aerosol          [1]
c     so4frac       so4 mole fraction in aerosol          [1]
c     so4dens       aerosol density                       [kg/m3]
c     so4kelvin     factor for Kelvin effect              [m]
c     pathair       mean free path of air                 [m]
c     pathso4       mean free path of h2so4 in air        [m]
c     diffso4       diffusion coeff. for h2so4 in air     [m2/s]
c     viscos        dynamic viscosity of air              [Ns/m2]
c     so4crit       h2so4 conc with nuc' rate of 1/cm3    [#molec/m3]
c     so4sol        h2so4 sat pressure over solution      [#molec/m3]
c
c     pmsmerg       limit particle mass for merging       [#molec]
c     condnon       condensation coef's for non-so4 aer   [m3/#molec]
c                   incl. correction for collision geometry
c                   following Fuchs and Sutugin (1971)
c     xnonum        non-so4 aerosol number concentrations [#part/m3]
c     rwetnon       humid volume mean radius              [m]
c     xnonkelvin    Kelvin effect for non-sulfate aerosol [1]
c     diffnon       coefficient for coagulation
c                   for non-sulfate aerosol
c     so4sig        assumed geometric standard deviation  [1]
c-----------------------------------------------------------------------
      integer, parameter :: nvec=511

      real(kdef), save :: h2osat(nvec),h2ogas(nvec),so4sat(nvec),
     $                    so4mfrac(nvec),so4frac(nvec),
     $                    so4dens(nvec),so4kelvin(nvec),
     $                    pathair(nvec),pathso4(nvec),
     $                    diffso4(nvec),viscos(nvec),
     $                    so4crit(nvec),so4sol(nvec)

      real(kdef), allocatable, save :: pmsmerg(:,:),
     $                    condnon(:,:),xnonum(:,:),rwetnon(:,:),
     $                    xnonkelvin(:,:),diffnon(:,:)

      real(kdef), allocatable, save :: so4sig(:,:),xnonsig(:,:),
     $                    fracsrc(:,:),fracsrcnon(:,:),
     $                    so4alpha(:,:)

#ifdef DIAG
c-----------------------------------------------------------------------
c---->arrays for diagnostic quantities needed during time integration
c-----------------------------------------------------------------------
      real(kdef), pointer, save :: dxnucn(:),dxnucm(:),
     $                             dxcon (:),dxconn(:),
     $                             dxcoag(:),dxcoagn(:),
     $                             dxgrvn(:),dxgrvm(:)
      real(kdef), pointer, save :: fxnucn(:),fxnucm(:),
     $                             fxcon (:),fxconn(:),
     $                             fxcoag(:),fxcoagn(:),
     $                             fxgrvn(:),fxgrvm(:)

      real(kdef), target, save :: 
     $                    dynucn(nvec,2),dynucm(nvec,2),
     $                    dycon (nvec,2),dyconn(nvec,2),
     $                    dycoag(nvec,2),dycoagn(nvec,2),
     $                    dygrvn(nvec,2),dygrvm(nvec,2)
      real(kdef), target, save :: 
     $                    fynucn(nvec,3),fynucm(nvec,3),
     $                    fycon (nvec,3),fyconn(nvec,3),
     $                    fycoag(nvec,3),fycoagn(nvec,3),
     $                    fygrvn(nvec,3),fygrvm(nvec,3)

      real(kdef), save :: fsavenucn(nvec),fsavenucm(nvec),
     $                    fsavecon (nvec),fsaveconn(nvec),
     $                    fsavecoag(nvec),fsavecoagn(nvec),
     $                    fsavegrvn(nvec),fsavegrvm(nvec)

      real(kdef), save :: xnucn(nvec),xnucm(nvec),
     $                    xcon (nvec),xconn(nvec),
     $                    xcoag(nvec),xcoagn(nvec),
     $                    xgrvn(nvec),xgrvm(nvec),
     $                    xbeg(2,nvec)

#endif
      end module modbox_umaer
