      module modinit_umaer

      contains

      subroutine umaerinit(nso4_i,nmomso4_i,nnon_i,nmomnon_i,ng_i,
     $                     sigmod_i,radmerg_i,
     $                     radgsrc_i,siggsrc_i,frcmsrc_i,
     $                     radgso4_i,siggso4_i,fracso4_i,
     $                     radgnon_i,siggnon_i,fracnon_i,
     $                     rhonon_i,xmolnon_i,nrh_flg_i,
     $                     crh1_i,crh2_i,crh3_i,crh4_i)
c
c---->initialize umaerosol: input for number of modes and momemts
c                           info about background aerosol size distributions
c
c     nso4             number of sulfate aerosol modes
c     nmomso4          number of sulfate aerosol moments
c     nnon             number of non-sulfate aerosol modes
c     nmomnon          number of non-sulfate aerosol moments
c
c     ng               number of lognormal distr. in background aerosol
c
c---->assumed width and position of each mode
c     order: from smallest to largest mode
c
c     sigmod(nso4)     assumed geometric standard deviation  [1]
c     radmerg(nso4)    merging radius                        [m]
c
c---->sulfate aerosol in near SO2 source regions:
c
c     treated as direct particle emissions
c     radgsrc(nso4)    geometric radius                      [m]
c     siggsrc(nso4)    geometric standard deviation          [1]
c     frcmsrc(nso4)    mass fraction                         [1]
c
c---->background sulfate aerosol:
c
c     radgso4(ng)      geometric radius                      [m]
c     siggso4(ng)      geometric standard deviation          [1]
c     fracso4(ng)      relative fraction                     [1]
c
c---->background non-sulfate aerosol:
c
c     radgnon(ng,nnon) geometric radius                      [m]
c     siggnon(ng,nnon) geometric standard deviation          [1]
c     fracnon(ng,nnon) relative fraction                     [1]
c     rhonon(nnon)     aerosol density                       [kg/m3]
c     xmolnon(nnon)    molecular weight                      [kg/mol]
c     nrh_flg(nnon)    flag for humidity growth (1->sea salt)
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
c
c---->define accuracy and input arrays
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      integer ::
     $     nso4_i,nmomso4_i,nnon_i,nmomnon_i,ng_i
      real(kdef) ::
     $     sigmod_i(nso4_i),radmerg_i(nso4_i),
     $     radgsrc_i(nso4_i),siggsrc_i(nso4_i),frcmsrc_i(nso4_i),
     $     radgso4_i(ng_i),siggso4_i(ng_i),fracso4_i(ng_i),
     $     radgnon_i(ng_i,nnon_i),siggnon_i(ng_i,nnon_i),
     $     fracnon_i(ng_i,nnon_i),rhonon_i(nnon_i),xmolnon_i(nnon_i)
      integer :: nrh_flg_i(nnon_i)
      real(kdef) :: crh1_i(nnon_i),crh2_i(nnon_i),
     $              crh3_i(nnon_i),crh4_i(nnon_i)
c
c---->define local arrays and parameter
c
      parameter (r10=10.d0,r100=100.d0,
     $           r1h=0.5d0,r3h=1.5d0,
     $           r4td=4.d0/3.d0,r1td=1.d0/3.d0)
      parameter (r5=5.d0,r25=25.d0)
      parameter (r6=6.d0,r36=36.d0)
      dimension radvolm(ng_i), radsufm(ng_i)    ! Roy
c$$$c
c$$$c---->density of dry sulfate aerosol (pure sulfuric acid) [kg/m3]
c$$$c
c$$$      parameter (rhoso4=1861.d0)
c
c---->so4 density of dry ammonium sulfate aerosol [kg/m3]
c
      parameter (rhoso4=1313.d0)
c
c---->allocate and set data for module modpar_umaer
c
      nso4   =nso4_i
      nmomso4=nmomso4_i
      nnon   =nnon_i
      nmomnon=nmomnon_i
      ng     =ng_i

      naer=nmomso4*nso4
      ntot=nnon+naer+1

      allocate(radgsrc(1:nso4),siggsrc(1:nso4),frcmsrc(1:nso4))
      allocate(pmssrc(nso4))
      do iso4=1,nso4
       iinv=nso4-iso4+1
       radgsrc(iinv)=radgsrc_i(iso4)
       siggsrc(iinv)=siggsrc_i(iso4)
       frcmsrc(iinv)=frcmsrc_i(iso4)
       radius=radgsrc(iinv)*exp(r3h*log(siggsrc(iinv))**2)
       pmssrc(iinv)=r4td*pi*radius**3*rhoso4*avno/so4mol
      enddo

      allocate(radgso4(1:ng),siggso4(1:ng),fracso4(1:ng))
      radgso4(1:ng)=radgso4_i(1:ng)
      siggso4(1:ng)=siggso4_i(1:ng)
      fracso4(1:ng)=fracso4_i(1:ng)

      allocate (radgnon(1:ng,1:nnon),siggnon(1:ng,1:nnon),
     $          fracnon(1:ng,1:nnon),
     $          rhonon(1:nnon),xmolnon(1:nnon))
      radgnon(1:ng,1:nnon)=radgnon_i(1:ng,1:nnon)
      siggnon(1:ng,1:nnon)=siggnon_i(1:ng,1:nnon)
      fracnon(1:ng,1:nnon)=fracnon_i(1:ng,1:nnon)
      rhonon(1:nnon)=rhonon_i(1:nnon)
      xmolnon(1:nnon)=xmolnon_i(1:nnon)

      allocate (nrh_flg(1:nnon))
      nrh_flg(1:nnon)=nrh_flg_i(1:nnon)

      allocate (crh1(1:nnon),crh2(1:nnon),
     $          crh3(1:nnon),crh4(1:nnon))
      crh1(1:nnon)=crh1_i(1:nnon)
      crh2(1:nnon)=crh2_i(1:nnon)
      crh3(1:nnon)=crh3_i(1:nnon)
      crh4(1:nnon)=crh4_i(1:nnon)

c
c---->allocate memory for module modpar_umaer
c
      allocate(xscalemin(ntot))
      allocate(fracmerg(nmerg+1),radgmerg(nmerg+1,nso4),
     $         radmerg(nso4),pmsfac(nso4))
      allocate(fracaccu(0:naccu+1,nso4),radgaccu(0:naccu+1,nso4,2))
      allocate(fraccnon(0:naccu+1),radgaccnon(0:naccu+1,nnon))
      allocate(pmsnon(nnon),radvnon(nnon),radsnon(nnon))  ! Roy
      allocate(etaso4(nso4),
     $         etanon(nnon),alphanon(nnon))
c
c---->allocate memory for module modbox_umaer
c
      allocate(pmsmerg(nvec,nso4+1))
      allocate(condnon(nvec,nnon),xnonum(nvec,nnon),
     $         rwetnon(nvec,nnon),xnonkelvin(nvec,nnon),
     $         diffnon(nvec,nnon))

      allocate(so4sig(nvec,nso4),xnonsig(nvec,nnon))
      allocate(fracsrc(nvec,nso4),fracsrcnon(nvec,nnon))
      allocate(so4alpha(nvec,nso4))
c
c---->set maximum allowed error
c
      xscalemin(1:nnon)=1000.d0
      xscalemin(nnon+1:nnon+naer)=10.d0
c$$$      xscalemin(nnon+1:nnon+naer:nmomso4)=1000.d0
      do inon=nnon+1,nnon+naer,nmomso4
       xscalemin(inon)=1000.d0
      enddo
      xscalemin(nnon+(nso4-1)*nmomso4+2:ntot-1)=200.d0
      xscalemin(ntot)=10000.d0
c
c---->set accommodation coefficient
c
#ifdef LOWACCOEF
      etaso4(1:nso4)=0.04d0
      etanon(1:nnon)=0.04d0
#else
      etaso4(1:nso4)=1.d0
      etanon(1:nnon)=1.d0
c----------------------------------------------------------------
c      only change the fossil fuel BC/OC
c      modifed by Yuxing Yun 7/23/2010
c----------------------------------------------------------------
      etanon(2) = 0.018d0            ! fOC  according to Olga
      etanon(3) = 0.018d0            ! fBC  according to Olga
      etanon(6) = 0.018d0            ! OCphil1 (hydrophilic fOC)
      etanon(7) = 0.650d0            ! OCphil2 (hygroscopic fOC)
      etanon(8) = 0.018d0            ! BCphil1 (hydrophilic fBC)
      etanon(9) = 0.650d0            ! BCphil2 (hygroscopic fBC)

#endif
c
c---->calculate volume mean radii for background aerosol in [m]
c     and particle mass in [kg]
c
      radvolm(:)=radgso4(:)*exp(r3h*log(siggso4(:))**2)
      radvso4=sum(fracso4(:)*radvolm(:)**3)
      radvso4=radvso4**r1td

      do inon=1,nnon
       radvolm(:)=radgnon(:,inon)*exp(r3h*log(siggnon(:,inon))**2)
       radvnon(inon)=sum(fracnon(:,inon)*radvolm(:)**3)
       pmsnon(inon)=r4td*pi*rhonon(inon)*radvnon(inon)
       radvnon(inon)=radvnon(inon)**r1td
       xnonsig(1,inon)=sum(fracnon(:,inon)*log(siggnon(:,inon))**2)
     $                /sum(fracnon(:,inon))
       xnonsig(:,inon)=exp(sqrt(xnonsig(1,inon)))
cyun
       radsufm(:)=radgnon(:,inon)*exp(log(siggnon(:,inon))**2)
       radsnon(inon)=sum(fracnon(:,inon)*radsufm(:)**2)
       radsnon(inon)=radsnon(inon)**r1h
cyun
      enddo
c
c---->correction factor for condensation for non-sulfate aerosol 
c     for using the volume mean radius instead of the mean radius
c
      do inon=1,nnon
       alphanon(inon)=exp(-sum(fracnon(:,inon)*log(siggnon(:,inon))**2))
      enddo
c
c---->so4sig  - assumed geometric standard deviation [1]
c     radmerg - radius of merged particles [m]
c     pmsfac  - factor for mass of a merged particle [#molec m3/kg]
c
      do iso4=1,nso4
       iinv=nso4-iso4+1
       so4sig(:,iinv)=sigmod_i(iso4)
       radmerg(iinv)=radmerg_i(iso4)
       pmsfac(iinv)=r4td*pi*radmerg(iinv)**3/so4mol*avno
      enddo
c
c---->correction factor for condensation for sulfate aerosol 
c     for using the volume mean radius instead of the mean radius
c
      do iso4=1,nso4
       so4alpha(:,iso4)=exp(-log(so4sig(:,iso4))**2)
      enddo
c
c---->calculate integrals over size distribution for merging
c
      call mergeinit
c
c---->calculate integrals over size distribution for accumulation mode
c
      call accuminit

      end subroutine umaerinit
c-----------------------------------------------------------------------
      subroutine mergeinit
c
c---->calculate integrals over size distribution for merging
c     determine geometric mean radii for each bin 
c     for a given fractional change due to merging
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
c
c---->define accuracy
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
c
c---->define local arrays and parameter
c
      parameter (r5=5.d0,r10=10.d0,r1h=0.5d0,r3h=1.5d0)

      dimension func(3),radg(3)
c
c---->search radg for given value of integral over size distribution
c
      fracint=r1/real(nmerg,kdef)
      do iso4=2,nso4
       sigma=so4sig(1,iso4)
       do imerg=1,nmerg
        fracmerg(imerg)=fracint*(real(imerg,kdef)-r1h)

        radg(1)=radmerg(iso4)/(r5*sigma)
        radg(3)=radmerg(iso4)* r5*sigma
        func(1)=fracmerg_f(radg(1),sigma,radmerg(iso4))-fracmerg(imerg)
        func(3)=fracmerg_f(radg(3),sigma,radmerg(iso4))-fracmerg(imerg)

        do iter=1,20
         radg(2)=r1h*(radg(1)+radg(3))
         func(2)=fracmerg_f(radg(2),sigma,radmerg(iso4))-fracmerg(imerg)

         flag=r1h-sign(r1h,func(2)*func(3))
         flaginv=r1-flag

         radg(1)=radg(1)*flaginv+radg(2)*flag
         radg(3)=radg(2)*flaginv+radg(3)*flag
         func(1)=func(1)*flaginv+func(2)*flag
         func(3)=func(2)*flaginv+func(3)*flag
        enddo
c
c---->do linear interpolation
c
        radg(2)=radg(1)-(radg(3)-radg(1))
     $         *func(1)/(func(3)-func(1))
        radg(2)=max(radg(1),min(radg(3),radg(2)))

        radgmerg(imerg,iso4)=radg(2)
       enddo
      enddo
c
c---->set upper boundary
c
      fracmerg(nmerg+1)=r1
      radgmerg(nmerg+1,2:nso4)=radgmerg(nmerg  ,2:nso4)
     $                        *radgmerg(nmerg  ,2:nso4)
     $                        /radgmerg(nmerg-1,2:nso4)

      end subroutine mergeinit
c-----------------------------------------------------------------------
      function fracmerg_f(radg,sigma,radmrg)
c
c---->calculate integral over part of lognormal distribution
c
c
c---->load modules
c
      use precision
      use modpar_umaer
c
c---->define accuracy
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
c
c---->set local parameter
c
      parameter (r10=10.d0,r1h=0.5d0)
      parameter (dfrac=0.05d0)

      if (sigma<=r1) then
       if (radg>radmrg) then
        fracmerg_f=r1
       else
        fracmerg_f=r0
       endif
      else
       xlnsg=log(sigma)
       xln2sg=xlnsg*xlnsg
       xln2sginv=-r1h/xln2sg
       rscale=max(radg,radmrg)
       drad=dfrac*rscale*(r1-r1/sigma)
       sqr2pi=drad/(xlnsg*sqrt(r2*pi))

       fracmerg_f=r0
       radloc=radmrg+r1h*drad
       do while (radloc<r10*rscale)
        fracmerg_f=fracmerg_f
     $            +exp(xln2sginv*(log(radloc/radg))**2)/radloc
        radloc=radloc+drad
       enddo
       fracmerg_f=fracmerg_f*sqr2pi
      endif

      end function fracmerg_f
c-----------------------------------------------------------------------
      subroutine accuminit
c
c---->calculate integrals over size distribution for accumulation mode
c     determine geometric mean radii for each bin 
c     for a given fraction in the accumulation mode
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
c
c---->define accuracy
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
c
c---->define local arrays and parameter
c
      parameter (r5=5.d0,r10=10.d0,r1h=0.5d0,r3h=1.5d0)

      dimension func(3),radg(3)
      dimension fracmax(nso4)
c
c---->define accumulation mode as radius interval [m]
c     calculate limit radius (maximum overlap)    [m]
c
      radaccu(1:2)=(/0.05d-6,1.25d-6/)
      racculim=sqrt(radaccu(1)*radaccu(2))
c
c---->search radg for given value of integral over size distribution
c
      fracint=r1/real(naccu,kdef)
      do iso4=1,nso4
       sigma=so4sig(1,iso4)
       fracmax(iso4)=fracaccu_f(racculim,sigma)
       do iaccu=1,naccu
        fracaccu(iaccu,iso4)=fracint*(real(iaccu,kdef)-r1h)
        if (fracaccu(iaccu,iso4)>=fracmax(iso4)) then
         radgaccu(iaccu,iso4,1:2)=racculim
        else
         do iloc=1,2
          if (iloc==1) then
           radg(1)=radaccu(iloc)/(r5*sigma)
           radg(3)=racculim
          else
           radg(1)=racculim
           radg(3)=radaccu(iloc)* r5*sigma
          endif
    
          func(1)=fracaccu_f(radg(1),sigma)-fracaccu(iaccu,iso4)
          func(3)=fracaccu_f(radg(3),sigma)-fracaccu(iaccu,iso4)
          if (func(1)*func(3)>r0) then
           print*, 'ACCUMINIT: no zero found'
           stop
          endif
    
          do iter=1,20
           radg(2)=r1h*(radg(1)+radg(3))
           func(2)=fracaccu_f(radg(2),sigma)-fracaccu(iaccu,iso4)
    
           flag=r1h-sign(r1h,func(2)*func(3))
           flaginv=r1-flag
    
           radg(1)=radg(1)*flaginv+radg(2)*flag
           radg(3)=radg(2)*flaginv+radg(3)*flag
           func(1)=func(1)*flaginv+func(2)*flag
           func(3)=func(2)*flaginv+func(3)*flag
          enddo
c
c---->do linear interpolation
c
          radg(2)=radg(1)-(radg(3)-radg(1))
     $           *func(1)/(func(3)-func(1))
          radg(2)=max(radg(1),min(radg(3),radg(2)))

          radgaccu(iaccu,iso4,iloc)=radg(2)
         enddo
        endif
       enddo
      enddo
c
c---->set lower/upper boundary
c
      fracaccu(0,1:nso4)=r0
      radgaccu(0,1:nso4,1:2)=r0
      fracaccu(naccu+1,1:nso4)=fracmax(1:nso4)
      radgaccu(naccu+1,1:nso4,1:2)=racculim
#ifndef NONON
c
c---->for non-sulfate aerosol:
c     all particles larger than radaccu(1) are in accumulation mode
c
c---->search radg for given value of integral over size distribution
c
      fracint=r1/real(naccu,kdef)
      do inon=1,nnon
       sigma=xnonsig(1,inon)
       do iaccu=1,naccu
        fraccnon(iaccu)=fracint*(real(iaccu,kdef)-r1h)

        radg(1)=radaccu(1)/(r5*sigma)
        radg(3)=radaccu(1)* r5*sigma
        func(1)=fracmerg_f(radg(1),sigma,radaccu(1))-fraccnon(iaccu)
        func(3)=fracmerg_f(radg(3),sigma,radaccu(1))-fraccnon(iaccu)

        do iter=1,20
         radg(2)=r1h*(radg(1)+radg(3))
         func(2)=fracmerg_f(radg(2),sigma,radaccu(1))-fraccnon(iaccu)

         flag=r1h-sign(r1h,func(2)*func(3))
         flaginv=r1-flag

         radg(1)=radg(1)*flaginv+radg(2)*flag
         radg(3)=radg(2)*flaginv+radg(3)*flag
         func(1)=func(1)*flaginv+func(2)*flag
         func(3)=func(2)*flaginv+func(3)*flag
        enddo
c
c---->do linear interpolation
c
        radg(2)=radg(1)-(radg(3)-radg(1))
     $         *func(1)/(func(3)-func(1))
        radg(2)=max(radg(1),min(radg(3),radg(2)))

        radgaccnon(iaccu,inon)=radg(2)
       enddo
      enddo
c
c---->set lower/upper boundary
c
      fraccnon(0)=r0
      radgaccnon(0,1:nnon)=r0
      fraccnon(naccu+1)=r1
      radgaccnon(naccu+1,1:nnon)=radgaccnon(naccu  ,1:nnon)
     $                          *radgaccnon(naccu  ,1:nnon)
     $                          /radgaccnon(naccu-1,1:nnon)
#endif
      end subroutine accuminit
c-----------------------------------------------------------------------
      function fracaccu_f(radg,sigma)
c
c---->calculate integral over part of lognormal distribution
c     between radaccu(1) and radaccu(2)
c
c---->load modules
c
      use precision
      use modpar_umaer
c
c---->define accuracy
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
c
c---->set local parameter
c
      parameter (r10=10.d0,r1h=0.5d0)
      parameter (dfrac=0.05d0)

      if (sigma<=r1) then
       if (radg>radaccu(1) .and. radg<radaccu(2)) then
        fracaccu_f=r1
       else
        fracaccu_f=r0
       endif
      else
       xlnsg=log(sigma)
       xln2sg=xlnsg*xlnsg
       xln2sginv=-r1h/xln2sg
       drad=dfrac*radg*(r1-r1/sigma)
       sqr2pi=drad/(xlnsg*sqrt(r2*pi))

       fracaccu_f=r0
       radloc=radaccu(1)+r1h*drad
       do while (radloc<radaccu(2))
        fracaccu_f=fracaccu_f
     $            +exp(xln2sginv*(log(radloc/radg))**2)/radloc
        radloc=radloc+drad
       enddo
       fracaccu_f=fracaccu_f*sqr2pi
      endif

      end function fracaccu_f
c-----------------------------------------------------------------------
      end module modinit_umaer
