      module modpar_umaer
c-----------------------------------------------------------------------
c---->kdef             default accuracy, set in module precision
c-----------------------------------------------------------------------
      use precision
      implicit real(kdef) (a-h,o-z), integer (i-n)
c-----------------------------------------------------------------------
c---->problem size
c
c     nso4             number of sulfate aerosol modes
c     nmomso4          number of sulfate aerosol moments
c     nnon             number of non-sulfate aerosol modes
c     nmomnon          number of non-sulfate aerosol moments
c     ng               number of lognormal distr. in background aerosol
c     naer             number of modes and moments for sulfate aerosol
c     ntot             total number of prognostic variables
c-----------------------------------------------------------------------
      integer, save :: nso4,nmomso4,nnon,nmomnon,ng
      integer, save :: naer,ntot
c-----------------------------------------------------------------------
c---->r<n>             real numbers
c-----------------------------------------------------------------------
      real(kdef), parameter  :: r0=0.d0, r1=1.d0, r2=2.d0
c-----------------------------------------------------------------------
c---->time stepping
c
c     dtmin            minimum time step                [s]
c     dtmax            maximum time step                [s]
c     itermax          maximum number of iterations per period
c     xscalemin(ntot)  minimum allowed absolute error   [#/m3]
c-----------------------------------------------------------------------
      real(kdef), parameter :: dtmin=1.d-2,
     $                         dtmax=3600.d0

c      integer,    parameter :: itermax=300000
c
c  According Roy Chen, itermax is set to be 80000 to decrease the simulation time, 
c  This will not affect the results much. --Minghuai Wang (minghuai@umich.edu) 
      integer,    parameter :: itermax= 80000  

      real(kdef), allocatable, save ::xscalemin(:)
c-----------------------------------------------------------------------
c---->physical constants and numbers
c
c     pi               pi                           [1]
c     epsilon          epsilon                      [1]
c     avno             Avogadro number              [mol-1]
c     rgas             gas constant                 [J/K/mol]
c     boltz            Boltzmann constant           [J/K]
c     <spec>mol        molecular weights            [kg] 
c     so4min           mass of so4 molecule         [kg/molecule]
c     grav             gravitational constant       [m/s2]
c-----------------------------------------------------------------------
      real(kdef), parameter :: pi     =3.141592653589793d0,
     $                         epsilon=1.d-30,
     $                         avno   =6.02252d23,
     $                         rgas   =8.314d0,
     $                         boltz  =1.380485d-23

      real(kdef), parameter :: drymol =28.966d-3,
     $                         h2omol =18.017d-3,
     $                         so4mol =98.080d-3

      real(kdef), parameter :: grav=9.81d0

      real(kdef), parameter :: so4min=so4mol/avno
c-----------------------------------------------------------------------
c---->merging
c
c     nmerg            number of bins for table lookup
c                      of partial integral over lognormal distribution
c
c     fracmerg(nmerg+1)      fraction merged particles            [1]
c     radgmerg(nmerg+1,nso4) geometric radius that corresponds to fracmerg [m]
c     radmerg(nso4)          limit radius of merged particles     [m]
c     pmsfac(nso4)           factor for mass of a merged particle [#molec m3/kg]
c-----------------------------------------------------------------------
      integer, parameter            :: nmerg=20
      real(kdef), allocatable, save :: fracmerg(:),radgmerg(:,:),
     $                                 radmerg(:),pmsfac(:)
c-----------------------------------------------------------------------
c---->accumulation mode
c
c     naccu            number of bins for table lookup
c                      of partial integral over lognormal distribution
c
c     fracaccu(naccu+1)      fraction particles in accumulation mode  [1]
c     radgaccu(naccu+1,nso4) geometric radius that corresponds to fracaccu [m]
c     radaccu(2)             lower & upper radius of accu. mode       [m]
c     racculim               limit radius for which size distribution [m]
c                            is in the middle of accumulation mode
c-----------------------------------------------------------------------
      integer, parameter            :: naccu=20
      real(kdef), allocatable, save :: fracaccu(:,:),radgaccu(:,:,:),
     $                                 fraccnon(:),radgaccnon(:,:)
      real(kdef), save              :: radaccu(2),racculim
c-----------------------------------------------------------------------
c---->aerosol properties
c
c     radatkin           Atkin radius                   [m]
c     radmin             minimun radius of a nucleus    [m]
c
c     sulfate aerosol in near SO2 source regions:
c     treated as direct particle emissions
c     radgsrc(nso4)      geometric radius               [m]
c     siggsrc(nso4)      geometric standard deviation   [1]
c     frcmsrc(nso4)      mass fraction                  [1]
c     pmssrc(nso4)       sulfate particle mass          [#molec/particle]
c
c     background sulfate aerosol:
c     radgso4(ng)        geometric radius               [m]
c     siggso4(ng)        geometric standard deviation   [1]
c     fracso4(ng)        relative fraction              [1]
c     radvso4            dry volume mean radius         [m]
c
c     background non-sulfate aerosol:
c     radgnon(ng,nnon)   geometric radius               [m]
c     siggnon(ng,nnon)   geometric standard deviation   [1]
c     fracnon(ng,nnon)   relative fraction              [1]
c     rhonon(nnon)       aerosol density                [kg/m3]
c     xmolnon(nnon)      molecular weight               [kg/mol]
c     pmsnon(nnon)       particle mass                  [kg/particle]       
c     radvnon(nnon)      volume mean radius             [m]
c
c---->accommodation coefficient
c
c     etaso4(nso4)       for sulfate aerosol            [1]
c     etanon(nnon)       for non-sulfate aerosol        [1]
c
c---->correction factor for condensation
c     for using the volume mean radius instead of the mean radius
c    (alphaso4           for sulfate aerosol            [1])
c     alphanon(nnon)     for non-sulfate aerosol        [1]
c
c---->crh[1-4]           coefficient for humidity growth
c                        of sea salt aerosol
c-----------------------------------------------------------------------
      real(kdef),parameter :: radatkin=0.05d-6,
     $                        radmin=3.d-10

      real(kdef),save :: radvso4

      real(kdef), allocatable, save :: radgsrc(:),siggsrc(:),frcmsrc(:),
     $                    radgso4(:),siggso4(:),fracso4(:),
     $                    radgnon(:,:),siggnon(:,:),fracnon(:,:),
     $                    rhonon(:),xmolnon(:)
      integer, allocatable, save :: nrh_flg(:)

      real(kdef), allocatable, save :: pmssrc(:)
      real(kdef), allocatable, save :: pmsnon(:),radvnon(:),radsnon(:) !Roy
      real(kdef), allocatable, save :: etaso4(:),etanon(:)
      real(kdef), allocatable, save :: alphanon(:)
      
c     real(kdef), parameter :: crh1= 1.10413d0,
c    $                         crh2= 3.07900d0,
c    $                         crh3= 3.65124d-14,
c    $                         crh4=-1.42400d0

      real(kdef), allocatable, save :: crh1(:),
     $                                 crh2(:),
     $                                 crh3(:),
     $                                 crh4(:)

c$$$      real(kdef), parameter :: crh1= 2.07014d0,
c$$$     $                         crh2= 3.07900d0,
c$$$     $                         crh3= 5.18131d-17,
c$$$     $                         crh4=-1.42400d0

      end module modpar_umaer
