c=============================================================================
c
c $Id: qk_gas_rates.F,v 1.3 2003/06/30 22:50:11 dan Exp $
c
c CODE DEVELOPER
c   originated from GRANTOUR programmed by John Walton
c   modified by Xiaohong Liu
c
c FILE
c   qk_gas_rates.F
c
c ROUTINES
c   Do_QK_Gas_Rates
c
c=============================================================================

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Do_QK_Gas_Rates
c
c DESCRIPTION
c   This is the routine for the gas chemical reaction rates
c
c ARGUMENTS
c   itloop        : # of zones (ilong * ilat * ivert)
c   tgcm          : temperature (K)
c   zmair         : air density (#/cm3)
c   qh1p, qh2p, qh3p, qh5p, qh8p, qh9p 
c                 : gas reaction rate coefficients
c
c ---------------------------------------------------------------------------
   
      subroutine Do_QK_Gas_Rates
     &  (itloop, tgcm, zmair, 
     &   qh1p, qh2p, qh3p, qh5p, qh8p, qh9p)

      implicit none

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop

      real*8 ::  tgcm (itloop)
      real*8 ::  zmair(itloop)
      real*8 ::  qh1p (itloop)
      real*8 ::  qh2p (itloop)
      real*8 ::  qh3p (itloop)
      real*8 ::  qh5p (itloop)
      real*8 ::  qh8p (itloop)
      real*8 ::  qh9p (itloop)


c     ----------------
c     Begin execution.
c     ----------------

c.... reaction rate coefficients in the gas phase 
c     (qh1p, qh2p, qh3p, qh5p, qh8p, qh9p)

c          ======
      call chemk1
c          ======
     &  (tgcm, itloop, qh1p)

c          ======
      call chemk2
c          ======
     &  (tgcm, itloop, qh2p, zmair)

c          ======
      call chemk3
c          ======
     &  (tgcm, itloop, qh3p, zmair)

c          ======
      call chemk5
c          ======
     &  (tgcm, itloop, qh5p)

c          ======
      call chemk8
c          ======
     &  (tgcm, itloop, qh8p, zmair)

c          ======
      call chemk9
c          ======
     &  (tgcm, itloop, qh9p)


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   chemk1
c
c DESCRIPTION
c
c     *****************************************************************
c     * reaction rate for DMS + OH -> SO2
c     *****************************************************************
c
c     *     tpp  : air temperature (K)
c     *     qh1p : reaction rate coefficient (cm3 molecules-1 s-1)
c
c-----------------------------------------------------------------------------
c Change note by Cheng Zhou
c 1) from include to use statement
      subroutine chemk1
     &  (tpp, itloop, qh1p)
        use h_sulfchem,only: ijk
      implicit none

c#     include "sulfchem.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop

      real*8  :: tpp (itloop)
      real*8  :: qh1p(itloop)

c -------------------------------------------------------------
c  *** LOCAL  VARIABLES WERE DECLARED HERE.
c -------------------------------------------------------------

      real*8  c2, c3, c4, c5

      data c2 /8.46d-10/, c3 /2.68d-10/, c4 /88.1d0/, c5 /1.04d11/

      real*8 :: t1(itloop)
      real*8 :: t2(itloop)
      real*8 :: t3(itloop)
      real*8 :: t4(itloop)

      t1 = 0.0d0
      t2 = 0.0d0
      t3 = 0.0d0
      t4 = 0.0d0


c     *****************************************************************
c     * DMS + OH --> SO2 rate coef.
c     *****************************************************************

      do 100 ijk = 1, itloop

        t1(ijk) = exp(-234.d0 / tpp(ijk))
        t2(ijk) = exp(7230.d0 / tpp(ijk))
        t3(ijk) = exp(7810.d0 / tpp(ijk))
        t4(ijk) = exp(7460.d0 / tpp(ijk))

  100 continue

      do 200 ijk = 1, itloop

        qh1p(ijk) = (tpp(ijk) * t1(ijk) + c2 * t2(ijk) + c3 * t3(ijk))
     &            / (c4 * t4(ijk) + c5 * tpp(ijk))

  200 continue


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   chemk2
c
c DESCRIPTION
c
c
c     *****************************************************************
c     * reaction rate for SO2 + OH -> H2SO4
c     *****************************************************************
c
c     *     tpp  : air temperature (K)
c     *     zmp  : air density (molecules cm-3)
c     *     qh2p : reaction rate coefficient (cm3 molecules-1 s-1)
c
c     update reaction rate according to DeMore et al., 1997
c     qhzro300 = 3.0d-31 cm6 molecules-2 s-1
c     qhinf    = 1.5d-12 cm3 molecules-1 s-1
c     fc       = 0.6
c-----------------------------------------------------------------------------

      subroutine chemk2
     &  (tpp, itloop, qh2p, zmp)

        use h_sulfchem,only: ijk
      implicit none

c#     include "sulfchem.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop

      real*8  :: tpp (itloop)
      real*8  :: qh2p(itloop)
      real*8  :: zmp (itloop)

c -------------------------------------------------------------
c  *** LOCAL  VARIABLES WERE DECLARED HERE.
c -------------------------------------------------------------

      real*8
     &  qhzro300, qhinf, en, fc

c     data qhzro300 /3.16d-31/, qhinf /2.d-12/, en /3.3d0/, fc /0.45d0/
      data qhzro300 /3.0d-31/, qhinf /1.5d-12/, en /3.3d0/, fc /0.6d0/

      real*8 :: t1(itloop)
      real*8 :: t2(itloop)
      real*8 :: t3(itloop)
      real*8 :: t4(itloop)
      real*8 :: t5(itloop)

      t1 = 0.0d0
      t2 = 0.0d0
      t3 = 0.0d0
      t4 = 0.0d0
      t5 = 0.0d0

c     *****************************************************************
c     * SO2 + OH --> H2SO4 rate coef.
c     *****************************************************************

      do 100 ijk = 1, itloop

        t1(ijk) = qhzro300 * (300.d0 / tpp(ijk)) ** en

  100 continue

      do 200 ijk = 1, itloop

        t3(ijk) = t1(ijk) * zmp(ijk) / qhinf

  200 continue

      do 300 ijk = 1, itloop

        t4(ijk) = t1(ijk) * zmp(ijk) / (1.d0 + t3(ijk))
        t5(ijk) = 1.d0 / (1.d0 + log10(t3(ijk))**2)

  300 continue

      do 400 ijk = 1, itloop
 
        qh2p(ijk) = t4(ijk) * fc ** t5(ijk)

  400 continue


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   chemk3
c
c DESCRIPTION
c
c
c     *****************************************************************
c     * reaction rate for HO2 + HO2 -> H2O2 + O2
c     *****************************************************************
c     *
c     *     tpp  : air temperature (K)
c     *     zmp  : air density (molecules cm-3)
c     *     qh3p : reaction rate coefficient (cm3 molecules-1 s-1)
c
c-----------------------------------------------------------------------------

      subroutine chemk3
     &  (tpp, itloop, qh3p, zmp)

        use h_sulfchem,only: ijk
      implicit none

c#     include "sulfchem.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop

      real*8  :: tpp (itloop)
      real*8  :: qh3p(itloop)
      real*8  :: zmp (itloop)

c     *****************************************************************
c     * HO2 + HO2 --> H2O2 + O2
c     *****************************************************************

      do 100 ijk = 1, itloop

        qh3p(ijk) = 2.3d-13 * exp(600.d0 / tpp(ijk)) 
     &            + 1.7d-33 * zmp(ijk) * exp(1000.d0 / tpp(ijk))

  100 continue


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   chemk5
c
c DESCRIPTION
c
c     *****************************************************************
c     * reaction rate for H2O2 + OH -> H2O + HO2
c     *****************************************************************
c
c     *     tpp  : air temperature (K)
c     *     qh5p : reaction rate coefficient (cm3 molecules-1 s-1)
c
c-----------------------------------------------------------------------------

      subroutine chemk5
     &  (tpp, itloop, qh5p)

        use h_sulfchem,only: ijk
      implicit none

c#     include "sulfchem.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop

      real*8  :: tpp (itloop)
      real*8  :: qh5p(itloop)


c     *****************************************************************
c     * H2O2 + OH --> H2O + HO2
c     *****************************************************************

      do 100 ijk = 1, itloop

        qh5p(ijk) = 2.9d-12 * exp(-160.d0 / tpp(ijk))

  100 continue


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   chemk8
c
c DESCRIPTION
c
c
c     *****************************************************************
c     * reaction rate for HO2 + HO2 + H2O --> H2O2 + O2 + H2O
c     *****************************************************************
c
c     *     tpp : air temperature (K)
c     *     zmp : air density (molecules cm-3)
c     *     qh8p: reaction rate coefficient (cm6 molecules^-2 s-1)
c
c-----------------------------------------------------------------------------

      subroutine chemk8
     &  (tpp, itloop, qh8p, zmp)

        use h_sulfchem,only: ijk
      implicit none

c#     include "sulfchem.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop

      real*8  :: tpp (itloop)
      real*8  :: qh8p(itloop)
      real*8  :: zmp (itloop)

c     *****************************************************************
c     * HO2 + HO2 + H2O --> H2O2 + O2 + H2O
c     *****************************************************************

      do 100 ijk = 1, itloop

        qh8p(ijk) = 3.22d-34 * exp(2800.d0 / tpp(ijk))
     &            + 2.38d-54 * exp(3200.d0 / tpp(ijk)) * zmp(ijk)

  100 continue


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   chemk9
c
c DESCRIPTION
c
c     *****************************************************************
c     * reaction rate for DMS + NO3 -> SO2 + ...
c     *****************************************************************
c
c     *     qh9p: reaction rate coefficient (cm3 molecules-1 s-1)
c
c-----------------------------------------------------------------------------

      subroutine chemk9
     &  (tpp, itloop, qh9p)

      use h_sulfchem,only: ijk
      implicit none

c#     include "sulfchem.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop

      real*8  :: tpp (itloop)
      real*8  :: qh9p(itloop)


c     *****************************************************************
c     * DMS + NO3 --> SO2 + ....
c     *****************************************************************

      do 100 ijk = 1, itloop

c       qh9p(ijk) = 1.9d-13 * exp(520.d0 / tpp(ijk))
c       qh9p(ijk) = 1.9d-13 * exp(500.d0 / tpp(ijk))  ! update 10/2002

c     &################################################

c       Modification on 97/05/14 to turn off the NO3 reaction

        qh9p(ijk) = 0.d0

c     &################################################

  100 continue


      return

      end

