 
c=============================================================================
c
c $Id: solrpos.F,v 1.1.1.1 2001/06/07 17:44:10 jrt Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL (Original code from Keith Grant, LLNL)
c   jrt@llnl.gov
c
c FILE
c   solrpos.F
c
c ROUTINES
c   Solrpos
c
c=============================================================================
 

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Solrpos
c
c DESCRIPTION
c   Given the Julian day, this routine calculates the solar declination and
c   the square of the ratio of the mean Earth-Sun distance to the current
c   Earth-Sun distance.   Refer to:  Paltridge, G.W., and C.M.R. Platt, 1976:
c   "Radiative Processes in Meteorology and Climatology", Elsevier, pp. 57-63.
c
c ARGUMENTS
c   julday  : Julian day counting ranging from 0 on Jan. 1st to 364 on
c             Dec. 31st
c   decl    : solar declination (deg)
c   rdistsq : the square of the ratio of the mean Earth-Sun distance
c             to the current Earth-Sun distance
c
c-----------------------------------------------------------------------------

C change note by Cheng Zhou
c 1) change include to use

      subroutine Solrpos
     &  (julday, decl, rdistsq)
      use h_imp_phys_constants
      implicit none
 
c#     include "imp_phys_constants.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      real*8  :: julday
      real*8  :: decl
      real*8  :: rdistsq


c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter :: TWO_PI = 2.0d0 * IMP_PI


c     ----------------------
c     Variable declarations.
c     ----------------------

      real*8  :: theta           ! year angle on specified day
      real*8  :: theta2, theta3

 
c     ----------------
c     Begin execution.
c     ----------------

cc?   Leap year?
      theta  = (TWO_PI * julday) / 365.0d0

      theta2 = 2.0d0 * theta
      theta3 = 3.0d0 * theta
 

      decl =
     &  (360.0d0 / TWO_PI) *
     &  (0.006918d0 -
     &   0.399912d0 * Cos (theta)  + 0.070257d0 * Sin (theta)  -
     &   0.006758d0 * Cos (theta2) + 0.000907d0 * Sin (theta2) -
     &   0.002697d0 * Cos (theta3) + 0.001480d0 * Sin (theta3))
 

      rdistsq =
     &  1.000110d0  +
     &  0.034221d0  * Cos (theta)  + 0.001280d0 * Sin (theta)  +
     &  0.000719d0  * Cos (theta2) + 0.000077d0 * Sin (theta2)
 

      return

      end

