 
c=============================================================================
c
c $Id: solrza.F,v 1.1.1.1 2001/06/07 17:44:10 jrt Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL (Original code from Keith Grant, LLNL)
c   jrt@llnl.gov
c
c FILE
c   solrza.F
c
c ROUTINES
c   Solrza
c
c=============================================================================
 

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Solrza
c
c DESCRIPTION
c   Given a Greenwich time, a solar declination and reference latitude and
c   longitudes,  this routine returns a corresponding list of cosines of solar
c   zenith angles.  Refer to:  Paltridge, G.W., and C.M.R. Platt, 1976:
c   "Radiative Processes in Meteorology and Climatology", Elsevier, p. 62.
c
c ARGUMENTS
c   time   : Greenwich time since Jan 1, counting zero from midnight (days)
c   decl   : solar declination (deg)
c   lat    : latitude   (deg)
c   lon    : longitudes (deg)
c   nn     : number of longitudes for which to calculate cosines of the
c            solar zenith angle
c   cossza : cosines of the solar zenith angle (output)
c
c-----------------------------------------------------------------------------

C change note by Cheng Zhou
c 1) change include to use

      subroutine Solrza
     &  (time, decl, lat, lon, nn, cossza)
      use h_imp_phys_constants
      implicit none
 
c#     include "imp_phys_constants.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: nn
      real*8  :: time 
      real*8  :: decl 
      real*8  :: lat 
      real*8  :: lon   (nn)
      real*8  :: cossza(nn)


c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8,  parameter :: TWO_PI = 2.0d0 * IMP_PI
      real*8,  parameter :: PI_180 = TWO_PI / 360.0d0


c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: ii

      real*8  :: cosha


c     ----------------
c     Begin execution.
c     ----------------

      do ii = 1, nn

c       -------------------------------------------------------------
c       Calculate the cosine of the hour angle which is referenced to
c       noon.
c       -------------------------------------------------------------

        cosha =
     &    Cos (TWO_PI *
     &         (Mod (time + (lon(ii) / 360.d0), 1.0d0) - 0.5d0))
 

c       -----------------------------------------------
c       Calculate the cosine of the solar zenith angle.
c       -----------------------------------------------

        cossza(ii) =
     &    Sin (PI_180 * lat) * Sin (PI_180 * decl) +
     &    Cos (PI_180 * lat) * Cos (PI_180 * decl) * cosha

      end do
 

      return

      end

