c=============================================================================
c
c $Id: sulfchem_solver.F,v 1.3 2003/06/30 22:50:11 dan Exp $
c
c CODE DEVELOPER
c   Xiaohong Liu
c
c FILE
c   sulfchem_solver.F
c
c ROUTINES
c   Do_Sulf_Solver
c
c=============================================================================
 
c-----------------------------------------------------------------------------
c
c ROUTINE
c   Do_Sulf_Solver
c
c DESCRIPTION
c   This is the main control routine for the chemistry solver, "Sulfchem".
c
c ARGUMENTS
c   time       : chemistry time step (s)
c   nymd       : current  year/month/day (YYMMDD)
c   tempc_nb   : temperature (K)
c   presc_nb   : atmospheric pressure at the center of each grid box (mb)
c   ispb_nb    : the flag for boundary layer
c   const_nb   : species concentration, known at zone centers 
c                (molecules/cm^3 for gas-species and g/g for aerosols)
c   humidity   : specific humidity (g/kg)
c   sulf_emiss : array of sulfur emissions (molecules/cm^3/s)
c   qjimp      : photolysis rate constants (s^-1)
c   qkimp      : thermal    rate constants (units vary)
c   clfrac     : total cloud fraction [0 - 1]
c   lwccol     : in-cloud liquid water in each grid box (g/g)
c   relhum     : relative humidity [0 - 1]
c
c-----------------------------------------------------------------------------

c change note by Cheng Zhou
c 1) change incuded head file to use statements
c 2) do not to interpolation for qjimp, do it somewhere else, maybe in the h_init

c 3) for CAM make dimension and subscript of londeg same as latdeg, since now we are dealing with pcols
c 4) added where statements, ensure hox_qj_scaler >0.
      subroutine Do_Sulf_Solver
     &  (time, nymd, nhms, tempc_nb, presc_nb, ispb_nb, const_nb, 
     &   humidity, sulf_emiss, qjimp, qkimp, clfrac, lwccol, relhum,
     &   aqua_infile_name, num_time_steps, londeg, latdeg, mass,
     &   pr_sulf_src, dms_oh, dms_no3, so2_oh, so2_h2o2, so2_o3,
     &   grid_height,ncol,lchnk)
     
      use h_imp_dims, only: i1_gl, i2_gl,ju1_gl,j2_gl, 
     &k1,k2,i1,i2, ju1,j2,itloop,
     &num_qjs,num_qks,num_species,
     &pr_diag,loc_proc
     
      use h_setkin_par
      
c      use h_sulfchem
 !----     
      use cam_history,     only: outfld
 !----        
      implicit none

c#     include "imp_dims.h"
c#     include "setkin_par.h"
c#     include "sulfchem.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: nymd
      integer :: nhms
      integer :: num_time_steps

      character*(*) :: aqua_infile_name

      real*8  :: time
      real*8  :: tempc_nb  (i1:i2, ju1:j2, k1:k2)
      real*8  :: presc_nb  (i1:i2, ju1:j2, k1:k2)
      integer :: ispb_nb   (i1:i2, ju1:j2, k1:k2)      
      real*8  :: const_nb  (i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: humidity  (i1:i2, ju1:j2, k1:k2)
      real*8  :: sulf_emiss(i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: qjimp     (i1:i2, ju1:j2, k1:k2, num_qjs)
      real*8  :: qkimp     (i1:i2, ju1:j2, k1:k2, num_qks)
      real*8  :: clfrac    (i1:i2, ju1:j2, k1:k2)
      real*8  :: lwccol    (i1:i2, ju1:j2, k1:k2)
      real*8  :: relhum    (i1:i2, ju1:j2, k1:k2)
      real*8  :: londeg    (ju1_gl:j2_gl) !just pcols
      real*8  :: latdeg    (ju1_gl:j2_gl)
      real*8  :: mass      (i1:i2, ju1:j2, k1:k2)
      logical :: pr_sulf_src
      real*8  :: dms_oh    (i1:i2, ju1:j2, k1:k2)
      real*8  :: dms_no3   (i1:i2, ju1:j2, k1:k2)
      real*8  :: so2_oh    (i1:i2, ju1:j2, k1:k2)
      real*8  :: so2_h2o2  (i1:i2, ju1:j2, k1:k2)
      real*8  :: so2_o3    (i1:i2, ju1:j2, k1:k2)
      real*8  :: grid_height(i1:i2, ju1:j2, k1:k2)
      integer :: ncol,lchnk   

c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: il, ij, ik
      integer :: idumday, idumyear
      integer :: im

      real*8 :: oh_saved (i1:i2, ju1:j2, k1:k2)
      real*8 :: ho2_saved(i1:i2, ju1:j2, k1:k2)
      real*8 :: no3_saved(i1:i2, ju1:j2, k1:k2)

      real*8  :: hox_qj_scaler(i1:i2, ju1:j2)
      real*8  :: no3_scaler   (i1:i2, ju1:j2)


c     ----------------
c     Begin execution.
c     ----------------
 
      if (pr_diag) then
        Write (6,*) 'Do_Sulf_Solver called by ', loc_proc
      end if
 

      hox_qj_scaler = 0.0d0
      no3_scaler    = 0.0d0


c     ====================
      call Split_Time_Flds
c     ====================
     &  (nymd, idumyear, im, idumday)

c     Cheng Zhou, do not do it here,do it somewhere else.
cc     Interpolation of qj (91x5) to the IMPACT pressure levels.
c
c      do ik = k1, k2
c        do ij = ju1, j2
c          do il = i1, i2
c
c            k = Int (2.0d0 
c     &        + (presc_nb(il,ij,ik) * 1.0d-3 - 0.1d0) / 0.3d0)
c            k  = Min (k, 4)
c
c            qjimp(il,ij,ik,1) = 
c     &               qjh2o2_2d(ij,k,im)
c     &               + (qjh2o2_2d(ij,k+1,im) - qjh2o2_2d(ij,k,im))
c     &               * (presc_nb(il,ij,ik) * 1.0d-3 - ppr(k))
c     &               / (ppr(k+1) - ppr(k))
c
c          end do
c        end do
c      end do


c     Calculate the scaling factor for OH, HO2, NO3, Qj diurnal variations
 
c     =====================
      call Scale_HOx_NO3_Qj
c     =====================
     &  (nymd, nhms, londeg, latdeg, hox_qj_scaler, no3_scaler) 

      oh_saved  = const_nb(i1:i2,ju1:j2,k1:k2,IOH)
      ho2_saved = const_nb(i1:i2,ju1:j2,k1:k2,IHO2)
      no3_saved = const_nb(i1:i2,ju1:j2,k1:k2,INO3)

      do ik = k1, k2
        do ij = ju1, j2
          do il = i1, i2

            const_nb(il,ij,ik,IOH) =
     &        const_nb(il,ij,ik,IOH) * hox_qj_scaler(il,ij)
            const_nb(il,ij,ik,IHO2) = 
     &        const_nb(il,ij,ik,IHO2) * hox_qj_scaler(il,ij)

!            qjimp(il,ij,ik,1) =
!     &        qjimp(il,ij,ik,1) * hox_qj_scaler(il,ij)  ! do not apply this just for now. need to apply late,need to rename 
!when qjimp read in is ready still need this factor, now just set qjimp to the value at the time.     
     
c            print*,'hox_qj_scaler(il,ij)',hox_qj_scaler(il,ij)
c            print*,'no3_scaler(il,ij)',no3_scaler(il,ij)
            
            const_nb(il,ij,ik,INO3) =
     &        const_nb(il,ij,ik,INO3) * no3_scaler(il,ij)

          end do
        end do
      end do
      
       call outfld("um_hox_scaler",hox_qj_scaler(1,:ncol),ncol,lchnk)
       call outfld("um_nox_scaler",no3_scaler(1,:ncol),ncol,lchnk)    

c     ================
      call Do_Sulfchem
c     ================
     &  (itloop, time, tempc_nb, presc_nb, ispb_nb, const_nb, humidity,
     &   sulf_emiss, qjimp, qkimp, clfrac, lwccol, relhum,
     &   aqua_infile_name, pr_diag, loc_proc, num_time_steps,
     &   mass, pr_sulf_src, 
     &   dms_oh, dms_no3, so2_oh, so2_h2o2, so2_o3, grid_height)

      const_nb(i1:i2,ju1:j2,k1:k2,IOH)  = oh_saved
      const_nb(i1:i2,ju1:j2,k1:k2,IHO2) = ho2_saved
      const_nb(i1:i2,ju1:j2,k1:k2,INO3) = no3_saved

      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Scale_HOx_NO3_Qj
c
c CODE DEVELOPER
c   Xiaohong Liu
c
c DESCRIPTION
c   This routine computes the scaling factor for HOx-Qj based on
c
c   OH(instantaneous) = OH(max) * cos(theta)
c
c   where cos(theta) is cosine of solar zenith angle.
c
c   thus, OH(monthly_ave) * 24 (hr) = OH(max) * Sum (cos(theta) * dt)
c                                     ( t (hr) = triz -> tset)
c
c   OH(max) = OH(monthly_ave) * 24 (hr) / (Sum (cos(theta) * dt) )
c
c   so, hox_qj_scaler = 24 / (Sum (cos(theta) * dt) ) * cos(theta)
c
c   and, computes the scaling factor for NO3 assuming NO3 even distributed
c        over night.
c   so, no3_scaler = 24 / (24-(tset-triz))
c
c-----------------------------------------------------------------------------

      subroutine Scale_HOx_NO3_Qj
     &  (nymd, nhms, londeg, latdeg, hox_qj_scaler, no3_scaler)

      use h_imp_time_constants
      use h_imp_dims, only:i1_gl,i2_gl,ju1_gl,j2_gl,i1,i2,ju1,j2,
     & ilong

      implicit none

c#     include "imp_time_constants.h"
c#     include "imp_dims.h"

c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: nymd
      integer :: nhms

      real*8  :: londeg (ju1_gl:j2_gl)
      real*8  :: latdeg (ju1_gl:j2_gl)
      real*8  :: hox_qj_scaler(i1:i2, ju1:j2)
      real*8  :: no3_scaler   (i1:i2, ju1:j2)

c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: il, ij, it
      integer :: idumyear, idummin, idumsec
      integer :: im, id, ih
      integer :: nday_jan1
      integer :: nsec_jan1
      integer :: nt

      real*8 :: tloc
      real*8 :: stime_loc
      real*8 :: days
      real*8 :: decl     ! solar declination
      real*8 :: rdistsq  ! solar distance correction
      real*8 :: time
      real*8 :: triz(ju1:j2)
      real*8 :: tset(ju1:j2)
      real*8 :: tot_cossza(ju1:j2)
      real*8, save :: lon_dummy(1) = 0.0d0
      real*8 :: cossza(1)
      real*8 :: csza(i1:i2)

c     ----------------
c     Begin execution.
c     ----------------

      csza(:) = 0.0d0

c     ====================
      call Split_Time_Flds
c     ====================
     &  (nymd, idumyear, im, id)

c     ====================
      call Split_Time_Flds
c     ====================
     &  (nhms, ih, idummin, idumsec)

c     =================
      call Get_Day_Jan1
c     =================
     &  (nday_jan1, nymd)


c     Calculate the sun rize and sun set time (local time: hr)

      do ij = ju1, j2

c       ==============
        call SunRizSet
c       ==============
     &    (im, id, latdeg(ij), triz(ij), tset(ij))

      end do


      nt = 100       ! divide one-day (24 hr) into 100 time intervals
      tot_cossza(:) = 0.0d0

c     =============
      do it = 0, nt
c     =============

        tloc = it * 24.0d0/nt      ! hr
        stime_loc = nday_jan1 * SECPDY + tloc * SECPHR

        days = stime_loc / SECPDY

c       ============
        call Solrpos
c       ============
     &    (days, decl, rdistsq)


c       ===============
        do ij = ju1, j2
c       ===============

c         ---------------------------------------------
c         Calculate solar zenith angle at this latitude
c         ---------------------------------------------

c         ===========
          call Solrza
c         ===========
     &      (days, decl, latdeg(ij), lon_dummy, 1, cossza)


          if(tloc > triz(ij) .and. tloc < tset(ij)) then

            tot_cossza(ij) = tot_cossza(ij) +
     &                       cossza(1) * 24.0d0/nt

          end if

c       ======
        end do
c       ======

c     ======
      end do
c     ======


c     =================
      call Get_Sec_Jan1
c     =================
     &  (nsec_jan1, nymd, nhms)

      time = nsec_jan1
      days = time / SECPDY

c     ============
      call Solrpos
c     ============
     &  (days, decl, rdistsq)


c     ===============
      do ij = ju1, j2
c     ===============

c       -----------------------------------------------------
c       Calculate solar zenith angle at this latitude for all
c       longitudes.
c       -----------------------------------------------------
c       ===========
        call Solrza
c       ===========
     &    (days, decl, latdeg(ij), londeg(ij), ilong, csza)


        do il = i1, i2

          tloc = ih + londeg(ij) / 360.d0 * 24.0d0  ! change the index of londeg same as latdeg
          if(tloc > 24.0d0) tloc = tloc - 24.0d0

          if(tloc > triz(ij) .and. tloc < tset(ij)) then
            hox_qj_scaler(il,ij) = 24.0d0 /
     &                             tot_cossza(ij) * csza(il) 
          else
            hox_qj_scaler(il,ij) = 1.0d-5
          end if

c added by Cheng , ensure that no negative points
           WHERE(hox_qj_scaler(:,:) < 1.0d-5) hox_qj_scaler(:,:) = 1.0d-5
c added by Cheng , ensure that no wild points, sometimes point at the boundary could be have really large values. 
           WHERE(hox_qj_scaler(:,:) > 20.0  ) hox_qj_scaler(:,:) = 1.0d-5
           
          if(tloc < triz(ij) .or. tloc > tset(ij)) then
            no3_scaler(il,ij) = 24.0d0 /
     &                          (24.0d0 - (tset(ij) - triz(ij)))
          else
            no3_scaler(il,ij) = 1.0d-5
          end if
          
c added by Cheng , ensure that no negative points
           WHERE(no3_scaler(:,:) < 1.0d-5) no3_scaler(:,:) = 1.0d-5
        end do

c     ======
      end do
c     ======
c      print*, 'nhms=',nhms,'tloc=',tloc,triz,tset
c      print*, 'csza=',csza
c      print*,'tot_cossza=',tot_cossza

      return

      end


CLL Subroutine: SunRizSet.f--------------------------------------------
CLL Purpose: Compute the local time of sunrise and sunset given
CLL          latitude and date.
CLL
CLL Author: Wenwei Pan
CLL         Center for Meteorology and Oceanography
CLL         Center for Global Change Science
CLL         Department of Earth, Atmospheric and Planetary Sciences
CLL
CLL Version: 1.0
CLL Date: April 3, 1997
CLL
CLL Tested under compiler: f77 V4.0
CLL Tested under OS version: SunOS 5.5.1 Generic sun4u sparc SUNW,Ultra-1
CLL
CLL Code modification history:
CLL Version Date
CLL---------------------------------------------------------------------

      subroutine SunRizSet(month,iday,rlat,triz,tset)

      implicit none

C Input variables.
      integer
     &     month,
     &     iday
      real*8 ::
     &     rlat           !Latitude in degrees.

C Output variables.
      real*8 :: triz,tset      !local time of sunrise and sunset(hr)

C Local variables.
      real*8 ::
     &     fac,
     &     Pi,
     &     x,
     &     xlat,          !Latitude in radians.
     &     phi,cosphi,sinphi,
     &     tphi,costph,sintph,
     &     ttphi,costtp,sinttp,
     &     delta,         !Solar inclination angle in radians.
     &     h              !Hour angle at sunrise and sunset in radians.

C*----------------------------------------------------------------------

      FAC = .01745d0
      PI = 3.1415926d0
      X = FLOAT((MONTH-1)*30+IDAY+MONTH/2)
      XLAT = RLAT*FAC
      PHI = 2.0d0*3.14159d0*X/365.0d0
      TPHI = PHI*2.0d0
      TTPHI = PHI*3.0d0
      COSPHI = COS(PHI)
      SINPHI = SIN(PHI)
      COSTPH = COS(TPHI)
      SINTPH = SIN(TPHI)
      COSTTP = COS(TTPHI)
      SINTTP = SIN(TTPHI)
      DELTA = .006918d0 - 0.399912d0*COSPHI + 0.070257d0*SINPHI -
     +  0.006758d0*COSTPH + 0.000907d0*SINTPH - 0.002697d0*COSTTP +
     +  0.00148d0*SINTTP
      IF(DELTA.GE.0.0d0)THEN                   !SUMMER
         IF(XLAT.GE.(PI/2.0d0-DELTA))THEN      !NO SUNSET
            H=PI
         ELSEIF(XLAT.LE.(DELTA-PI/2.0d0))THEN  !NO SUNRISE
            H=0.0d0
         ELSE
            H=ACOS(-TAN(XLAT)*TAN(DELTA))
         ENDIF
      ELSE
         IF(XLAT.GE.(PI/2.0d0+DELTA))THEN      !NO SUNRISE
            H=0.0d0
         ELSEIF(XLAT.LE.(-DELTA-PI/2.0d0))THEN !NO SUNSET
            H=PI
         ELSE
            H=ACOS(-TAN(XLAT)*TAN(DELTA))
         ENDIF
      ENDIF
      Triz=12.0d0*(-H+PI)/PI
      Tset=12.0d0*(H+PI)/PI

      return

      end

